@@ -29,18 +29,18 @@ const gchar *audio_name ();
 const gchar *audio_version ();
 
 static inline gint16
-audio_mix_channels (gint16 ** src, guint channels)
+audio_mix_channels (gint16 **src, guint channels)
 {
   gdouble mix = 0;
   for (gint i = 0; i < channels; i++, (*src)++)
     {
       mix += **src;
     }
-  return (gint16) (mix * MULTICHANNEL_MIX_GAIN (channels));
+  return (gint16) (mix * MONO_MIX_GAIN (channels));
 }
 
 static inline void
-audio_copy_sample (gint16 * dst, gint16 * src, struct audio *audio)
+audio_copy_sample (gint16 *dst, gint16 *src, struct audio *audio)
 {
 #if defined(ELEKTROID_RTAUDIO)
   *dst = (gint16) (*src * audio->volume);
@@ -52,19 +52,44 @@ audio_copy_sample (gint16 * dst, gint16 * src, struct audio *audio)
 void
 audio_write_to_output (struct audio *audio, void *buffer, gint frames)
 {
+  guint32 start, end;
+  size_t size;
   gint16 *dst, *src;
-  guint32 len = audio->sel_len ? audio->sel_start + audio->sel_len :
-    audio->sample_info.frames;
-  guint bytes_per_frame = SAMPLE_INFO_FRAME_SIZE (&audio->sample_info);
-  size_t size = frames * FRAME_SIZE (AUDIO_CHANNELS, SF_FORMAT_PCM_16);
+  guint bytes_per_frame;
+  gboolean stopping = FALSE;
+  struct sample_info *sample_info;
+  guint8 *data;
 
-  debug_print (2, "Writing %d frames...\n", frames);
-
-  memset (buffer, 0, size);
+  debug_print (2, "Writing %d frames...", frames);
 
   g_mutex_lock (&audio->control.mutex);
 
-  if ((audio->pos == len && !audio->loop) ||
+  sample_info = audio->sample.info;
+
+  if (!sample_info)
+    {
+      goto end;
+    }
+
+  data = audio->sample.content->data;
+
+  if (AUDIO_SEL_LEN (audio))
+    {
+      start = audio->sel_start;
+      end = audio->sel_end;
+    }
+  else
+    {
+      start = sample_info->loop_start;
+      end = sample_info->loop_end;
+    }
+
+  bytes_per_frame = SAMPLE_INFO_FRAME_SIZE (sample_info);
+  size = frames * FRAME_SIZE (AUDIO_CHANNELS, SF_FORMAT_PCM_16);
+
+  memset (buffer, 0, size);
+
+  if ((audio->pos == sample_info->frames && !audio->loop) ||
       audio->status == AUDIO_STATUS_PREPARING_PLAYBACK ||
       audio->status == AUDIO_STATUS_STOPPING_PLAYBACK)
     {
@@ -74,37 +99,34 @@ audio_write_to_output (struct audio *audio, void *buffer, gint frames)
 	}
       else			//Stopping...
 	{
-	  audio->release_frames += frames;
+	  stopping = TRUE;
 	}
       goto end;
     }
 
   dst = buffer;
 
-  src = (gint16 *) & audio->sample->data[audio->pos * bytes_per_frame];
+  src = (gint16 *) & data[audio->pos * bytes_per_frame];
   for (gint i = 0; i < frames; i++)
     {
-      if (audio->pos == audio->sample_info.loop_end + 1 && audio->loop)
-	{
-	  debug_print (2, "Sample reset\n");
-	  audio->pos = audio->sample_info.loop_start;
-	  src = (gint16 *) & audio->sample->data[audio->pos *
-						 bytes_per_frame];
-	}
-      else if (audio->pos == len)
+      //Using "audio->pos >" instead of "audio->pos ==" improves the playback
+      //of the selection while changing it because it's possible that an audio
+      //iteration might be in the middle of 2 selection changes and in this
+      //case the equality might not have a change.
+      if (audio->pos == sample_info->frames || audio->pos > end + 1)
 	{
 	  if (!audio->loop)
 	    {
 	      break;
 	    }
-	  debug_print (2, "Sample reset\n");
-	  audio->pos = audio->sel_len ? audio->sel_start : 0;
-	  src = (gint16 *) audio->sample->data;
+	  debug_print (2, "Sample reset");
+	  audio->pos = start;
+	  src = (gint16 *) & data[audio->pos * bytes_per_frame];
 	}
 
       if (audio->mono_mix)
 	{
-	  gint16 mix = audio_mix_channels (&src, audio->sample_info.channels);
+	  gint16 mix = audio_mix_channels (&src, sample_info->channels);
 	  audio_copy_sample (dst, &mix, audio);
 	  dst++;
 	  audio_copy_sample (dst, &mix, audio);
@@ -124,6 +146,11 @@ audio_write_to_output (struct audio *audio, void *buffer, gint frames)
     }
 
 end:
+  if (!sample_info || stopping)
+    {
+      audio->release_frames += frames;
+    }
+
   g_mutex_unlock (&audio->control.mutex);
 
   if (audio->release_frames > AUDIO_BUF_FRAMES)
@@ -143,19 +170,20 @@ audio_read_from_input (struct audio *audio, void *buffer, gint frames)
     (audio->record_options & RECORD_STEREO) == RECORD_STEREO ? 2 : 1;
   guint bytes_per_frame = FRAME_SIZE (channels, SF_FORMAT_PCM_16);
   guint record = !(audio->record_options & RECORD_MONITOR_ONLY);
+  struct sample_info *sample_info = audio->sample.info;
 
-  debug_print (2, "Reading %d frames (recording = %d)...\n", frames, record);
+  debug_print (2, "Reading %d frames (recording = %d)...", frames, record);
 
   g_mutex_lock (&audio->control.mutex);
-  recorded_frames = audio->sample->len / bytes_per_frame;
-  remaining_frames = audio->sample_info.frames - recorded_frames;
+  recorded_frames = audio->sample.content->len / bytes_per_frame;
+  remaining_frames = sample_info->frames - recorded_frames;
   recording_frames = remaining_frames > frames ? frames : remaining_frames;
 
   if (channels == 2)
     {
       if (record)
 	{
-	  g_byte_array_append (audio->sample, buffer,
+	  g_byte_array_append (audio->sample.content, buffer,
 			       recording_frames * bytes_per_frame);
 	}
       data = buffer;
@@ -178,7 +206,7 @@ audio_read_from_input (struct audio *audio, void *buffer, gint frames)
 	{
 	  if (record)
 	    {
-	      g_byte_array_append (audio->sample, (guint8 *) data,
+	      g_byte_array_append (audio->sample.content, (guint8 *) data,
 				   sizeof (gint16));
 	    }
 	  if (*data > level)
@@ -208,41 +236,51 @@ audio_reset_record_buffer (struct audio *audio, guint record_options,
 			   void (*monitor) (void *, gdouble),
 			   void *monitor_data)
 {
-  audio->sample_info.channels = (record_options & RECORD_STEREO) == 3 ? 2 : 1;
-  audio->sample_info.frames = audio->sample_info.rate * MAX_RECORDING_TIME_S;
-  audio->sample_info.loop_start = audio->sample_info.frames - 1;
-  audio->sample_info.loop_end = audio->sample_info.loop_start;
-  audio->sample_info.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
-  guint size =
-    audio->sample_info.frames * SAMPLE_INFO_FRAME_SIZE (&audio->sample_info);
-  g_byte_array_set_size (audio->sample, size);
-  audio->sample->len = 0;
+  guint size;
+  GByteArray *content;
+  struct sample_info *si = g_malloc (sizeof (struct sample_info));
+
+  debug_print (1, "Resetting record buffer...");
+
+  si->channels = (record_options & RECORD_STEREO) == 3 ? 2 : 1;
+  si->frames = audio->rate * MAX_RECORDING_TIME_S;
+  si->loop_start = si->frames - 1;
+  si->loop_end = si->loop_start;
+  si->format = SF_FORMAT_PCM_16;
+  si->rate = audio->rate;
+  si->midi_note = 0;
+  si->loop_type = 0;
+
+  size = si->frames * SAMPLE_INFO_FRAME_SIZE (si);
+  content = g_byte_array_sized_new (size);
+
+  g_mutex_lock (&audio->control.mutex);
+  idata_free (&audio->sample);
+  idata_init (&audio->sample, content, NULL, si);
   audio->pos = 0;
   audio->record_options = record_options;
   audio->monitor = monitor;
   audio->monitor_data = monitor_data;
+  g_mutex_unlock (&audio->control.mutex);
 }
 
 void
 audio_init (struct audio *audio,
 	    void (*volume_change_callback) (gpointer, gdouble),
-	    void (*audio_ready_callback) (), gpointer data)
+	    void (*audio_ready_callback) (gpointer), gpointer data)
 {
-  debug_print (1, "Initializing audio (%s %s)...\n", audio_name (),
+  debug_print (1, "Initializing audio (%s %s)...", audio_name (),
 	       audio_version ());
-  audio->sample = g_byte_array_new ();
-  audio->sample_info.frames = 0;
-  audio->sample_info.rate = 0;
-  audio->sample_info.channels = 0;
+  idata_init (&audio->sample, NULL, NULL, NULL);
   audio->loop = FALSE;
   audio->path = NULL;
   audio->status = AUDIO_STATUS_STOPPED;
   audio->volume_change_callback = volume_change_callback;
-  audio->volume_change_callback_data = data;
   audio->ready_callback = audio_ready_callback;
-  audio->control.data = g_malloc (sizeof (struct sample_info));
+  audio->callback_data = data;
   audio->control.callback = NULL;
-  audio->sel_len = 0;
+  audio->sel_start = -1;
+  audio->sel_end = -1;
 
   audio_init_int (audio);
 }
@@ -250,36 +288,29 @@ audio_init (struct audio *audio,
 void
 audio_destroy (struct audio *audio)
 {
-  debug_print (1, "Destroying audio...\n");
+  debug_print (1, "Destroying audio...");
 
   audio_stop_playback (audio);
   audio_stop_recording (audio);
   audio_reset_sample (audio);
 
   g_mutex_lock (&audio->control.mutex);
-
   audio_destroy_int (audio);
-
-  g_free (audio->control.data);
-  g_byte_array_free (audio->sample, TRUE);
-  audio->sample = NULL;
-
   g_mutex_unlock (&audio->control.mutex);
 }
 
 void
 audio_reset_sample (struct audio *audio)
 {
+  debug_print (1, "Resetting sample...");
+
   g_mutex_lock (&audio->control.mutex);
-  debug_print (1, "Resetting sample...\n");
-  g_byte_array_set_size (audio->sample, 0);
-  audio->sample_info.frames = 0;
+  idata_free (&audio->sample);
   audio->pos = 0;
   g_free (audio->path);
   audio->path = NULL;
   audio->release_frames = 0;
   audio->status = AUDIO_STATUS_STOPPED;
-  memset (audio->control.data, 0, sizeof (struct sample_info));
   g_mutex_unlock (&audio->control.mutex);
 }
 
@@ -287,7 +318,7 @@ void
 audio_prepare (struct audio *audio, enum audio_status status)
 {
   g_mutex_lock (&audio->control.mutex);
-  audio->pos = audio->sel_len ? audio->sel_start : 0;
+  audio->pos = audio->sel_end - audio->sel_start ? audio->sel_start : 0;
   audio->release_frames = 0;
   audio->status = status;
   g_mutex_unlock (&audio->control.mutex);
@@ -297,12 +328,13 @@ guint
 audio_detect_start (struct audio *audio)
 {
   guint start_frame = 0;
-  gint16 *data = (gint16 *) audio->sample->data;
+  struct sample_info *sample_info = audio->sample.info;
+  gint16 *data = (gint16 *) audio->sample.content->data;
 
 //Searching for audio data...
-  for (gint i = 0; i < audio->sample_info.frames; i++)
+  for (gint i = 0; i < sample_info->frames; i++)
     {
-      for (gint j = 0; j < audio->sample_info.channels; j++, data++)
+      for (gint j = 0; j < sample_info->channels; j++, data++)
 	{
 	  if (!start_frame && abs (*data) >= SHRT_MAX * 0.01)
 	    {
@@ -316,10 +348,10 @@ audio_detect_start (struct audio *audio)
 search_last_zero:
   for (gint i = start_frame - 1; i >= 1; i--)
     {
-      for (gint j = 0; j < audio->sample_info.channels; j++, data--)
+      for (gint j = 0; j < sample_info->channels; j++, data--)
 	{
 	  gint16 curr = *data;
-	  gint16 prev = *(data - audio->sample_info.channels);
+	  gint16 prev = *(data - sample_info->channels);
 	  if ((curr > 0 && prev < 0) || (curr < 0 && prev > 0))
 	    {
 	      start_frame = i - 1;
@@ -329,58 +361,55 @@ search_last_zero:
     }
 
 end:
-  data = (gint16 *) & audio->sample->data[start_frame];
-  for (gint j = 0; j < audio->sample_info.channels; j++, data++)
+  data = (gint16 *) & audio->sample.content->data[start_frame];
+  for (gint j = 0; j < sample_info->channels; j++, data++)
     {
       *data = 0;
     }
-  debug_print (1, "Detected start at frame %d\n", start_frame);
+  debug_print (1, "Detected start at frame %d", start_frame);
   return start_frame;
 }
 
 void
-audio_delete_range (struct audio *audio, guint start_frame, guint frames)
+audio_delete_range (struct audio *audio, guint32 start, guint32 length)
 {
-  gdouble r;
-  struct sample_info *sample_info_src = audio->control.data;
-  guint bytes_per_frame = SAMPLE_INFO_FRAME_SIZE (&audio->sample_info);
-  guint index = start_frame * bytes_per_frame;
-  guint len = frames * bytes_per_frame;
-
-  debug_print (2, "Deleting range from %d with len %d...\n", index, len);
-  g_byte_array_remove_range (audio->sample, index, len);
+  guint bytes_per_frame, index, len;
+  struct sample_info *sample_info = audio->sample.info;
 
   g_mutex_lock (&audio->control.mutex);
-  audio->sample_info.frames -= (guint32) frames;
 
-  if (audio->sample_info.loop_start >= audio->sel_start + audio->sel_len)
+  bytes_per_frame = SAMPLE_INFO_FRAME_SIZE (sample_info);
+  index = start * bytes_per_frame;
+  len = length * bytes_per_frame;
+
+  debug_print (2, "Deleting range from %d with len %d...", index, len);
+  g_byte_array_remove_range (audio->sample.content, index, len);
+
+  sample_info->frames -= length;
+
+  if (sample_info->loop_start >= audio->sel_end)
     {
-      audio->sample_info.loop_start -= (guint32) audio->sel_len;
+      sample_info->loop_start -= length;
     }
-  else if (audio->sample_info.loop_start >= audio->sel_start &&
-	   audio->sample_info.loop_start < audio->sel_start + audio->sel_len)
+  else if (sample_info->loop_start >= audio->sel_start &&
+	   sample_info->loop_start < audio->sel_end)
     {
-      audio->sample_info.loop_start = 0;
+      sample_info->loop_start = sample_info->frames - 1;
     }
 
-  if (audio->sample_info.loop_end >= audio->sel_start + audio->sel_len)
+  if (sample_info->loop_end >= audio->sel_end)
     {
-      audio->sample_info.loop_end -= (guint32) audio->sel_len;
+      sample_info->loop_end -= length;
     }
-  else if (audio->sample_info.loop_end >= audio->sel_start &&
-	   audio->sample_info.loop_end < audio->sel_start + audio->sel_len)
+  else if (sample_info->loop_end >= audio->sel_start &&
+	   sample_info->loop_end < audio->sel_end)
     {
-      audio->sample_info.loop_end = audio->sample_info.frames - 1;
+      sample_info->loop_end = sample_info->frames - 1;
     }
 
-  audio->sel_start = 0;
-  audio->sel_len = 0;
+  audio->sel_start = -1;
+  audio->sel_end = -1;
 
-  r = sample_info_src->rate / (double) audio->sample_info.rate;
-  sample_info_src->frames = floor (audio->sample_info.frames * r);
-  sample_info_src->loop_start = round (audio->sample_info.loop_start * r);
-  sample_info_src->loop_end = round (audio->sample_info.loop_end * r);
-  sample_check_and_fix_loop_points (sample_info_src);
   g_mutex_unlock (&audio->control.mutex);
 }
 
@@ -389,9 +418,9 @@ audio_normalize (struct audio *audio)
 {
   gdouble ratio, ratiop, ration;
   gint16 *data, maxp = 1, minn = -1;
-  guint samples = audio->sample->len / SAMPLE_SIZE (SF_FORMAT_PCM_16);
+  guint samples = audio->sample.content->len / SAMPLE_SIZE (SF_FORMAT_PCM_16);
 
-  data = (gint16 *) audio->sample->data;
+  data = (gint16 *) audio->sample.content->data;
   for (gint i = 0; i < samples; i++, data++)
     {
       gint16 v = *data;
@@ -414,9 +443,9 @@ audio_normalize (struct audio *audio)
   ration = SHRT_MIN / (gdouble) minn;
   ratio = ratiop < ration ? ratiop : ration;
 
-  debug_print (1, "Normalizing to %f...\n", ratio);
+  debug_print (1, "Normalizing to %f...", ratio);
 
-  data = (gint16 *) audio->sample->data;
+  data = (gint16 *) audio->sample.content->data;
   for (gint i = 0; i < samples; i++, data++)
     {
       *data = (gint16) (*data * ratio);
@@ -426,16 +455,15 @@ audio_normalize (struct audio *audio)
 void
 audio_finish_recording (struct audio *audio)
 {
-  struct sample_info *sample_info = audio->control.data;
+  struct sample_info *sample_info = audio->sample.info;
   guint record = !(audio->record_options & RECORD_MONITOR_ONLY);
 
   g_mutex_lock (&audio->control.mutex);
   audio->status = AUDIO_STATUS_STOPPED;
-  audio->sample_info.frames =
-    audio->sample->len / SAMPLE_INFO_FRAME_SIZE (&audio->sample_info);
-  audio->sample_info.loop_start = audio->sample_info.frames - 1;
-  audio->sample_info.loop_end = audio->sample_info.loop_start;
-  memcpy (sample_info, &audio->sample_info, sizeof (struct sample_info));
+  sample_info->frames =
+    audio->sample.content->len / SAMPLE_INFO_FRAME_SIZE (sample_info);
+  sample_info->loop_start = sample_info->frames - 1;
+  sample_info->loop_end = sample_info->loop_start;
   if (record)
     {
       audio_normalize (audio);
