@@ -18,8 +18,14 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <stdarg.h>
 #include "backend.h"
 #include "local.h"
+#include "sample.h"
+#include "preferences.h"
+
+struct connector *system_connector = NULL;
+GSList *connectors = NULL;
 
 // When sending a batch of SysEx messages we want the trasfer status to be controlled outside this function.
 // This is what the update parameter is for.
@@ -45,50 +51,34 @@ backend_get_storage_stats_percent (struct backend_storage_stats *statfs)
   return (statfs->bsize - statfs->bfree) * 100.0 / statfs->bsize;
 }
 
+static gint
+backend_get_fs_operations_id_comparator (gconstpointer a, gconstpointer b)
+{
+  const struct fs_operations *ops = a;
+  return ops->id != *((guint32 *) b);
+}
+
 const struct fs_operations *
-backend_get_fs_operations (struct backend *backend, gint fs,
-			   const gchar * name)
+backend_get_fs_operations_by_id (struct backend *backend, guint32 id)
 {
-  const struct fs_operations **fs_operations = backend->fs_ops;
-  if (!fs_operations)
-    {
-      return NULL;
-    }
-  while (*fs_operations)
-    {
-      const struct fs_operations *ops = *fs_operations;
-      if (ops->fs == fs || (name && !strcmp (ops->name, name)))
-	{
-	  return ops;
-	}
-      fs_operations++;
-    }
-  return NULL;
+  GSList *e = g_slist_find_custom (backend->fs_ops, &id,
+				   backend_get_fs_operations_id_comparator);
+  return e ? e->data : NULL;
 }
 
-void
-backend_enable_cache (struct backend *backend)
+static gint
+backend_get_fs_operations_name_comparator (gconstpointer a, gconstpointer b)
 {
-  g_mutex_lock (&backend->mutex);
-  if (!backend->cache)
-    {
-      backend->cache = g_hash_table_new_full (g_bytes_hash, g_bytes_equal,
-					      (GDestroyNotify) g_bytes_unref,
-					      (GDestroyNotify) free_msg);
-    }
-  g_mutex_unlock (&backend->mutex);
+  const struct fs_operations *ops = a;
+  return strcmp (ops->name, (gchar *) b);
 }
 
-void
-backend_disable_cache (struct backend *backend)
+const struct fs_operations *
+backend_get_fs_operations_by_name (struct backend *backend, const gchar *name)
 {
-  g_mutex_lock (&backend->mutex);
-  if (backend->cache)
-    {
-      g_hash_table_destroy (backend->cache);
-      backend->cache = NULL;
-    }
-  g_mutex_unlock (&backend->mutex);
+  GSList *e = g_slist_find_custom (backend->fs_ops, name,
+				   backend_get_fs_operations_name_comparator);
+  return e ? e->data : NULL;
 }
 
 void
@@ -113,7 +103,7 @@ backend_midi_handshake (struct backend *backend)
 				    BE_SYSEX_TIMEOUT_GUESS_MS);
   if (!rx_msg)
     {
-      debug_print (1, "No MIDI identity reply\n");
+      debug_print (1, "No MIDI identity reply");
       return;
     }
 
@@ -145,20 +135,22 @@ backend_midi_handshake (struct backend *backend)
 		    backend->midi_info.version[1],
 		    backend->midi_info.version[2],
 		    backend->midi_info.version[3]);
-	  debug_print (1, "Detected device: %s %s\n", backend->name,
+	  debug_print (1, "Detected device: %s %s", backend->name,
 		       backend->version);
 	}
       else
 	{
-	  debug_print (1, "Illegal MIDI identity reply length\n");
+	  debug_print (1, "Illegal MIDI identity reply length");
 	}
     }
   else
     {
-      debug_print (1, "Illegal SUB-ID2\n");
+      debug_print (1, "Illegal SUB-ID2");
     }
 
   free_msg (rx_msg);
+
+  usleep (BE_REST_TIME_US);
 }
 
 gint
@@ -177,7 +169,7 @@ backend_tx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 //Synchronized
 
 gint
-backend_tx (struct backend *backend, GByteArray * tx_msg)
+backend_tx (struct backend *backend, GByteArray *tx_msg)
 {
   struct sysex_transfer transfer;
   transfer.raw = tx_msg;
@@ -188,29 +180,6 @@ backend_tx (struct backend *backend, GByteArray * tx_msg)
   return transfer.err;
 }
 
-//Not synchronized. Only meant to be called from backend_tx_and_rx_sysex_transfer.
-
-static gint
-backend_tx_and_rx_sysex_transfer_no_cache (struct backend *backend,
-					   struct sysex_transfer *transfer,
-					   gboolean free)
-{
-  transfer->batch = FALSE;
-
-  backend_tx_sysex (backend, transfer);
-  if (free)
-    {
-      free_msg (transfer->raw);
-      transfer->raw = NULL;
-    }
-  if (!transfer->err)
-    {
-      backend_rx_sysex (backend, transfer);
-    }
-
-  return transfer->err;
-}
-
 //Synchronized
 
 gint
@@ -218,41 +187,26 @@ backend_tx_and_rx_sysex_transfer (struct backend *backend,
 				  struct sysex_transfer *transfer,
 				  gboolean free)
 {
-  GBytes *key;
-  GByteArray *rx_msg;
   transfer->batch = FALSE;
 
   g_mutex_lock (&backend->mutex);
-  if (backend->cache)
+
+  if (transfer->raw)
     {
-      key = g_bytes_new (transfer->raw->data, transfer->raw->len);
-      rx_msg = g_hash_table_lookup (backend->cache, key);
-      if (rx_msg)
+      backend_tx_sysex (backend, transfer);
+      if (free)
 	{
-	  transfer->raw = g_byte_array_sized_new (rx_msg->len);
-	  g_byte_array_append (transfer->raw, rx_msg->data, rx_msg->len);
-	  transfer->err = 0;
-	  g_bytes_unref (key);
-	  goto end;
+	  free_msg (transfer->raw);
+	  transfer->raw = NULL;
 	}
-
-      if (backend_tx_and_rx_sysex_transfer_no_cache (backend, transfer, free))
-	{
-	  g_bytes_unref (key);
-	  goto end;
-	}
-
-      rx_msg = g_byte_array_sized_new (transfer->raw->len);
-      g_byte_array_append (rx_msg, transfer->raw->data, transfer->raw->len);
-      g_hash_table_insert (backend->cache, key, rx_msg);
     }
-  else
+  if (!transfer->err)
     {
-      backend_tx_and_rx_sysex_transfer_no_cache (backend, transfer, free);
+      backend_rx_sysex (backend, transfer);
     }
 
-end:
   g_mutex_unlock (&backend->mutex);
+
   return transfer->err;
 }
 
@@ -260,7 +214,7 @@ end:
 //A timeout of 0 means infinity; a negative timeout means the default timeout.
 
 GByteArray *
-backend_tx_and_rx_sysex (struct backend *backend, GByteArray * tx_msg,
+backend_tx_and_rx_sysex (struct backend *backend, GByteArray *tx_msg,
 			 gint timeout)
 {
   struct sysex_transfer transfer;
@@ -270,18 +224,10 @@ backend_tx_and_rx_sysex (struct backend *backend, GByteArray * tx_msg,
   return transfer.raw;
 }
 
-gchar *
-backend_get_fs_ext (struct backend *backend, const struct fs_operations *ops)
-{
-  gchar *ext = g_malloc (LABEL_MAX);
-  snprintf (ext, LABEL_MAX, "%s", ops->type_ext);
-  return ext;
-}
-
 void
 backend_destroy_data (struct backend *backend)
 {
-  debug_print (1, "Destroying backend data...\n");
+  debug_print (1, "Destroying backend data...");
   g_free (backend->data);
   backend->data = NULL;
 }
@@ -296,7 +242,7 @@ backend_program_change (struct backend *backend, guint8 channel,
   msg[0] = 0xc0 | (channel & 0xf);
   msg[1] = program & 0x7f;
 
-  debug_print (1, "Sending MIDI program %d...\n", msg[1]);
+  debug_print (1, "Sending MIDI program %d...", msg[1]);
 
   if ((size = backend_tx_raw (backend, msg, 2)) < 0)
     {
@@ -316,7 +262,7 @@ backend_send_3_byte_message (struct backend *backend, guint8 msg_type,
   msg[1] = d1 & 0x7f;
   msg[2] = d2 & 0x7f;
 
-  debug_print (1, "Sending MIDI message: status %08x; data %d, %d...\n",
+  debug_print (1, "Sending MIDI message: status %08x; data %d, %d...",
 	       msg[0], msg[1], msg[2]);
 
   if ((size = backend_tx_raw (backend, msg, 3)) < 0)
@@ -362,9 +308,9 @@ backend_send_rpn (struct backend *backend, guint8 channel,
 }
 
 gint
-backend_init (struct backend *backend, const gchar * id)
+backend_init_midi (struct backend *backend, const gchar *id)
 {
-  debug_print (1, "Initializing backend (%s) to '%s'...\n",
+  debug_print (1, "Initializing backend (%s) to '%s'...",
 	       backend_name (), id);
   backend->type = BE_TYPE_MIDI;
   gint err = backend_init_int (backend, id);
@@ -375,26 +321,29 @@ backend_init (struct backend *backend, const gchar * id)
       g_mutex_unlock (&backend->mutex);
     }
 
-  debug_print (1, "Stopping device...\n");
-  if (backend_tx_raw (backend, (guint8 *) "\xfc", 1) < 0)
+  if (preferences_get_boolean (PREF_KEY_STOP_DEVICE_WHEN_CONNECTING))
     {
-      error_print ("Error while stopping device\n");
+      debug_print (1, "Stopping device...");
+      if (backend_tx_raw (backend, (guint8 *) "\xfc", 1) < 0)
+	{
+	  error_print ("Error while stopping device");
+	}
+      usleep (BE_REST_TIME_US);
     }
+
   return err;
 }
 
 void
 backend_destroy (struct backend *backend)
 {
-  debug_print (1, "Destroying backend...\n");
+  debug_print (1, "Destroying backend...");
 
   if (backend->destroy_data)
     {
       backend->destroy_data (backend);
     }
 
-  backend_disable_cache (backend);
-
   if (backend->type == BE_TYPE_MIDI)
     {
       backend_destroy_int (backend);
@@ -404,7 +353,7 @@ backend_destroy (struct backend *backend)
   backend->get_storage_stats = NULL;
   backend->destroy_data = NULL;
   backend->type = BE_TYPE_NONE;
-  backend->fs_ops = NULL;
+  g_slist_free (backend->fs_ops);
 }
 
 gboolean
@@ -431,11 +380,11 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
 
   if (!backend->inputp)
     {
-      error_print ("Input port is NULL\n");
+      error_print ("Input port is NULL");
       return -ENOTCONN;
     }
 
-  debug_print (4, "Reading data...\n");
+  debug_print (4, "Reading data...");
 
   while (1)
     {
@@ -444,17 +393,17 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
 	  return -ECANCELED;
 	}
 
-      debug_print (6, "Checking timeout (%d ms, %d ms, %s mode)...\n",
+      debug_print (6, "Checking timeout (%d ms, %d ms, %s mode)...",
 		   transfer->time, transfer->timeout,
 		   transfer->batch ? "batch" : "single");
       if (((transfer->batch && transfer->status == RECEIVING)
 	   || !transfer->batch) && transfer->timeout > -1
 	  && transfer->time >= transfer->timeout)
 	{
-	  debug_print (1, "Timeout (%d)\n", transfer->timeout);
+	  debug_print (1, "Timeout (%d)", transfer->timeout);
 	  gchar *text = debug_get_hex_data (debug_level, backend->buffer,
 					    backend->rx_len);
-	  debug_print (4, "Internal buffer data (%zd): %s\n", backend->rx_len,
+	  debug_print (4, "Internal buffer data (%zd): %s", backend->rx_len,
 		       text);
 	  g_free (text);
 	  return -ETIMEDOUT;
@@ -482,7 +431,7 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
 	  if (debug_level >= 4)
 	    {
 	      gchar *text = debug_get_hex_data (debug_level, tmp, rx_len);
-	      debug_print (4, "Skipping non SysEx data (%zd): %s\n", rx_len,
+	      debug_print (4, "Skipping non SysEx data (%zd): %s", rx_len,
 			   text);
 	      g_free (text);
 	    }
@@ -521,7 +470,7 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
   if (debug_level >= 3)
     {
       text = debug_get_hex_data (debug_level, data, rx_len);
-      debug_print (3, "Queued data (%zu): %s\n", rx_len, text);
+      debug_print (3, "Queued data (%zu): %s", rx_len, text);
       g_free (text);
     }
 
@@ -548,7 +497,7 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
     {
       if (backend->rx_len == next_check)
 	{
-	  debug_print (4, "Reading from MIDI device...\n");
+	  debug_print (4, "Reading from MIDI device...");
 	  if (transfer->batch)
 	    {
 	      transfer->time = 0;
@@ -576,7 +525,7 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 	}
       else
 	{
-	  debug_print (4, "Reading from internal buffer...\n");
+	  debug_print (4, "Reading from internal buffer...");
 	}
 
       transfer->status = RECEIVING;
@@ -603,11 +552,11 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 	    {
 	      gchar *text = debug_get_hex_data (debug_level, backend->buffer,
 						i);
-	      debug_print (4, "Skipping non SysEx data in buffer (%d): %s\n",
+	      debug_print (4, "Skipping non SysEx data in buffer (%d): %s",
 			   i, text);
 	      g_free (text);
 	    }
-	  debug_print (3, "Copying %d bytes...\n", len - i);
+	  debug_print (3, "Copying %d bytes...", len - i);
 	  g_byte_array_append (transfer->raw, b, len - i);
 
 	  backend->rx_len -= len;
@@ -620,7 +569,7 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 	  if (transfer->raw->len == 2
 	      && !memcmp (transfer->raw->data, "\xf0\xf7", 2))
 	    {
-	      debug_print (4, "Removing empty message...\n");
+	      debug_print (4, "Removing empty message...");
 	      g_byte_array_remove_range (transfer->raw, 0, 2);
 	      continue;
 	    }
@@ -630,14 +579,14 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 	      gchar *text =
 		debug_get_hex_data (debug_level, transfer->raw->data,
 				    transfer->raw->len);
-	      debug_print (4, "Queued data (%d): %s\n",
+	      debug_print (4, "Queued data (%d): %s",
 			   transfer->raw->len, text);
 	      g_free (text);
 	    }
 	}
       else
 	{
-	  debug_print (4, "No message in the queue. Continuing...\n");
+	  debug_print (4, "No message in the queue. Continuing...");
 	}
 
       if (transfer->raw->len && !transfer->batch)
@@ -662,7 +611,7 @@ end:
 	{
 	  gchar *text = debug_get_hex_data (debug_level, transfer->raw->data,
 					    transfer->raw->len);
-	  debug_print (2, "Raw message received (%d): %s\n",
+	  debug_print (2, "Raw message received (%d): %s",
 		       transfer->raw->len, text);
 	  g_free (text);
 	}
@@ -682,7 +631,7 @@ backend_rx_drain (struct backend *backend)
   transfer.timeout = 1000;
   transfer.batch = FALSE;
 
-  debug_print (2, "Draining buffers...\n");
+  debug_print (2, "Draining buffers...");
   backend->rx_len = 0;
   backend_rx_drain_int (backend);
   while (!backend_rx_sysex (backend, &transfer))
@@ -692,7 +641,7 @@ backend_rx_drain (struct backend *backend)
 }
 
 enum path_type
-path_type_from_backend (struct backend *backend)
+backend_get_path_type (struct backend *backend)
 {
   return (!backend || backend->type == BE_TYPE_SYSTEM) ? PATH_SYSTEM :
     PATH_INTERNAL;
@@ -714,3 +663,130 @@ backend_get_devices ()
   backend_fill_devices_array (devices);
   return devices;
 }
+
+// A handshake function might return these values:
+// 0, the device matches the connector.
+// -ENODEV, the device does not match the connector but we can continue with the next connector.
+// Other negative errors are allowed but we will not continue with the remaining connectors.
+
+gint
+backend_init_connector (struct backend *backend,
+			struct backend_device *device,
+			const gchar *conn_name,
+			struct sysex_transfer *sysex_transfer)
+{
+  gint err;
+  GSList *list = NULL, *iterator;
+  gboolean active = TRUE;
+  GSList *c;
+
+  if (device->type == BE_TYPE_SYSTEM)
+    {
+      backend->conn_name = system_connector->name;
+      backend->type = BE_TYPE_SYSTEM;
+      return system_connector->handshake (backend);
+    }
+
+  err = backend_init_midi (backend, device->id);
+  if (err)
+    {
+      return err;
+    }
+
+  c = connectors;
+  while (c)
+    {
+      struct connector *connector = c->data;
+      if (connector->regex)
+	{
+	  GRegex *regex = g_regex_new (connector->regex, G_REGEX_CASELESS,
+				       0, NULL);
+	  if (g_regex_match (regex, device->name, 0, NULL))
+	    {
+	      debug_print (1, "Connector %s matches the device",
+			   connector->name);
+	      list = g_slist_prepend (list, (void *) connector);
+	    }
+	  else
+	    {
+	      list = g_slist_append (list, (void *) connector);
+	    }
+	  g_regex_unref (regex);
+	}
+      else
+	{
+	  list = g_slist_append (list, (void *) connector);
+	}
+      c = c->next;
+    }
+
+  if (!conn_name)
+    {
+      backend_midi_handshake (backend);
+    }
+
+  err = -ENODEV;
+  for (iterator = list; iterator; iterator = iterator->next)
+    {
+      const struct connector *c = iterator->data;
+
+      if (sysex_transfer)
+	{
+	  g_mutex_lock (&sysex_transfer->mutex);
+	  active = sysex_transfer->active;
+	  g_mutex_unlock (&sysex_transfer->mutex);
+	}
+
+      if (!active)
+	{
+	  err = -ECANCELED;
+	  goto end;
+	}
+
+      debug_print (1, "Testing %s connector (%sstandard handshake)...",
+		   c->name, c->standard ? "" : "non ");
+
+      if (conn_name)
+	{
+	  if (!strcmp (conn_name, c->name))
+	    {
+	      if (c->standard)
+		{
+		  backend_midi_handshake (backend);
+		}
+	      err = c->handshake (backend);
+	      if (!err)
+		{
+		  debug_print (1, "Using %s connector...", c->name);
+		  backend->conn_name = c->name;
+		}
+	      goto end;
+	    }
+	}
+      else
+	{
+	  err = c->handshake (backend);
+	  if (err && err != -ENODEV)
+	    {
+	      goto end;
+	    }
+
+	  if (!err)
+	    {
+	      debug_print (1, "Using %s connector...", c->name);
+	      backend->conn_name = c->name;
+	      goto end;
+	    }
+	}
+    }
+
+  error_print ("No device recognized");
+
+end:
+  g_slist_free (list);
+  if (err)
+    {
+      backend_destroy (backend);
+    }
+  return err;
+}
