@@ -22,15 +22,13 @@
 #include "utils.h"
 #include "notifier.h"
 #include "preferences.h"
-#include "backend.h"
+#include "connector.h"
 
 #ifndef BROWSER_H
 #define BROWSER_H
 
 #define SIZE_LABEL_LEN 16
 
-#define DIR_ICON "folder-visiting-symbolic"
-
 //Common columns
 #define BROWSER_LIST_STORE_ICON_FIELD 0
 #define BROWSER_LIST_STORE_NAME_FIELD 1	//This is the value returned by the funciton se in the get_item_key member in struct fs_operations. It's the filename.
@@ -38,20 +36,21 @@
 #define BROWSER_LIST_STORE_SIZE_STR_FIELD 3
 #define BROWSER_LIST_STORE_TYPE_FIELD 4
 #define BROWSER_LIST_STORE_ID_FIELD 5
+#define BROWSER_LIST_STORE_INFO_FIELD 6
+#define BROWSER_LIST_STORE_SAMPLE_FRAMES_FIELD 7
+#define BROWSER_LIST_STORE_SAMPLE_RATE_FIELD 8
+#define BROWSER_LIST_STORE_SAMPLE_TIME_FIELD 9
+#define BROWSER_LIST_STORE_SAMPLE_FORMAT_FIELD 10
+#define BROWSER_LIST_STORE_SAMPLE_CHANNELS_FIELD 11
+#define BROWSER_LIST_STORE_SAMPLE_MIDI_NOTE_FIELD 12
 //Remote columns
-#define BROWSER_LIST_STORE_SLOT_FIELD 6	//This is an optional map of the id (number) to some string like "A1", "001" or "[A:001]" to mimic the device way of numbering the items.
-//Local columns
-#define BROWSER_LIST_STORE_SAMPLE_FRAMES_FIELD 6
-#define BROWSER_LIST_STORE_SAMPLE_RATE_FIELD 7
-#define BROWSER_LIST_STORE_SAMPLE_TIME_FIELD 8
-#define BROWSER_LIST_STORE_SAMPLE_FORMAT_FIELD 9
-#define BROWSER_LIST_STORE_SAMPLE_CHANNELS_FIELD 10
-#define BROWSER_LIST_STORE_SAMPLE_MIDI_NOTE_FIELD 11
+#define BROWSER_LIST_STORE_SLOT_FIELD 13	//This is an optional map of the id (number) to some string like "A1", "001" or "[A:001]" to mimic the device way of numbering the items.
+
+#define BROWSER_IS_SYSTEM(b) (!(b)->backend || (b)->backend->type == BE_TYPE_SYSTEM)
 
 struct browser
 {
   const gchar *name;
-  GSourceFunc check_selection;
   GtkTreeView *view;
   GtkWidget *buttons_stack;
   GtkWidget *up_button;
@@ -70,6 +69,7 @@ struct browser
   struct backend *backend;
     gboolean (*check_callback) ();
   void (*set_columns_visibility) ();
+  void (*set_popup_menuitems_visibility) ();
   struct notifier *notifier;
   //Background loading members
   GSList *sensitive_widgets;
@@ -81,6 +81,7 @@ struct browser
   gboolean dirty;
   gboolean search_mode;
   const gchar *filter;
+  gint64 last_selected_index;	//This needs space for gint and -1
   //Menu
   GtkWidget *transfer_menuitem;
   GtkWidget *play_separator;
@@ -92,22 +93,14 @@ struct browser
   GtkWidget *rename_menuitem;
   GtkWidget *delete_menuitem;
   GtkTreeViewColumn *tree_view_name_column;
-};
-
-struct local_browser
-{
-  struct browser browser;
+  GtkTreeViewColumn *tree_view_info_column;
   GtkTreeViewColumn *tree_view_sample_frames_column;
   GtkTreeViewColumn *tree_view_sample_rate_column;
   GtkTreeViewColumn *tree_view_sample_duration_column;
   GtkTreeViewColumn *tree_view_sample_channels_column;
   GtkTreeViewColumn *tree_view_sample_bits_column;
   GtkTreeViewColumn *tree_view_sample_midi_note_column;
-};
-
-struct remote_browser
-{
-  struct browser browser;
+  //Only present in the remote browser
   GtkTreeViewColumn *tree_view_id_column;
   GtkTreeViewColumn *tree_view_slot_column;
   GtkTreeViewColumn *tree_view_size_column;
@@ -119,7 +112,7 @@ gint browser_get_selected_items_count (struct browser *);
 
 void browser_clear_selection (struct browser *);
 
-void browser_set_selected_row_iter (struct browser *, GtkTreeIter *);
+gint browser_set_selected_row_iter (struct browser *, GtkTreeIter *);
 
 void browser_selection_changed (GtkTreeSelection *, gpointer);
 
@@ -134,19 +127,16 @@ gchar *browser_get_item_path (struct browser *, struct item *);
 
 gboolean browser_load_dir (gpointer);
 
+gboolean browser_load_dir_if_needed (gpointer);
+
 void browser_update_fs_options (struct browser *);
 
-void browser_local_init (struct local_browser *, GtkBuilder *, gchar *);
+void browser_local_init (struct browser *, GtkBuilder *, gchar *);
 
-void browser_remote_init (struct remote_browser *, GtkBuilder *,
-			  struct backend *);
+void browser_remote_init (struct browser *, GtkBuilder *, struct backend *);
 
 void browser_destroy (struct browser *);
 
-gboolean browser_set_file_extensions (struct browser *, const gchar **);
-
-gboolean browser_set_file_extension (struct browser *, gchar *);
-
 void browser_reset (struct browser *);
 
 void browser_clear_dnd_function (struct browser *);
@@ -159,6 +149,4 @@ void browser_close_search (GtkSearchEntry *, gpointer);
 
 void browser_search_changed (GtkSearchEntry *, gpointer);
 
-void browser_disable_sample_menuitems (struct browser *);
-
 #endif
