@@ -31,22 +31,109 @@
 #define PKG_TAG_PAYLOAD "Payload"
 #define PKG_TAG_FILE_TYPE "FileType"
 #define PKG_TAG_FIRMWARE_VERSION "FirmwareVersion"
+#define PKG_TAG_METAINFO "MetaInfo"
+#define PKG_TAG_TAGS "Tags"
 #define PKG_TAG_SAMPLES "Samples"
 #define PKG_TAG_FILE_NAME "FileName"
 #define PKG_TAG_FILE_SIZE "FileSize"
 #define PKG_TAG_HASH "Hash"
 #define PKG_VAL_FILE_TYPE_PRJ "Project"
 #define PKG_VAL_FILE_TYPE_SND "Sound"
+#define PKG_VAL_FILE_TYPE_PST "Preset"
 #define PKG_VAL_FILE_TYPE_UNK "Unknown"
 
 #define MAN_TAG_SAMPLE_REFS "sample_references"
 #define MAN_TAG_HASH "hash"
 #define MAN_TAG_SIZE "size"
 
+#define METADATA_TAG_SAMPLE_REFS "sound_tags"
+
 #define MAX_PACKAGE_LEN (64 * 1024 * 1024)
 #define MAX_MANIFEST_LEN (128 * 1024)
 #define MANIFEST_FILENAME "manifest.json"
 
+static GSList *
+package_get_tags_from_snd_metadata_int (JsonReader *reader)
+{
+  gint elements;
+  GSList *tags = NULL;
+
+  if (!json_reader_read_member (reader, METADATA_TAG_SAMPLE_REFS))
+    {
+      debug_print (1, "Member '%s' not found", METADATA_TAG_SAMPLE_REFS);
+      return NULL;
+    }
+
+  if (!json_reader_is_array (reader))
+    {
+      error_print ("Member '%s' is not an array. Continuing...",
+		   METADATA_TAG_SAMPLE_REFS);
+      goto end;
+    }
+
+  elements = json_reader_count_elements (reader);
+  if (!elements)
+    {
+      debug_print (1, "No tags found");
+      return NULL;
+    }
+
+  for (gint i = 0; i < elements; i++)
+    {
+      const gchar *tag;
+
+      if (!json_reader_read_element (reader, i))
+	{
+	  error_print ("Cannot read element %d. Continuing...", i);
+	  continue;
+	}
+
+      tag = json_reader_get_string_value (reader);
+      tags = g_slist_append (tags, strdup (tag));
+      json_reader_end_element (reader);
+    }
+
+end:
+  json_reader_end_element (reader);
+
+  return tags;
+}
+
+GSList *
+package_get_tags_from_snd_metadata (GByteArray *metadata)
+{
+  JsonParser *parser;
+  JsonReader *reader;
+  GError *error;
+  GSList *tags = NULL;
+
+  parser = json_parser_new ();
+  if (!json_parser_load_from_data (parser, (gchar *) metadata->data,
+				   metadata->len, &error))
+    {
+      error_print ("Unable to parse stream: %s. Continuing...",
+		   error->message);
+      g_clear_error (&error);
+      goto cleanup_parser;
+    }
+
+  reader = json_reader_new (json_parser_get_root (parser));
+  if (!reader)
+    {
+      error_print ("Unable to read from parser. Continuing...");
+      goto cleanup_parser;
+    }
+
+  tags = package_get_tags_from_snd_metadata_int (reader);
+
+  g_object_unref (reader);
+
+cleanup_parser:
+  g_object_unref (parser);
+
+  return tags;
+}
+
 static gint
 package_add_resource (struct package *pkg,
 		      struct package_resource *pkg_resource, gboolean new)
@@ -55,25 +142,24 @@ package_add_resource (struct package *pkg,
   zip_int64_t index;
   zip_error_t zerror;
 
-  debug_print (1, "Adding file %s to zip (%d B)...\n", pkg_resource->path,
+  debug_print (1, "Adding file %s to zip (%d B)...", pkg_resource->path,
 	       pkg_resource->data->len);
-  sample_source =
-    zip_source_buffer_create (pkg_resource->data->data,
-			      pkg_resource->data->len, 0, &zerror);
+  sample_source = zip_source_buffer_create (pkg_resource->data->data,
+					    pkg_resource->data->len, 0,
+					    &zerror);
   if (!sample_source)
     {
-      error_print ("Error while creating file source: %s\n",
+      error_print ("Error while creating file source: %s",
 		   zip_error_strerror (&zerror));
       zip_error_fini (&zerror);
       return -1;
     }
 
-  index =
-    zip_file_add (pkg->zip, pkg_resource->path, sample_source,
-		  ZIP_FL_OVERWRITE | ZIP_FL_ENC_UTF_8);
+  index = zip_file_add (pkg->zip, pkg_resource->path, sample_source,
+			ZIP_FL_OVERWRITE | ZIP_FL_ENC_UTF_8);
   if (index < 0)
     {
-      error_print ("Error while adding file: %s\n",
+      error_print ("Error while adding file: %s",
 		   zip_error_strerror (zip_get_error (pkg->zip)));
       zip_source_free (sample_source);
       return -1;
@@ -89,7 +175,7 @@ package_add_resource (struct package *pkg,
 
 
 gint
-package_begin (struct package *pkg, gchar * name, const gchar * fw_version,
+package_begin (struct package *pkg, gchar *name, const gchar *fw_version,
 	       const struct device_desc *device_desc, enum package_type type)
 {
   zip_error_t zerror;
@@ -100,14 +186,14 @@ package_begin (struct package *pkg, gchar * name, const gchar * fw_version,
   pkg->device_desc = device_desc;
   pkg->type = type;
 
-  debug_print (1, "Creating zip buffer...\n");
+  debug_print (1, "Creating zip buffer...");
 
   zip_error_init (&zerror);
-  pkg->zip_source =
-    zip_source_buffer_create (pkg->buff, MAX_PACKAGE_LEN, 0, &zerror);
+  pkg->zip_source = zip_source_buffer_create (pkg->buff, MAX_PACKAGE_LEN, 0,
+					      &zerror);
   if (!pkg->zip_source)
     {
-      error_print ("Error while creating zip source: %s\n",
+      error_print ("Error while creating zip source: %s",
 		   zip_error_strerror (&zerror));
       zip_error_fini (&zerror);
       g_free (pkg->buff);
@@ -117,7 +203,7 @@ package_begin (struct package *pkg, gchar * name, const gchar * fw_version,
   pkg->zip = zip_open_from_source (pkg->zip_source, ZIP_TRUNCATE, &zerror);
   if (!pkg->zip)
     {
-      error_print ("Error while creating in memory zip: %s\n",
+      error_print ("Error while creating in memory zip: %s",
 		   zip_error_strerror (&zerror));
       zip_error_fini (&zerror);
       zip_source_free (pkg->zip_source);
@@ -136,6 +222,27 @@ package_begin (struct package *pkg, gchar * name, const gchar * fw_version,
   return 0;
 }
 
+static const gchar *
+package_get_file_type (enum package_type type)
+{
+  if (type & PKG_FILE_TYPE_DATA_PROJECT)
+    {
+      return PKG_VAL_FILE_TYPE_PRJ;
+    }
+  else if (type & PKG_FILE_TYPE_DATA_SOUND)
+    {
+      return PKG_VAL_FILE_TYPE_SND;
+    }
+  else if (type & PKG_FILE_TYPE_DATA_PRESET)
+    {
+      return PKG_VAL_FILE_TYPE_PST;
+    }
+  else
+    {
+      return PKG_VAL_FILE_TYPE_UNK;
+    }
+}
+
 static gint
 package_add_manifest (struct package *pkg)
 {
@@ -166,13 +273,9 @@ package_add_manifest (struct package *pkg)
   json_builder_add_string_value (builder, pkg->name);
 
   json_builder_set_member_name (builder, PKG_TAG_FILE_TYPE);
-  json_builder_add_string_value (builder,
-				 pkg->type & PKG_FILE_TYPE_SOUND ?
-				 PKG_VAL_FILE_TYPE_SND : pkg->type &
-				 PKG_FILE_TYPE_PROJECT ? PKG_VAL_FILE_TYPE_PRJ
-				 : PKG_VAL_FILE_TYPE_UNK);
+  json_builder_add_string_value (builder, package_get_file_type (pkg->type));
 
-  if (pkg->type != PKG_FILE_TYPE_PRESET)
+  if (pkg->type != PKG_FILE_TYPE_RAW_PRESET)
     {
       json_builder_set_member_name (builder, PKG_TAG_FIRMWARE_VERSION);
       json_builder_add_string_value (builder, pkg->fw_version);
@@ -188,6 +291,26 @@ package_add_manifest (struct package *pkg)
 	}
     }
 
+  if (pkg->manifest->tags)	// PKG_FILE_TYPE_DATA_SOUND or PKG_FILE_TYPE_DATA_PRESET
+    {
+      json_builder_set_member_name (builder, PKG_TAG_METAINFO);
+      json_builder_begin_object (builder);
+      json_builder_set_member_name (builder, PKG_TAG_TAGS);
+      json_builder_begin_array (builder);
+
+      GSList *e = pkg->manifest->tags;
+
+      while (e)
+	{
+	  gchar *tag = (gchar *) e->data;
+	  json_builder_add_string_value (builder, tag);
+	  e = e->next;
+	}
+
+      json_builder_end_array (builder);
+      json_builder_end_object (builder);
+    }
+
   if (samples_found)
     {
       json_builder_set_member_name (builder, PKG_TAG_SAMPLES);
@@ -238,34 +361,38 @@ package_add_manifest (struct package *pkg)
 }
 
 gint
-package_end (struct package *pkg, GByteArray * out)
+package_end (struct package *pkg, struct idata *out)
 {
   int ret = 0;
   zip_stat_t zstat;
+  GByteArray *content;
 
   ret = package_add_manifest (pkg);
   if (ret)
     {
-      error_print ("Error while formatting %s\n", MANIFEST_FILENAME);
+      error_print ("Error while formatting %s", MANIFEST_FILENAME);
       return ret;
     }
 
-  debug_print (1, "Writing zip to buffer...\n");
+  debug_print (1, "Writing zip to buffer...");
   if (zip_close (pkg->zip))
     {
-      error_print ("Error while creating in memory zip: %s\n",
+      error_print ("Error while creating in memory zip: %s",
 		   zip_error_strerror (zip_get_error (pkg->zip)));
       return -1;
     }
 
   zip_source_stat (pkg->zip_source, &zstat);
-  debug_print (1, "%" PRIu64 " B written to package\n", zstat.comp_size);
+  debug_print (1, "%" PRIu64 " B written to package", zstat.comp_size);
 
   zip_source_open (pkg->zip_source);
-  g_byte_array_set_size (out, zstat.comp_size);
-  zip_source_read (pkg->zip_source, out->data, zstat.comp_size);
+  content = g_byte_array_sized_new (zstat.comp_size);
+  content->len = zstat.comp_size;
+  zip_source_read (pkg->zip_source, content->data, zstat.comp_size);
   zip_source_close (pkg->zip_source);
 
+  idata_init (out, content, NULL, NULL);
+
   return 0;
 }
 
@@ -288,22 +415,23 @@ package_destroy (struct package *pkg)
 }
 
 gint
-package_open (struct package *pkg, GByteArray * data,
+package_open (struct package *pkg, struct idata *idata,
 	      const struct device_desc *device_desc)
 {
-  gint ret;
+  gint err;
   zip_error_t zerror;
   zip_file_t *manifest_file;
   zip_stat_t zstat;
+  GByteArray *data = idata->content;
 
-  debug_print (1, "Opening zip stream...\n");
+  debug_print (1, "Opening zip stream...");
 
   zip_error_init (&zerror);
   pkg->zip_source = zip_source_buffer_create (data->data, data->len, 0,
 					      &zerror);
   if (!pkg->zip_source)
     {
-      error_print ("Error while creating zip source: %s\n",
+      error_print ("Error while creating zip source: %s",
 		   zip_error_strerror (&zerror));
       zip_error_fini (&zerror);
       return -1;
@@ -312,17 +440,17 @@ package_open (struct package *pkg, GByteArray * data,
   pkg->zip = zip_open_from_source (pkg->zip_source, ZIP_RDONLY, &zerror);
   if (!pkg->zip)
     {
-      error_print ("Error while creating in memory zip: %s\n",
+      error_print ("Error while creating in memory zip: %s",
 		   zip_error_strerror (&zerror));
       zip_error_fini (&zerror);
       zip_source_free (pkg->zip_source);
       return -1;
     }
 
-  ret = zip_stat (pkg->zip, MANIFEST_FILENAME, ZIP_FL_ENC_STRICT, &zstat);
-  if (ret)
+  err = zip_stat (pkg->zip, MANIFEST_FILENAME, ZIP_FL_ENC_STRICT, &zstat);
+  if (err)
     {
-      error_print ("Error while loading '%s': %s\n", MANIFEST_FILENAME,
+      error_print ("Error while loading '%s': %s", MANIFEST_FILENAME,
 		   zip_error_strerror (&zerror));
       zip_error_fini (&zerror);
       zip_source_free (pkg->zip_source);
@@ -346,7 +474,7 @@ package_open (struct package *pkg, GByteArray * data,
   pkg->fw_version = NULL;
   pkg->device_desc = device_desc;
 
-  return ret;
+  return 0;
 }
 
 void
@@ -356,13 +484,26 @@ package_close (struct package *pkg)
   package_destroy (pkg);
 }
 
+static gint
+packaget_get_pkg_sample_slots (struct backend *backend)
+{
+  struct elektron_data *data = backend->data;
+  gint slots = 127;		// slot 0 is never used
+  if (data->device_desc.id == 42)	// Digitakt II has 8 banks (1026)
+    {
+      slots *= 8;
+    }
+  return slots;
+}
+
 gint
 package_receive_pkg_resources (struct package *pkg,
-			       const gchar * payload_path,
+			       const gchar *payload_path,
 			       struct job_control *control,
 			       struct backend *backend,
 			       fs_remote_file_op download_data,
-			       fs_remote_file_op download_sample)
+			       fs_remote_file_op download_sample,
+			       enum package_type type)
 {
   gint ret, i, elements;
   JsonParser *parser;
@@ -371,19 +512,20 @@ package_receive_pkg_resources (struct package *pkg,
   GError *error;
   gchar *sample_path, *metadata_path;
   struct package_resource *pkg_resource;
-  GByteArray *wave, *payload, *metadata, *sample;
   GString *package_resource_path;
+  struct idata metadata_file, payload_file, sample_file, file;
 
-  metadata_path = path_chain (PATH_INTERNAL, payload_path, ".metadata");
-  debug_print (1, "Getting metadata from %s...\n", metadata_path);
-  metadata = g_byte_array_new ();
-  control->parts = 130;		// 128 sample slots, metadata and main.
+  metadata_path = path_chain (PATH_INTERNAL, payload_path,
+			      FS_DATA_METADATA_FILE);
+  debug_print (1, "Getting metadata from %s...", metadata_path);
+  control->parts = 2 + packaget_get_pkg_sample_slots (backend);	// main, metadata and sample slots.
   control->part = 0;
-  set_job_control_progress (control, 0.0);
-  ret = download_data (backend, metadata_path, metadata, control);
+  job_control_set_progress (control, 0.0);
+
+  ret = download_data (backend, metadata_path, &metadata_file, control);
   if (ret)
     {
-      debug_print (1, "Metadata file not available\n");
+      debug_print (1, "Metadata file not available");
       control->parts = 1;
       goto get_payload;
     }
@@ -391,14 +533,15 @@ package_receive_pkg_resources (struct package *pkg,
   control->part++;
 
   parser = json_parser_new ();
-  if (!json_parser_load_from_data
-      (parser, (gchar *) metadata->data, metadata->len, &error))
+  if (!json_parser_load_from_data (parser,
+				   (gchar *) metadata_file.content->data,
+				   metadata_file.content->len, &error))
     {
       error_print ("Unable to parse stream: %s. Continuing...",
 		   error->message);
       g_clear_error (&error);
       control->parts = 2;
-      goto get_payload;
+      goto cleanup_parser;
     }
 
   reader = json_reader_new (json_parser_get_root (parser));
@@ -406,19 +549,28 @@ package_receive_pkg_resources (struct package *pkg,
     {
       error_print ("Unable to read from parser. Continuing...");
       control->parts = 2;
-      goto get_payload;
+      goto cleanup_parser;
+    }
+
+  if (type == PKG_FILE_TYPE_DATA_SOUND || type == PKG_FILE_TYPE_DATA_PRESET)
+    {
+      pkg->manifest->tags = package_get_tags_from_snd_metadata_int (reader);
+    }
+  else
+    {
+      pkg->manifest->tags = NULL;
     }
 
   if (!json_reader_read_member (reader, MAN_TAG_SAMPLE_REFS))
     {
-      debug_print (1, "Member '%s' not found\n", MAN_TAG_SAMPLE_REFS);
+      debug_print (1, "Member '%s' not found", MAN_TAG_SAMPLE_REFS);
       control->parts = 2;
-      goto get_payload;
+      goto cleanup_reader;
     }
 
   if (!json_reader_is_array (reader))
     {
-      error_print ("Member '%s' is not an array. Continuing...\n",
+      error_print ("Member '%s' is not an array. Continuing...",
 		   MAN_TAG_SAMPLE_REFS);
       control->parts = 2;
       goto cleanup_reader;
@@ -427,24 +579,23 @@ package_receive_pkg_resources (struct package *pkg,
   elements = json_reader_count_elements (reader);
   if (!elements)
     {
-      debug_print (1, "No samples found\n");
+      debug_print (1, "No samples found");
       control->parts = 2;
       goto cleanup_reader;
     }
 
-  sample = g_byte_array_new ();
   control->parts = 2 + elements;
-  set_job_control_progress (control, 0.0);
+  job_control_set_progress (control, 0.0);
   for (i = 0; i < elements; i++, control->part++)
     {
       if (!json_reader_read_element (reader, i))
 	{
-	  error_print ("Cannot read element %d. Continuing...\n", i);
+	  error_print ("Cannot read element %d. Continuing...", i);
 	  continue;
 	}
       if (!json_reader_read_member (reader, MAN_TAG_HASH))
 	{
-	  error_print ("Cannot read member '%s'. Continuing...\n",
+	  error_print ("Cannot read member '%s'. Continuing...",
 		       MAN_TAG_HASH);
 	  continue;
 	}
@@ -453,7 +604,7 @@ package_receive_pkg_resources (struct package *pkg,
 
       if (!json_reader_read_member (reader, MAN_TAG_SIZE))
 	{
-	  error_print ("Cannot read member '%s'. Continuing...\n",
+	  error_print ("Cannot read member '%s'. Continuing...",
 		       MAN_TAG_SIZE);
 	  continue;
 	}
@@ -466,37 +617,34 @@ package_receive_pkg_resources (struct package *pkg,
 							     size);
       if (!sample_path)
 	{
-	  debug_print (1, "Sample not found. Skipping...\n");
+	  debug_print (1, "Sample not found. Skipping...");
 	  continue;
 	}
 
-      debug_print (1, "Hash: %" PRIu64 "; size: %" PRIu64 "; path: %s\n",
+      debug_print (1, "Hash: %" PRIu64 "; size: %" PRIu64 "; path: %s",
 		   hash, size, sample_path);
-      debug_print (1, "Getting sample %s...\n", sample_path);
-      g_byte_array_set_size (sample, 0);
-      if (download_sample (backend, sample_path, sample, control))
+      debug_print (1, "Getting sample %s...", sample_path);
+
+      if (download_sample (backend, sample_path, &sample_file, control))
 	{
 	  g_free (sample_path);
-	  error_print ("Error while downloading sample. Continuing...\n");
+	  error_print ("Error while downloading sample. Continuing...");
 	  continue;
 	}
 
-      wave = g_byte_array_new ();
-      ret = sample_get_audio_file_data_from_array (sample, wave, control,
-						   SF_FORMAT_WAV |
-						   SF_FORMAT_PCM_16);
+      ret = sample_get_memfile_from_sample (&sample_file, &file, control,
+					    SF_FORMAT_WAV | SF_FORMAT_PCM_16);
       if (ret)
 	{
 	  error_print
-	    ("Error while converting sample to wave file. Continuing...\n");
-	  g_byte_array_free (wave, TRUE);
+	    ("Error while converting sample to wave file. Continuing...");
 	  g_free (sample_path);
 	  continue;
 	}
 
       pkg_resource = g_malloc (sizeof (struct package_resource));
       pkg_resource->type = PKG_RES_TYPE_SAMPLE;
-      pkg_resource->data = wave;
+      pkg_resource->data = idata_steal (&file);
       pkg_resource->hash = hash;
       pkg_resource->size = size;
       package_resource_path = g_string_new (NULL);
@@ -506,30 +654,31 @@ package_receive_pkg_resources (struct package *pkg,
       if (package_add_resource (pkg, pkg_resource, TRUE))
 	{
 	  package_free_package_resource (pkg_resource);
-	  error_print ("Error while packaging sample\n");
+	  error_print ("Error while packaging sample");
 	  continue;
 	}
+
+      idata_free (&sample_file);
     }
 
-  g_byte_array_free (sample, TRUE);
 cleanup_reader:
   g_object_unref (reader);
+cleanup_parser:
   g_object_unref (parser);
 get_payload:
-  g_byte_array_free (metadata, TRUE);
-  debug_print (1, "Getting payload from %s...\n", payload_path);
-  payload = g_byte_array_new ();
-  ret = download_data (backend, payload_path, payload, control);
+  idata_free (&metadata_file);
+  debug_print (1, "Getting payload from %s...", payload_path);
+  ret = download_data (backend, payload_path, &payload_file, control);
   if (ret)
     {
-      error_print ("Error while downloading payload\n");
+      error_print ("Error while downloading payload");
       ret = -1;
     }
   else
     {
       pkg_resource = g_malloc (sizeof (struct package_resource));
       pkg_resource->type = PKG_RES_TYPE_PAYLOAD;
-      pkg_resource->data = payload;
+      pkg_resource->data = idata_steal (&payload_file);
       pkg_resource->path = strdup (pkg->name);
       if (package_add_resource (pkg, pkg_resource, TRUE))
 	{
@@ -541,7 +690,7 @@ get_payload:
 }
 
 gint
-package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
+package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
 			    struct job_control *control,
 			    struct backend *backend,
 			    fs_remote_file_op upload_data)
@@ -556,15 +705,15 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
   zip_stat_t zstat;
   zip_error_t zerror;
   zip_file_t *zip_file;
-  GByteArray *wave, *raw;
   struct package_resource *pkg_resource;
+  struct idata file, sample, sample_file;
 
   zip_error_init (&zerror);
 
   parser = json_parser_new ();
-  if (!json_parser_load_from_data
-      (parser, (gchar *) pkg->manifest->data->data, pkg->manifest->data->len,
-       &error))
+  if (!json_parser_load_from_data (parser,
+				   (gchar *) pkg->manifest->data->data,
+				   pkg->manifest->data->len, &error))
     {
       error_print ("Unable to parse stream: %s", error->message);
       g_clear_error (&error);
@@ -581,7 +730,7 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
 
   if (!json_reader_read_member (reader, PKG_TAG_PAYLOAD))
     {
-      error_print ("No '%s' found\n", PKG_TAG_PAYLOAD);
+      error_print ("No '%s' found", PKG_TAG_PAYLOAD);
       ret = -1;
       goto cleanup_reader;
     }
@@ -590,7 +739,7 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
 
   if (zip_stat (pkg->zip, pkg->name, ZIP_FL_ENC_STRICT, &zstat))
     {
-      error_print ("Error while loading '%s': %s\n", MANIFEST_FILENAME,
+      error_print ("Error while loading '%s': %s", MANIFEST_FILENAME,
 		   zip_error_strerror (&zerror));
       zip_error_fini (&zerror);
       ret = -1;
@@ -608,19 +757,20 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
 
   pkg->resources = g_list_append (pkg->resources, pkg_resource);
 
-  control->parts = 129;		// 128 sample slots and main.
+  control->parts = 1 + packaget_get_pkg_sample_slots (backend);	// main and sample slots
   control->part = 0;
-  ret = upload_data (backend, payload_path, pkg_resource->data, control);
+  idata_init (&file, pkg_resource->data, NULL, NULL);
+  ret = upload_data (backend, payload_path, &file, control);
   if (ret)
     {
-      error_print ("Error while uploading payload to '%s'\n", payload_path);
+      error_print ("Error while uploading payload to '%s'", payload_path);
       goto cleanup_reader;
     }
   control->part++;
 
   if (!json_reader_read_member (reader, PKG_TAG_FIRMWARE_VERSION))
     {
-      error_print ("No '%s' found\n", PKG_TAG_FIRMWARE_VERSION);
+      error_print ("No '%s' found", PKG_TAG_FIRMWARE_VERSION);
       ret = -1;
       goto cleanup_reader;
     }
@@ -629,7 +779,7 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
 
   if (!json_reader_read_member (reader, PKG_TAG_FILE_TYPE))
     {
-      error_print ("No '%s' found\n", PKG_TAG_FILE_TYPE);
+      error_print ("No '%s' found", PKG_TAG_FILE_TYPE);
       ret = -1;
       goto cleanup_reader;
     }
@@ -638,33 +788,37 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
 
   if (strcmp (file_type, PKG_VAL_FILE_TYPE_SND) == 0)
     {
-      pkg->type = PKG_FILE_TYPE_SOUND;
+      pkg->type = PKG_FILE_TYPE_DATA_SOUND;
     }
   else if (strcmp (file_type, PKG_VAL_FILE_TYPE_PRJ) == 0)
     {
-      pkg->type = PKG_FILE_TYPE_PROJECT;
+      pkg->type = PKG_FILE_TYPE_DATA_PROJECT;
+    }
+  else if (strcmp (file_type, PKG_VAL_FILE_TYPE_PST) == 0)
+    {
+      pkg->type = PKG_FILE_TYPE_DATA_PRESET;
     }
   else
     {
       pkg->type = PKG_FILE_TYPE_NONE;
-      debug_print (1, "Invalid '%s': %s\n", PKG_TAG_FILE_TYPE, file_type);
+      debug_print (1, "Invalid '%s': %s", PKG_TAG_FILE_TYPE, file_type);
     }
 
   if (!json_reader_read_member (reader, PKG_TAG_PRODUCT_TYPE))
     {
-      error_print ("No '%s' found\n", PKG_TAG_PRODUCT_TYPE);
+      error_print ("No '%s' found", PKG_TAG_PRODUCT_TYPE);
       ret = 0;
       goto cleanup_reader;
     }
   if (!json_reader_is_array (reader))
     {
-      error_print ("Member '%s' is not an array\n", PKG_TAG_PRODUCT_TYPE);
+      error_print ("Member '%s' is not an array", PKG_TAG_PRODUCT_TYPE);
       ret = -1;
       goto cleanup_reader;
     }
   if (!json_reader_count_elements (reader))
     {
-      error_print ("No product types found\n");
+      error_print ("No product types found");
       ret = 0;
       goto cleanup_reader;
     }
@@ -674,10 +828,10 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
       goto cleanup_reader;
     }
   product_type = atoi (json_reader_get_string_value (reader));
-  debug_print (1, "ProductType: %" PRId64 "\n", product_type);
+  debug_print (1, "ProductType: %" PRId64 "", product_type);
   if (pkg->device_desc->id != product_type)
     {
-      debug_print (1, "Incompatible product type. Continuing...\n");
+      debug_print (1, "Incompatible product type. Continuing...");
     }
   json_reader_end_element (reader);
   json_reader_end_element (reader);
@@ -686,28 +840,31 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
     {
       control->parts = 1;	// Only payload and it's done.
       control->part = 0;
-      set_job_control_progress (control, 1.0);
+      job_control_set_progress (control, 1.0);
       goto cleanup_reader;
     }
 
   if (!json_reader_is_array (reader))
     {
-      error_print ("Member '%s' is not an array. Skipping samples...\n",
+      error_print ("Member '%s' is not an array. Skipping samples...",
 		   PKG_TAG_SAMPLES);
       ret = -1;
       goto cleanup_reader;
     }
 
-  wave = g_byte_array_sized_new (zstat.size);
-  raw = g_byte_array_sized_new (MAX_PACKAGE_LEN);
+  //We are reusing the same sample_file. Let's be careful.
+  idata_init (&sample_file, g_byte_array_sized_new (zstat.size), NULL, NULL);
+
   elements = json_reader_count_elements (reader);
   control->parts = elements + 1;
   control->part = 1;
+
   for (i = 0; i < elements; i++, control->part++)
     {
-      struct sample_info sample_info_dst;
-      sample_info_dst.rate = ELEKTRON_SAMPLE_RATE;
-      sample_info_dst.channels = ELEKTRON_SAMPLE_CHANNELS;
+      struct sample_info sample_info_req, sample_info_src;
+      sample_info_req.rate = ELEKTRON_SAMPLE_RATE;
+      sample_info_req.channels = 0;	//Automatic
+      sample_info_req.format = SF_FORMAT_PCM_16;
 
       json_reader_read_element (reader, i);
       json_reader_read_member (reader, PKG_TAG_FILE_NAME);
@@ -715,57 +872,60 @@ package_send_pkg_resources (struct package *pkg, const gchar * payload_path,
       json_reader_end_element (reader);
       json_reader_end_element (reader);
 
+      debug_print (2, "Uploading %s...", sample_path);
+
       if (zip_stat (pkg->zip, sample_path, ZIP_FL_ENC_STRICT, &zstat))
 	{
-	  error_print ("Error while loading '%s': %s\n",
+	  error_print ("Error while loading '%s': %s",
 		       MANIFEST_FILENAME, zip_error_strerror (&zerror));
 	  zip_error_fini (&zerror);
 	  ret = -1;
 	  continue;
 	}
 
-      g_byte_array_set_size (wave, zstat.size);
-      zip_file = zip_fopen (pkg->zip, sample_path, 0);
-      zip_fread (zip_file, wave->data, zstat.size);
-      wave->len = zstat.size;
-      zip_fclose (zip_file);
-
-      raw->len = 0;
-      if (sample_load_from_array (wave, raw, control, &sample_info_dst))
-	{
-	  error_print ("Error while loading '%s': %s\n",
-		       sample_path, zip_error_strerror (&zerror));
-	  continue;
-	}
-
-      pkg_resource = g_malloc (sizeof (struct package_resource));
-      pkg_resource->type = PKG_RES_TYPE_SAMPLE;
-      pkg_resource->data = g_byte_array_sized_new (raw->len);
-      pkg_resource->data->len = raw->len;
-      memcpy (pkg_resource->data->data, raw->data, raw->len);
-      pkg_resource->path = strdup (sample_path);
-
-      pkg->resources = g_list_append (pkg->resources, pkg_resource);
-
       //We remove the "Samples" at the beggining of the full zip path...
       dev_sample_path = strdup (&sample_path[7]);
       //... And the extension.
-      remove_ext (dev_sample_path);
-      ret = elektron_upload_sample_part (backend, dev_sample_path,
-					 pkg_resource->data, control);
-      g_free (dev_sample_path);
-      g_free (control->data);
-      control->data = NULL;
-      if (ret)
+      filename_remove_ext (dev_sample_path);
+      sample_file.name = g_path_get_basename (dev_sample_path);
+
+      g_byte_array_set_size (sample_file.content, zstat.size);
+      zip_file = zip_fopen (pkg->zip, sample_path, 0);
+      zip_fread (zip_file, sample_file.content->data, zstat.size);
+      sample_file.content->len = zstat.size;
+      zip_fclose (zip_file);
+
+      if (sample_load_from_memfile (&sample_file, &sample, control,
+				    &sample_info_req, &sample_info_src))
 	{
-	  error_print ("Error while uploading sample to '%s'\n",
-		       &sample_path[7]);
-	  continue;
+	  error_print ("Error while loading '%s': %s",
+		       sample_path, zip_error_strerror (&zerror));
 	}
+      else
+	{
+	  ret = elektron_upload_sample_part (backend, dev_sample_path,
+					     &sample, control);
+
+	  if (ret)
+	    {
+	      error_print ("Error while uploading sample to '%s'",
+			   &sample_path[7]);
+	    }
+
+	  pkg_resource = g_malloc (sizeof (struct package_resource));
+	  pkg_resource->type = PKG_RES_TYPE_SAMPLE;
+	  pkg_resource->data = idata_steal (&sample);
+	  pkg_resource->path = strdup (sample_path);
+
+	  pkg->resources = g_list_append (pkg->resources, pkg_resource);
+	}
+
+      g_free (dev_sample_path);
+      g_free (sample_file.name);
+      sample_file.name = NULL;
     }
 
-  g_byte_array_free (wave, TRUE);
-  g_byte_array_free (raw, TRUE);
+  idata_free (&sample_file);
 
 cleanup_reader:
   g_object_unref (reader);
