@@ -20,13 +20,15 @@
 
 #include <glib.h>
 #include <zip.h>
-#include "backend.h"
+#include "connector.h"
 
 #ifndef PACKAGE_H
 #define PACKAGE_H
 
 #define ELEKTRON_SAMPLE_RATE 48000
-#define ELEKTRON_SAMPLE_CHANNELS 1
+
+#define FS_DATA_METADATA_EXT "metadata"
+#define FS_DATA_METADATA_FILE "." FS_DATA_METADATA_EXT
 
 enum package_resource_type
 {
@@ -43,14 +45,16 @@ struct package_resource
   guint32 size;
   gchar *path;
   GByteArray *data;
+  GSList *tags;			//Used for PKG_RES_TYPE_MANIFEST only
 };
 
 enum package_type
 {
   PKG_FILE_TYPE_NONE,
-  PKG_FILE_TYPE_SOUND,
-  PKG_FILE_TYPE_PROJECT,
-  PKG_FILE_TYPE_PRESET,
+  PKG_FILE_TYPE_DATA_SOUND,
+  PKG_FILE_TYPE_DATA_PROJECT,
+  PKG_FILE_TYPE_DATA_PRESET,
+  PKG_FILE_TYPE_RAW_PRESET
 };
 
 struct device_desc
@@ -60,6 +64,13 @@ struct device_desc
   gchar alias[LABEL_MAX];
 };
 
+struct elektron_data
+{
+  guint16 seq;
+  guint8 storage;
+  struct device_desc device_desc;
+};
+
 struct package
 {
   gchar *name;
@@ -73,24 +84,34 @@ struct package
   struct package_resource *manifest;
 };
 
-gint package_begin (struct package *, gchar *, const gchar *,
-		    const struct device_desc *, enum package_type);
+GSList *package_get_tags_from_snd_metadata (GByteArray * metadata);
 
-gint package_receive_pkg_resources (struct package *, const gchar *,
-				    struct job_control *, struct backend *,
-				    fs_remote_file_op, fs_remote_file_op);
+gint package_begin (struct package *pkg, gchar * name,
+		    const gchar * fw_version,
+		    const struct device_desc *device_desc,
+		    enum package_type type);
 
-gint package_end (struct package *, GByteArray *);
+gint package_receive_pkg_resources (struct package *pkg,
+				    const gchar * payload_path,
+				    struct job_control *control,
+				    struct backend *backend,
+				    fs_remote_file_op download_data,
+				    fs_remote_file_op download_sample,
+				    enum package_type type);
 
-void package_destroy (struct package *);
+gint package_end (struct package *pkg, struct idata *idata);
 
-gint package_open (struct package *, GByteArray *,
-		   const struct device_desc *);
+void package_destroy (struct package *pkg);
 
-gint package_send_pkg_resources (struct package *, const gchar *,
-				 struct job_control *, struct backend *,
-				 fs_remote_file_op);
+gint package_open (struct package *pkg, struct idata *idata,
+		   const struct device_desc *device_desc);
 
-void package_close (struct package *);
+gint package_send_pkg_resources (struct package *pkg,
+				 const gchar * payload_path,
+				 struct job_control *control,
+				 struct backend *backend,
+				 fs_remote_file_op upload_data);
+
+void package_close (struct package *pkg);
 
 #endif
