@@ -27,13 +27,13 @@
 #define PHATTY_MAX_PRESETS 100
 #define PHATTY_PROGRAM_SIZE 193
 
+#define PHATTY_PRESET_TYPE_OFFSET 3	// 0x04 is panel; 0x05 is preset
 #define PHATTY_PRESET_ID_OFFSET 5
 
 #define MOOG_NAME_LEN 13
 
 #define PHATTY_PRESETS_DIR "/presets"
 #define PHATTY_PANEL "panel"
-#define PHATTY_PANEL_PATH "/" PHATTY_PANEL
 #define PHATTY_PANEL_ID 0x100
 #define PHATTY_MAX_SCALES 32
 
@@ -54,12 +54,12 @@ struct phatty_iter_data
 
 enum phatty_fs
 {
-  FS_PHATTY_PRESET = 1,
-  FS_PHATTY_SCALE = 2
+  FS_PHATTY_PRESET,
+  FS_PHATTY_SCALE
 };
 
 static gchar
-phatty_decode_char (guint8 * data, gint position)
+phatty_decode_char (guint8 *data, gint position)
 {
   gint index;
   gint k = (3 * (position / 2)) + 23;
@@ -82,7 +82,7 @@ phatty_decode_char (guint8 * data, gint position)
 }
 
 static void
-phatty_encode_char (guint8 * data, gchar c, gint position)
+phatty_encode_char (guint8 *data, gchar c, gint position)
 {
   gchar *s = PHATTY_ALPHABET;
   gint k, index = 0;
@@ -114,7 +114,7 @@ phatty_encode_char (guint8 * data, gchar c, gint position)
 }
 
 void
-phatty_set_preset_name (guint8 * preset, const gchar * preset_name)
+phatty_set_preset_name (guint8 *preset, const gchar *preset_name)
 {
   gint i;
   const gchar *c = preset_name;
@@ -129,7 +129,7 @@ phatty_set_preset_name (guint8 * preset, const gchar * preset_name)
 }
 
 void
-phatty_get_preset_name (guint8 * preset, gchar * preset_name)
+phatty_get_preset_name (guint8 *preset, gchar *preset_name)
 {
   gchar *c = preset_name;
   for (gint i = 0; i < MOOG_NAME_LEN; i++, c++)
@@ -154,38 +154,18 @@ phatty_get_preset_name (guint8 * preset, gchar * preset_name)
 static gchar *
 phatty_get_download_path (struct backend *backend,
 			  const struct fs_operations *ops,
-			  const gchar * dst_dir, const gchar * src_path,
-			  GByteArray * preset)
+			  const gchar *dst_dir, const gchar *src_path,
+			  struct idata *preset)
 {
-  gchar preset_name[MOOG_NAME_LEN + 1];
-  gchar *path, *name;
-  gint id;
 
-  if (!preset)
+  guint id = 0;
+  if (common_slot_get_id_from_path (src_path, &id))
     {
       return NULL;
     }
-
-  name = g_path_get_basename (src_path);
-  id = atoi (name);
-  g_free (name);
-
-  if (id == PHATTY_PANEL_ID)
-    {
-      GString *str = g_string_new (NULL);
-      g_string_append_printf (str, "%s %s %s.%s", backend->name, ops->name,
-			      PHATTY_PANEL, ops->type_ext);
-      path = path_chain (PATH_SYSTEM, dst_dir, str->str);
-      g_string_free (str, TRUE);
-    }
-  else
-    {
-      phatty_get_preset_name (preset->data, preset_name);
-      path = common_get_download_path_with_params (backend, ops, dst_dir, id,
-						   2, preset_name);
-    }
-
-  return path;
+  guint digits = id == PHATTY_PANEL_ID ? 0 : 2;
+  return common_slot_get_download_path (backend, ops, dst_dir, src_path,
+					preset, digits);
 }
 
 static GByteArray *
@@ -216,17 +196,15 @@ phatty_next_root_dentry (struct item_iterator *iter)
     {
       snprintf (iter->item.name, LABEL_MAX, "%s", "presets");
       iter->item.id = 0x1000;
-      iter->item.type = ELEKTROID_DIR;
+      iter->item.type = ITEM_TYPE_DIR;
       iter->item.size = -1;
-      iter->item.slot_used = TRUE;
     }
   else if (*next == 1)
     {
-      snprintf (iter->item.name, LABEL_MAX, "%s", "panel");
+      snprintf (iter->item.name, LABEL_MAX, "%s", PHATTY_PANEL);
       iter->item.id = PHATTY_PANEL_ID;
-      iter->item.type = ELEKTROID_FILE;
+      iter->item.type = ITEM_TYPE_FILE;
       iter->item.size = -1;
-      iter->item.slot_used = TRUE;
     }
   else
     {
@@ -260,9 +238,8 @@ phatty_next_preset_dentry (struct item_iterator *iter)
   phatty_get_preset_name (rx_msg->data, preset_name);
   snprintf (iter->item.name, LABEL_MAX, "%s", preset_name);
   iter->item.id = data->next;
-  iter->item.type = ELEKTROID_FILE;
+  iter->item.type = ITEM_TYPE_FILE;
   iter->item.size = PHATTY_PROGRAM_SIZE;
-  iter->item.slot_used = TRUE;
   (data->next)++;
 
   free_msg (rx_msg);
@@ -271,27 +248,23 @@ phatty_next_preset_dentry (struct item_iterator *iter)
 
 static gint
 phatty_read_dir (struct backend *backend, struct item_iterator *iter,
-		 const gchar * path, const gchar ** extensions)
+		 const gchar *dir, GSList *exts)
 {
   gint err = 0;
 
-  if (!strcmp (path, "/"))
+  if (!strcmp (dir, "/"))
     {
       guint *id = g_malloc (sizeof (guint));
       *id = 0;
-      iter->data = id;
-      iter->next = phatty_next_root_dentry;
-      iter->free = g_free;
+      item_iterator_init (iter, dir, id, phatty_next_root_dentry, g_free);
     }
-  else if (!strcmp (path, PHATTY_PRESETS_DIR))
+  else if (!strcmp (dir, PHATTY_PRESETS_DIR))
     {
       struct phatty_iter_data *data =
 	g_malloc (sizeof (struct phatty_iter_data));
       data->next = 0;
       data->backend = backend;
-      iter->data = data;
-      iter->next = phatty_next_preset_dentry;
-      iter->free = g_free;
+      item_iterator_init (iter, dir, data, phatty_next_preset_dentry, g_free);
     }
   else
     {
@@ -301,6 +274,9 @@ phatty_read_dir (struct backend *backend, struct item_iterator *iter,
   return err;
 }
 
+//This ommits the slot id for the panel. As it is a made up number, this is better
+//than showing the ID column.
+
 gchar *
 phatty_get_id_as_slot (struct item *item, struct backend *backend)
 {
@@ -317,34 +293,40 @@ phatty_get_id_as_slot (struct item *item, struct backend *backend)
 }
 
 static gint
-phatty_download (struct backend *backend, const gchar * path,
-		 GByteArray * output, struct job_control *control)
+phatty_download (struct backend *backend, const gchar *path,
+		 struct idata *preset, struct job_control *control)
 {
-  guint8 id;
+  guint id;
   gint err = 0;
-  gchar *name;
+  gboolean panel;
   GByteArray *tx_msg, *rx_msg;
+  gchar name[MOOG_NAME_LEN + 1];
 
-  if (strcmp (path, PHATTY_PANEL_PATH))
+  err = common_slot_get_id_from_path (path, &id);
+  if (err)
+    {
+      return err;
+    }
+
+  if (id == PHATTY_PANEL_ID)
+    {
+      tx_msg = phatty_get_panel_dump_msg ();
+      panel = TRUE;
+    }
+  else
     {
-      name = g_path_get_basename (path);
-      id = atoi (name);
-      g_free (name);
       if (id >= PHATTY_MAX_PRESETS)
 	{
 	  return -EINVAL;
 	}
       tx_msg = phatty_get_preset_dump_msg (id);
-    }
-  else
-    {
-      tx_msg = phatty_get_panel_dump_msg ();
+      panel = FALSE;
     }
 
-  err = common_data_download (backend, tx_msg, &rx_msg, control);
+  err = common_data_tx_and_rx (backend, tx_msg, &rx_msg, control);
   if (err)
     {
-      goto end;
+      return err;
     }
   if (rx_msg->len != PHATTY_PROGRAM_SIZE)
     {
@@ -352,88 +334,99 @@ phatty_download (struct backend *backend, const gchar * path,
       goto cleanup;
     }
 
-  g_byte_array_append (output, rx_msg->data, rx_msg->len);
+  if (!panel)
+    {
+      phatty_get_preset_name (rx_msg->data, name);
+    }
+  idata_init (preset, rx_msg, strdup (panel ? PHATTY_PANEL : name), NULL);
+  return 0;
 
 cleanup:
   free_msg (rx_msg);
-end:
   return err;
 }
 
 static gint
-phatty_upload (struct backend *backend, const gchar * path,
-	       GByteArray * input, struct job_control *control)
+phatty_upload (struct backend *backend, const gchar *path,
+	       struct idata *preset, struct job_control *control)
 {
-  guint id;
-
-  if (input->len != PHATTY_PROGRAM_SIZE)
-    {
-      return -EINVAL;
-    }
-
-  if (common_slot_get_id_name_from_path (path, &id, NULL))
-    {
-      return -EINVAL;
-    }
-
-  if (id >= PHATTY_MAX_PRESETS && id != PHATTY_PANEL_ID)
-    {
-      return -EINVAL;
-    }
-
-  if (!strcmp (path, PHATTY_PANEL_PATH))
-    {
-      return -EINVAL;
-    }
-
-  input->data[PHATTY_PRESET_ID_OFFSET] = id;
-  return common_data_upload (backend, input, control);
-}
-
-static gint
-phatty_rename (struct backend *backend, const gchar * src, const gchar * dst)
-{
-  guint id;
   gint err;
-  struct job_control control;
-  struct sysex_transfer transfer;
+  guint id;
 
-  debug_print (1, "Renaming preset...\n");
-  err = common_slot_get_id_name_from_path (src, &id, NULL);
+  if (preset->content->len != PHATTY_PROGRAM_SIZE)
+    {
+      return -EINVAL;
+    }
+
+  err = common_slot_get_id_from_path (path, &id);
+  if (err)
+    {
+      return err;
+    }
+
+  if (id == PHATTY_PANEL_ID)
+    {
+      preset->content->data[PHATTY_PRESET_TYPE_OFFSET] = 0x04;
+      preset->content->data[PHATTY_PRESET_ID_OFFSET] = 0x01;
+    }
+  else
+    {
+      if (id >= PHATTY_MAX_PRESETS)
+	{
+	  return -EINVAL;
+	}
+
+      preset->content->data[PHATTY_PRESET_TYPE_OFFSET] = 0x05;
+      preset->content->data[PHATTY_PRESET_ID_OFFSET] = id;
+    }
+
+  return common_data_tx (backend, preset->content, control);
+}
+
+static gint
+phatty_rename (struct backend *backend, const gchar *src, const gchar *dst)
+{
+  guint id;
+  gint err;
+  struct job_control control;
+  struct sysex_transfer transfer;
+  struct idata preset;
+
+  debug_print (1, "Renaming preset...");
+  err = common_slot_get_id_from_path (src, &id);
   if (err)
     {
       return err;
     }
 
-  transfer.raw = g_byte_array_new ();
   //The control initialization is needed.
   control.active = TRUE;
   control.callback = NULL;
   g_mutex_init (&control.mutex);
-  err = phatty_download (backend, src, transfer.raw, &control);
+  err = phatty_download (backend, src, &preset, &control);
   if (err)
     {
       goto end;
     }
 
-  phatty_set_preset_name (transfer.raw->data, dst);
+  phatty_set_preset_name (preset.content->data, dst);
+  transfer.raw = preset.content;
   err = backend_tx_sysex (backend, &transfer);
+  idata_free (&preset);
 
 end:
-  free_msg (transfer.raw);
   return err;
 }
 
 static const struct fs_operations FS_PHATTY_PRESET_OPERATIONS = {
-  .fs = FS_PHATTY_PRESET,
-  .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
-    FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID |
+  .id = FS_PHATTY_PRESET,
+  .options = FS_OPTION_SINGLE_OP | FS_OPTION_SLOT_STORAGE |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN |
     FS_OPTION_ALLOW_SEARCH,
   .name = "preset",
   .gui_name = "Presets",
-  .gui_icon = BE_FILE_ICON_SND,
-  .type_ext = "syx",
+  .gui_icon = FS_ICON_SND,
+  .ext = BE_SYSEX_EXT,
   .max_name_len = MOOG_NAME_LEN,
   .readdir = phatty_read_dir,
   .print_item = common_print_item,
@@ -441,9 +434,8 @@ static const struct fs_operations FS_PHATTY_PRESET_OPERATIONS = {
   .download = phatty_download,
   .upload = phatty_upload,
   .get_slot = phatty_get_id_as_slot,
-  .load = load_file,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
+  .load = file_load,
+  .save = file_save,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = phatty_get_download_path,
   .select_item = common_midi_program_change
@@ -451,32 +443,32 @@ static const struct fs_operations FS_PHATTY_PRESET_OPERATIONS = {
 
 static gint
 phatty_scale_read_dir (struct backend *backend, struct item_iterator *iter,
-		       const gchar * path, const gchar ** extensions)
+		       const gchar *dir, GSList *extensions)
 {
   struct common_simple_read_dir_data *data;
 
-  if (strcmp (path, "/"))
+  if (strcmp (dir, "/"))
     {
       return -ENOTDIR;
     }
 
   data = g_malloc (sizeof (struct common_simple_read_dir_data));
   data->next = 0;
-  data->max = PHATTY_MAX_SCALES;
-  iter->data = data;
-  iter->next = common_simple_next_dentry;
-  iter->free = g_free;
+  data->last = PHATTY_MAX_SCALES - 1;
+
+  item_iterator_init (iter, dir, data, common_simple_next_dentry, g_free);
 
   return 0;
 }
 
 static gint
-phatty_scale_upload (struct backend *backend, const gchar * path,
-		     GByteArray * input, struct job_control *control)
+phatty_scale_upload (struct backend *backend, const gchar *path,
+		     struct idata *scale, struct job_control *control)
 {
   guint id;
+  GByteArray *input = scale->content;
 
-  if (common_slot_get_id_name_from_path (path, &id, NULL))
+  if (common_slot_get_id_from_path (path, &id))
     {
       return -EINVAL;
     }
@@ -484,34 +476,26 @@ phatty_scale_upload (struct backend *backend, const gchar * path,
   input->data[5] = 0;		//bank
   input->data[6] = id;		//scale
 
-  return common_data_upload (backend, input, control);
+  return common_data_tx (backend, input, control);
 }
 
 static const struct fs_operations FS_PHATTY_SCALE_OPERATIONS = {
-  .fs = FS_PHATTY_SCALE,
-  .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
-    FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
+  .id = FS_PHATTY_SCALE,
+  .options = FS_OPTION_SINGLE_OP | FS_OPTION_SLOT_STORAGE,
   .name = "scale",
   .gui_name = "Scales",
-  .gui_icon = BE_FILE_ICON_SND,
-  .type_ext = "scl",
+  .gui_icon = FS_ICON_KEYS,
+  .ext = "scl",
   .readdir = phatty_scale_read_dir,
   .print_item = common_print_item,
   .upload = phatty_scale_upload,
-  .load = scl_get_2_byte_octave_tuning_msg_from_scala_file,
-  .get_ext = backend_get_fs_ext,
+  .load = scl_load_2_byte_octave_tuning_msg_from_scala_file,
   .get_upload_path = common_slot_get_upload_path
 };
 
-static const struct fs_operations *FS_PHATTY_OPERATIONS_LIST[] = {
-  &FS_PHATTY_PRESET_OPERATIONS, &FS_PHATTY_SCALE_OPERATIONS, NULL
-};
-
-gint
+static gint
 phatty_handshake (struct backend *backend)
 {
-  backend_midi_handshake (backend);
-
   if (memcmp (backend->midi_info.company, MOOG_ID, sizeof (MOOG_ID)) ||
       memcmp (backend->midi_info.family, FAMILY_ID, sizeof (FAMILY_ID)) ||
       memcmp (backend->midi_info.model, MODEL_ID, sizeof (MODEL_ID)))
@@ -519,9 +503,16 @@ phatty_handshake (struct backend *backend)
       return -ENODEV;
     }
 
-  backend->filesystems = FS_PHATTY_PRESET | FS_PHATTY_SCALE;
-  backend->fs_ops = FS_PHATTY_OPERATIONS_LIST;
+  gslist_fill (&backend->fs_ops, &FS_PHATTY_PRESET_OPERATIONS,
+	       &FS_PHATTY_SCALE_OPERATIONS, NULL);
   snprintf (backend->name, LABEL_MAX, "Moog Little Phatty");
 
   return 0;
 }
+
+const struct connector CONNECTOR_PHATTY = {
+  .name = "phatty",
+  .handshake = phatty_handshake,
+  .standard = TRUE,
+  .regex = ".*Phatty.*"
+};
