@@ -29,7 +29,7 @@
 #define SUMMIT_REST_TIME_US 10000
 #define SUMMIT_MSG_BANK_POS 12
 #define SUMMIT_MSG_PATCH_POS 13
-#define SUMMIT_MAX_TUNINGS 17	// Tuning 0 is stored but can't be changed form the UI.
+#define SUMMIT_MAX_TUNINGS 17	// Tuning 0 is stored but can't be changed from the UI.
 #define SUMMIT_MAX_WAVETABLES 10
 #define SUMMIT_WAVETABLE_NAME_LEN 7	//In the device there are 8 available characters in the wavetable names but the wavetable messages only contain the first 7 characters.
 #define SUMMIT_WAVETABLE_HEADER_LEN 23
@@ -42,6 +42,9 @@
 #define SUMMIT_GET_NAME_FROM_MSG(msg, type) (&msg->data[type == FS_SUMMIT_SINGLE_PATCH ? 0x10 : 0x19b])
 #define SUMMIT_GET_BANK_ID_FROM_DIR(dir) ((guint8) dir[1] - 0x40)	// Bank A is the bank 1.
 
+#define SUMMIT_ALPHABET " ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!\"#$%&'()*+,-./:;<=>?@[\\]^_`{|}"
+#define SUMMIT_DEFAULT_CHAR '?'
+
 static const guint8 NOVATION_ID[] = { 0x0, 0x20, 0x29 };
 static const guint8 SUMMIT_ID[] = { 0x33, 1, 0, 0 };
 
@@ -53,11 +56,10 @@ static const guint8 SUMMIT_BULK_TUNING_REQ[] =
 
 enum summit_fs
 {
-  FS_SUMMIT_SINGLE_PATCH = 1,
-  FS_SUMMIT_MULTI_PATCH = 2,
-  FS_SUMMIT_WAVETABLE = 4,
-  FS_SUMMIT_SCALE = 8,
-  FS_SUMMIT_BULK_TUNING = 0x10
+  FS_SUMMIT_SINGLE_PATCH,
+  FS_SUMMIT_MULTI_PATCH,
+  FS_SUMMIT_WAVETABLE,
+  FS_SUMMIT_BULK_TUNING
 };
 
 struct summit_bank_iterator_data
@@ -75,7 +77,7 @@ struct summit_wavetable_iterator_data
 };
 
 static gint
-summit_set_patch_bank_and_id (GByteArray * msg, guint8 bank, guint8 id)
+summit_set_patch_bank_and_id (GByteArray *msg, guint8 bank, guint8 id)
 {
   if (msg->len <= SUMMIT_MSG_PATCH_POS)
     {
@@ -98,8 +100,10 @@ summit_get_patch_dump_msg (gint bank, gint id, enum summit_fs fs)
   return tx_msg;
 }
 
+//This function truncates the name to the last useful char ignoring the trailing spaces.
+
 static void
-summit_truncate_name_at_last_useful_char (gchar * c)
+summit_truncate_name (gchar *c)
 {
   for (int i = SUMMIT_PATCH_NAME_LEN - 1; i >= 0; i--, c--)
     {
@@ -114,34 +118,44 @@ summit_truncate_name_at_last_useful_char (gchar * c)
     }
 }
 
-static gchar *
-summit_get_patch_download_path (struct backend *backend,
-				const struct fs_operations *ops,
-				const gchar * dst_dir, const gchar * src_path,
-				GByteArray * patch)
+static const gchar *
+summit_get_category_name (GByteArray *rx_msg)
 {
-  guint id;
-  gchar *path;
-  gchar name[SUMMIT_PATCH_NAME_LEN + 1];
-
-  if (!patch)
+  switch (rx_msg->data[32])
     {
-      return NULL;
+    case 0:
+      return "None";
+    case 1:
+      return "Arp";
+    case 2:
+      return "Bass";
+    case 3:
+      return "Bell";
+    case 4:
+      return "Classic";
+    case 5:
+      return "DrumPerc";
+    case 6:
+      return "Keyboard";
+    case 7:
+      return "Lead";
+    case 8:
+      return "Motion";
+    case 9:
+      return "Pad";
+    case 10:
+      return "Poly";
+    case 11:
+      return "SFX";
+    case 12:
+      return "String";
+    case 13:
+      return "User 1";
+    case 14:
+      return "User 2";
+    default:
+      return "";
     }
-
-  if (common_slot_get_id_name_from_path (src_path, &id, NULL))
-    {
-      return NULL;
-    }
-
-  memcpy (name, SUMMIT_GET_NAME_FROM_MSG (patch, ops->fs),
-	  SUMMIT_PATCH_NAME_LEN);
-  name[SUMMIT_PATCH_NAME_LEN] = 0;
-  summit_truncate_name_at_last_useful_char (&name[SUMMIT_PATCH_NAME_LEN - 1]);
-  path = common_get_download_path_with_params (backend, ops, dst_dir, id, 3,
-					       name);
-
-  return path;
 }
 
 static gint
@@ -166,14 +180,18 @@ summit_patch_next_dentry (struct item_iterator *iter)
 	  SUMMIT_PATCH_NAME_LEN);
   iter->item.name[SUMMIT_PATCH_NAME_LEN] = 0;
   gchar *c = &iter->item.name[SUMMIT_PATCH_NAME_LEN - 1];
-  summit_truncate_name_at_last_useful_char (c);
+  summit_truncate_name (c);
+  if (data->fs == FS_SUMMIT_SINGLE_PATCH)
+    {
+      const gchar *category = summit_get_category_name (rx_msg);
+      snprintf (iter->item.object_info, LABEL_MAX, "%s", category);
+    }
   free_msg (rx_msg);
 
   iter->item.id = data->next;
-  iter->item.type = ELEKTROID_FILE;
+  iter->item.type = ITEM_TYPE_FILE;
   iter->item.size =
     data->fs == FS_SUMMIT_SINGLE_PATCH ? SUMMIT_SINGLE_LEN : SUMMIT_MULTI_LEN;
-  iter->item.slot_used = TRUE;
   data->next++;
 
   usleep (SUMMIT_REST_TIME_US);
@@ -190,9 +208,8 @@ summit_patch_next_dentry_root (struct item_iterator *iter)
     {
       iter->item.id = 0x10000 + *next;	//Unique id
       snprintf (iter->item.name, LABEL_MAX, "%c", 0x41 + iter->item.id);
-      iter->item.type = ELEKTROID_DIR;
+      iter->item.type = ITEM_TYPE_DIR;
       iter->item.size = -1;
-      iter->item.slot_used = TRUE;
       (*next)++;
       return 0;
     }
@@ -202,22 +219,21 @@ summit_patch_next_dentry_root (struct item_iterator *iter)
 
 static gint
 summit_patch_read_dir (struct backend *backend, struct item_iterator *iter,
-		       const gchar * path, enum summit_fs fs)
+		       const gchar *dir, enum summit_fs fs)
 {
   guint bank;
 
-  if (!strcmp (path, "/"))
+  if (!strcmp (dir, "/"))
     {
       guint *next = g_malloc (sizeof (guint));
       *next = 0;
-      iter->data = next;
-      iter->next = summit_patch_next_dentry_root;
-      iter->free = g_free;
+      item_iterator_init (iter, dir, next, summit_patch_next_dentry_root,
+			  g_free);
       return 0;
     }
 
-  bank = SUMMIT_GET_BANK_ID_FROM_DIR (path);
-  if (strlen (path) == 2 && bank >= 1 && bank <= 4)
+  bank = SUMMIT_GET_BANK_ID_FROM_DIR (dir);
+  if (strlen (dir) == 2 && bank >= 1 && bank <= 4)
     {
       struct summit_bank_iterator_data *data =
 	g_malloc (sizeof (struct summit_bank_iterator_data));
@@ -225,9 +241,7 @@ summit_patch_read_dir (struct backend *backend, struct item_iterator *iter,
       data->fs = fs;
       data->bank = bank;
       data->backend = backend;
-      iter->data = data;
-      iter->next = summit_patch_next_dentry;
-      iter->free = g_free;
+      item_iterator_init (iter, dir, data, summit_patch_next_dentry, g_free);
       return 0;
     }
 
@@ -236,21 +250,20 @@ summit_patch_read_dir (struct backend *backend, struct item_iterator *iter,
 
 static gint
 summit_single_read_dir (struct backend *backend, struct item_iterator *iter,
-			const gchar * path, const gchar ** extensions)
+			const gchar *dir, GSList *extensions)
 {
-  return summit_patch_read_dir (backend, iter, path, FS_SUMMIT_SINGLE_PATCH);
+  return summit_patch_read_dir (backend, iter, dir, FS_SUMMIT_SINGLE_PATCH);
 }
 
 static gint
 summit_multi_read_dir (struct backend *backend, struct item_iterator *iter,
-		       const gchar * path, const gchar ** extensions)
+		       const gchar *path, GSList *extensions)
 {
   return summit_patch_read_dir (backend, iter, path, FS_SUMMIT_MULTI_PATCH);
 }
 
 static guint
-summit_get_bank_and_id_from_path (const gchar * path, guint8 * bank,
-				  guint8 * id)
+summit_get_bank_and_id_from_path (const gchar *path, guint8 *bank, guint8 *id)
 {
   if (strlen (path) < 4)
     {
@@ -269,13 +282,14 @@ summit_get_bank_and_id_from_path (const gchar * path, guint8 * bank,
 }
 
 static gint
-summit_patch_download (struct backend *backend, const gchar * path,
-		       GByteArray * output, struct job_control *control,
+summit_patch_download (struct backend *backend, const gchar *path,
+		       struct idata *patch, struct job_control *control,
 		       enum summit_fs fs)
 {
   guint8 id, bank;
   gint len, err;
   GByteArray *tx_msg, *rx_msg;
+  gchar name[SUMMIT_PATCH_NAME_LEN + 1];
 
   err = summit_get_bank_and_id_from_path (path, &bank, &id);
   if (err)
@@ -284,7 +298,7 @@ summit_patch_download (struct backend *backend, const gchar * path,
     }
 
   tx_msg = summit_get_patch_dump_msg (bank, id, fs);
-  err = common_data_download (backend, tx_msg, &rx_msg, control);
+  err = common_data_tx_and_rx (backend, tx_msg, &rx_msg, control);
   if (err)
     {
       goto end;
@@ -296,7 +310,12 @@ summit_patch_download (struct backend *backend, const gchar * path,
       goto cleanup;
     }
 
-  g_byte_array_append (output, rx_msg->data, rx_msg->len);
+  memcpy (name, SUMMIT_GET_NAME_FROM_MSG (rx_msg, fs), SUMMIT_PATCH_NAME_LEN);
+  name[SUMMIT_PATCH_NAME_LEN] = 0;
+  summit_truncate_name (&name[SUMMIT_PATCH_NAME_LEN - 1]);
+
+  idata_init (patch, rx_msg, strdup (name), NULL);
+  goto end;
 
 cleanup:
   free_msg (rx_msg);
@@ -306,24 +325,24 @@ end:
 }
 
 static gint
-summit_single_download (struct backend *backend, const gchar * path,
-			GByteArray * output, struct job_control *control)
+summit_single_download (struct backend *backend, const gchar *path,
+			struct idata *patch, struct job_control *control)
 {
-  return summit_patch_download (backend, path, output, control,
+  return summit_patch_download (backend, path, patch, control,
 				FS_SUMMIT_SINGLE_PATCH);
 }
 
 static gint
-summit_multi_download (struct backend *backend, const gchar * path,
-		       GByteArray * output, struct job_control *control)
+summit_multi_download (struct backend *backend, const gchar *path,
+		       struct idata *patch, struct job_control *control)
 {
-  return summit_patch_download (backend, path, output, control,
+  return summit_patch_download (backend, path, patch, control,
 				FS_SUMMIT_MULTI_PATCH);
 }
 
 static gint
-summit_patch_upload (struct backend *backend, const gchar * path,
-		     GByteArray * input, struct job_control *control)
+summit_patch_upload (struct backend *backend, const gchar *path,
+		     GByteArray *input, struct job_control *control)
 {
   guint8 id, bank;
   gint err;
@@ -344,7 +363,7 @@ summit_patch_upload (struct backend *backend, const gchar * path,
       goto cleanup;
     }
 
-  err = common_data_upload (backend, msg, control);
+  err = common_data_tx (backend, msg, control);
 
 cleanup:
   free_msg (msg);
@@ -354,60 +373,64 @@ end:
 }
 
 static gint
-summit_single_upload (struct backend *backend, const gchar * path,
-		      GByteArray * input, struct job_control *control)
+summit_single_upload (struct backend *backend, const gchar *path,
+		      struct idata *patch, struct job_control *control)
 {
-  if (input->len != SUMMIT_SINGLE_LEN)
+  if (patch->content->len != SUMMIT_SINGLE_LEN)
     {
       return -EINVAL;
     }
 
-  return summit_patch_upload (backend, path, input, control);
+  return summit_patch_upload (backend, path, patch->content, control);
 }
 
 static gint
-summit_multi_upload (struct backend *backend, const gchar * path,
-		     GByteArray * input, struct job_control *control)
+summit_multi_upload (struct backend *backend, const gchar *path,
+		     struct idata *patch, struct job_control *control)
 {
-  if (input->len != SUMMIT_MULTI_LEN)
+  if (patch->content->len != SUMMIT_MULTI_LEN)
     {
       return -EINVAL;
     }
 
-  return summit_patch_upload (backend, path, input, control);
+  return summit_patch_upload (backend, path, patch->content, control);
 }
 
 static gint
-summit_patch_rename (struct backend *backend, const gchar * src,
-		     const gchar * dst, enum summit_fs fs)
+summit_patch_rename (struct backend *backend, const gchar *src,
+		     const gchar *dst, enum summit_fs fs)
 {
-  GByteArray *preset, *rx_msg;
+  struct idata preset;
+  GByteArray *rx_msg;
   gint err, len;
   guint8 *name;
+  gchar *sanitized;
   struct job_control control;
-  debug_print (1, "Renaming from %s to %s...\n", src, dst);
+  debug_print (1, "Renaming from %s to %s...", src, dst);
 
   //The control initialization is needed.
   control.active = TRUE;
   control.callback = NULL;
   g_mutex_init (&control.mutex);
 
-  preset = g_byte_array_sized_new (1024);
-  err = summit_patch_download (backend, src, preset, &control, fs);
+  err = summit_patch_download (backend, src, &preset, &control, fs);
   if (err)
     {
-      free_msg (preset);
       return err;
     }
 
   usleep (SUMMIT_REST_TIME_US);
 
-  len = strlen (dst);
-  name = SUMMIT_GET_NAME_FROM_MSG (preset, fs);
-  memcpy (name, dst, len);
+  name = SUMMIT_GET_NAME_FROM_MSG (preset.content, fs);
+  sanitized = common_get_sanitized_name (dst, SUMMIT_ALPHABET,
+					 SUMMIT_DEFAULT_CHAR);
+  len = strlen (sanitized);
+  len = len > SUMMIT_PATCH_NAME_LEN ? SUMMIT_PATCH_NAME_LEN : len;
+  memcpy (name, sanitized, len);
+  g_free (sanitized);
   memset (name + len, ' ', SUMMIT_PATCH_NAME_LEN - len);
 
-  rx_msg = backend_tx_and_rx_sysex (backend, preset, 100);	//There must be no response.
+  rx_msg = backend_tx_and_rx_sysex (backend, preset.content, 100);	//There must be no response.
   if (rx_msg)
     {
       err = -EIO;
@@ -420,15 +443,15 @@ summit_patch_rename (struct backend *backend, const gchar * src,
 }
 
 static gint
-summit_single_rename (struct backend *backend, const gchar * src,
-		      const gchar * dst)
+summit_single_rename (struct backend *backend, const gchar *src,
+		      const gchar *dst)
 {
   return summit_patch_rename (backend, src, dst, FS_SUMMIT_SINGLE_PATCH);
 }
 
 static gint
-summit_multi_rename (struct backend *backend, const gchar * src,
-		     const gchar * dst)
+summit_multi_rename (struct backend *backend, const gchar *src,
+		     const gchar *dst)
 {
   return summit_patch_rename (backend, src, dst, FS_SUMMIT_MULTI_PATCH);
 }
@@ -457,7 +480,7 @@ summit_get_patch_id_as_slot (struct item *item, struct backend *backend)
 
 static void
 summit_common_patch_change (struct backend *backend, guint8 type,
-			    const gchar * dir, struct item *item)
+			    const gchar *dir, struct item *item)
 {
   guint8 msg[3];
 
@@ -482,29 +505,28 @@ summit_common_patch_change (struct backend *backend, guint8 type,
 }
 
 static void
-summit_single_patch_change (struct backend *backend, const gchar * dir,
+summit_single_patch_change (struct backend *backend, const gchar *dir,
 			    struct item *item)
 {
   summit_common_patch_change (backend, 0, dir, item);
 }
 
 static void
-summit_multi_patch_change (struct backend *backend, const gchar * dir,
+summit_multi_patch_change (struct backend *backend, const gchar *dir,
 			   struct item *item)
 {
   summit_common_patch_change (backend, 1, dir, item);
 }
 
 static const struct fs_operations FS_SUMMIT_SINGLE_OPERATIONS = {
-  .fs = FS_SUMMIT_SINGLE_PATCH,
-  .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
-    FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID |
+  .id = FS_SUMMIT_SINGLE_PATCH,
+  .options = FS_OPTION_SINGLE_OP | FS_OPTION_SLOT_STORAGE |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN |
-    FS_OPTION_ALLOW_SEARCH,
+    FS_OPTION_SHOW_INFO_COLUMN | FS_OPTION_ALLOW_SEARCH,
   .name = "single",
   .gui_name = "Single",
-  .gui_icon = BE_FILE_ICON_SND,
-  .type_ext = "syx",
+  .gui_icon = FS_ICON_SND,
+  .ext = BE_SYSEX_EXT,
   .max_name_len = SUMMIT_PATCH_NAME_LEN,
   .readdir = summit_single_read_dir,
   .print_item = common_print_item,
@@ -512,24 +534,22 @@ static const struct fs_operations FS_SUMMIT_SINGLE_OPERATIONS = {
   .download = summit_single_download,
   .upload = summit_single_upload,
   .get_slot = summit_get_patch_id_as_slot,
-  .load = load_file,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
+  .load = file_load,
+  .save = file_save,
   .get_upload_path = common_slot_get_upload_path,
-  .get_download_path = summit_get_patch_download_path,
+  .get_download_path = common_slot_get_download_path_nnn,
   .select_item = summit_single_patch_change
 };
 
 static const struct fs_operations FS_SUMMIT_MULTI_OPERATIONS = {
-  .fs = FS_SUMMIT_MULTI_PATCH,
-  .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
-    FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID |
+  .id = FS_SUMMIT_MULTI_PATCH,
+  .options = FS_OPTION_SINGLE_OP | FS_OPTION_SLOT_STORAGE |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN |
     FS_OPTION_ALLOW_SEARCH,
   .name = "multi",
   .gui_name = "Multi",
-  .gui_icon = BE_FILE_ICON_SND,
-  .type_ext = "syx",
+  .gui_icon = FS_ICON_SND,
+  .ext = BE_SYSEX_EXT,
   .max_name_len = SUMMIT_PATCH_NAME_LEN,
   .readdir = summit_multi_read_dir,
   .print_item = common_print_item,
@@ -537,42 +557,41 @@ static const struct fs_operations FS_SUMMIT_MULTI_OPERATIONS = {
   .download = summit_multi_download,
   .upload = summit_multi_upload,
   .get_slot = summit_get_patch_id_as_slot,
-  .load = load_file,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
+  .load = file_load,
+  .save = file_save,
   .get_upload_path = common_slot_get_upload_path,
-  .get_download_path = summit_get_patch_download_path,
+  .get_download_path = common_slot_get_download_path_nnn,
   .select_item = summit_multi_patch_change
 };
 
 static gint
-summit_scale_read_dir (struct backend *backend, struct item_iterator *iter,
-		       const gchar * path, const gchar ** extensions)
+summit_tuning_read_dir (struct backend *backend, struct item_iterator *iter,
+			const gchar *dir, GSList *extensions)
 {
   struct common_simple_read_dir_data *data;
 
-  if (strcmp (path, "/"))
+  if (strcmp (dir, "/"))
     {
       return -ENOTDIR;
     }
 
   data = g_malloc (sizeof (struct common_simple_read_dir_data));
   data->next = 0;
-  data->max = SUMMIT_MAX_TUNINGS;
-  iter->data = data;
-  iter->next = common_simple_next_dentry;
-  iter->free = g_free;
+  data->last = SUMMIT_MAX_TUNINGS - 1;
+
+  item_iterator_init (iter, dir, data, common_simple_next_dentry, g_free);
 
   return 0;
 }
 
 static gint
-summit_tuning_upload (struct backend *backend, const gchar * path,
-		      GByteArray * input, struct job_control *control)
+summit_tuning_upload (struct backend *backend, const gchar *path,
+		      struct idata *tuning, struct job_control *control)
 {
   guint id;
+  GByteArray *input = tuning->content;
 
-  if (common_slot_get_id_name_from_path (path, &id, NULL))
+  if (common_slot_get_id_from_path (path, &id))
     {
       return -EINVAL;
     }
@@ -589,34 +608,18 @@ summit_tuning_upload (struct backend *backend, const gchar * path,
   input->data[2] = 0;		//0x7f does not work with the Summit.
   input->data[5] = id;		//tuning
 
-  return common_data_upload (backend, input, control);
+  return common_data_tx (backend, input, control);
 }
 
-static const struct fs_operations FS_SUMMIT_SCALE_OPERATIONS = {
-  .fs = FS_SUMMIT_SCALE,
-  .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
-    FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
-  .name = "scale",
-  .gui_name = "Scales",
-  .gui_icon = BE_FILE_ICON_SND,
-  .type_ext = "scl",
-  .readdir = summit_scale_read_dir,
-  .print_item = common_print_item,
-  .upload = summit_tuning_upload,
-  .load = scl_get_key_based_tuning_msg_from_scala_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = common_slot_get_upload_path
-};
-
 static gint
-summit_tuning_download (struct backend *backend, const gchar * path,
-			GByteArray * output, struct job_control *control)
+summit_tuning_download (struct backend *backend, const gchar *path,
+			struct idata *tuning, struct job_control *control)
 {
   guint32 id;
   gint err = 0;
   GByteArray *tx_msg, *rx_msg;
 
-  if (common_slot_get_id_name_from_path (path, &id, NULL))
+  if (common_slot_get_id_from_path (path, &id))
     {
       return -EINVAL;
     }
@@ -629,7 +632,7 @@ summit_tuning_download (struct backend *backend, const gchar * path,
   g_byte_array_append (tx_msg, SUMMIT_BULK_TUNING_REQ,
 		       sizeof (SUMMIT_BULK_TUNING_REQ));
   tx_msg->data[5] = id;
-  err = common_data_download (backend, tx_msg, &rx_msg, control);
+  err = common_data_tx_and_rx (backend, tx_msg, &rx_msg, control);
   if (err)
     {
       goto end;
@@ -640,30 +643,57 @@ summit_tuning_download (struct backend *backend, const gchar * path,
       goto cleanup;
     }
 
-  g_byte_array_append (output, rx_msg->data, rx_msg->len);
+  idata_init (tuning, rx_msg, NULL, NULL);
+  goto end;
 
 cleanup:
   free_msg (rx_msg);
 end:
+  usleep (SUMMIT_REST_TIME_US);
   return err;
 }
 
+static gint
+summit_tuning_load (const gchar *path, struct idata *tuning,
+		    struct job_control *control)
+{
+  gint err;
+  gchar *filename = g_path_get_basename (path);
+  if (strcmp (filename_get_ext (filename), SCALA_EXT))
+    {
+      err = file_load (path, tuning, control);
+    }
+  else
+    {
+      err = scl_load_key_based_tuning_msg_from_scala_file (path, tuning,
+							   control);
+    }
+  g_free (filename);
+  return err;
+}
+
+static GSList *
+summit_tunning_get_extensions ()
+{
+  GSList *exts = g_slist_append (NULL, strdup (SCALA_EXT));
+  return g_slist_append (exts, strdup (BE_SYSEX_EXT));
+}
+
 static const struct fs_operations FS_SUMMIT_BULK_TUNING_OPERATIONS = {
-  .fs = FS_SUMMIT_BULK_TUNING,
-  .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
-    FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
+  .id = FS_SUMMIT_BULK_TUNING,
+  .options = FS_OPTION_SINGLE_OP | FS_OPTION_SLOT_STORAGE,
   .name = "tuning",
-  .gui_name = "Tunings",
-  .gui_icon = BE_FILE_ICON_SND,
-  .type_ext = "syx",
-  .readdir = summit_scale_read_dir,
+  .gui_name = "Tuning Tables",
+  .gui_icon = FS_ICON_KEYS,
+  .ext = BE_SYSEX_EXT,
+  .readdir = summit_tuning_read_dir,
   .print_item = common_print_item,
   .download = summit_tuning_download,
   .upload = summit_tuning_upload,
-  .load = load_file,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_download_path = common_get_download_path,
+  .load = summit_tuning_load,
+  .save = file_save,
+  .get_exts = summit_tunning_get_extensions,
+  .get_download_path = common_slot_get_download_path_nn,
   .get_upload_path = common_slot_get_upload_path
 };
 
@@ -711,13 +741,12 @@ summit_wavetable_next_dentry (struct item_iterator *iter)
   memcpy (iter->item.name, &rx_msg->data[15], SUMMIT_WAVETABLE_NAME_LEN);
   iter->item.name[SUMMIT_WAVETABLE_NAME_LEN] = 0;
   gchar *c = &iter->item.name[SUMMIT_WAVETABLE_NAME_LEN - 1];
-  summit_truncate_name_at_last_useful_char (c);
+  summit_truncate_name (c);
   free_msg (rx_msg);
 
   iter->item.id = data->next;
-  iter->item.type = ELEKTROID_FILE;
+  iter->item.type = ITEM_TYPE_FILE;
   iter->item.size = 2678;
-  iter->item.slot_used = TRUE;
   data->next++;
 
   usleep (SUMMIT_REST_TIME_US * 10);
@@ -725,26 +754,19 @@ summit_wavetable_next_dentry (struct item_iterator *iter)
   return 0;
 }
 
-static gchar *
-summit_get_wavetable_id_as_slot (struct item *item, struct backend *backend)
-{
-  return summit_get_id_as_slot (item, backend, 2);
-}
-
 static gint
 summit_wavetable_read_dir (struct backend *backend,
-			   struct item_iterator *iter, const gchar * path,
-			   const gchar ** extensions)
+			   struct item_iterator *iter, const gchar *dir,
+			   GSList *extensions)
 {
-  if (!strcmp (path, "/"))
+  if (!strcmp (dir, "/"))
     {
       struct summit_wavetable_iterator_data *data =
 	g_malloc (sizeof (struct summit_wavetable_iterator_data));
       data->next = 0;
       data->backend = backend;
-      iter->data = data;
-      iter->next = summit_wavetable_next_dentry;
-      iter->free = g_free;
+      item_iterator_init (iter, dir, data, summit_wavetable_next_dentry,
+			  g_free);
       return 0;
     }
 
@@ -752,14 +774,16 @@ summit_wavetable_read_dir (struct backend *backend,
 }
 
 static gint
-summit_wavetable_download (struct backend *backend, const gchar * path,
-			   GByteArray * output, struct job_control *control)
+summit_wavetable_download (struct backend *backend, const gchar *path,
+			   struct idata *wavetable,
+			   struct job_control *control)
 {
   guint32 id;
   gint err = 0;
-  GByteArray *tx_msg, *rx_msg;
+  gchar name[SUMMIT_PATCH_NAME_LEN + 1];
+  GByteArray *tx_msg, *rx_msg, *output;
 
-  if (common_slot_get_id_name_from_path (path, &id, NULL))
+  if (common_slot_get_id_from_path (path, &id))
     {
       return -EINVAL;
     }
@@ -771,9 +795,11 @@ summit_wavetable_download (struct backend *backend, const gchar * path,
   control->parts = 6;
   control->part = 0;
 
+  output = g_byte_array_new ();
+
   //Header
   tx_msg = summit_get_wavetable_header_dump_msg (id + 64);
-  err = common_data_download_part (backend, tx_msg, &rx_msg, control);
+  err = common_data_tx_and_rx_part (backend, tx_msg, &rx_msg, control);
   if (err)
     {
       return err;
@@ -788,14 +814,13 @@ summit_wavetable_download (struct backend *backend, const gchar * path,
   g_byte_array_append (output, rx_msg->data, rx_msg->len);
   free_msg (rx_msg);
 
-  control->part++;
   usleep (SUMMIT_REST_TIME_US);
 
   //Waves
   for (gint8 i = 0; i < SUMMIT_WAVETABLE_WAVES; i++)
     {
       tx_msg = summit_get_wavetable_wave_dump_msg (id, i);
-      err = common_data_download_part (backend, tx_msg, &rx_msg, control);
+      err = common_data_tx_and_rx_part (backend, tx_msg, &rx_msg, control);
       if (err)
 	{
 	  goto err;
@@ -809,50 +834,32 @@ summit_wavetable_download (struct backend *backend, const gchar * path,
       g_byte_array_append (output, rx_msg->data, rx_msg->len);
       free_msg (rx_msg);
 
-      control->part++;
       usleep (SUMMIT_REST_TIME_US);
     }
 
-  return 0;
+  memcpy (name, &output->data[15], SUMMIT_WAVETABLE_NAME_LEN);
+  name[SUMMIT_WAVETABLE_NAME_LEN] = 0;
+  summit_truncate_name (&name[SUMMIT_WAVETABLE_NAME_LEN - 1]);
+
+  idata_init (wavetable, output, strdup (name), NULL);
+  goto end;
 
 err:
+  g_byte_array_free (output, TRUE);
   free_msg (rx_msg);
+end:
   usleep (SUMMIT_REST_TIME_US);
   return err;
 }
 
-static gchar *
-summit_get_wavetable_download_path (struct backend *backend,
-				    const struct fs_operations *ops,
-				    const gchar * dst_dir,
-				    const gchar * src_path,
-				    GByteArray * patch)
-{
-  guint id;
-  gchar *path;
-  gchar name[SUMMIT_PATCH_NAME_LEN + 1];
-
-  if (common_slot_get_id_name_from_path (src_path, &id, NULL))
-    {
-      return NULL;
-    }
-
-  memcpy (name, &patch->data[15], SUMMIT_WAVETABLE_NAME_LEN);
-  name[SUMMIT_WAVETABLE_NAME_LEN] = 0;
-  summit_truncate_name_at_last_useful_char (&name
-					    [SUMMIT_WAVETABLE_NAME_LEN - 1]);
-  path = common_get_download_path_with_params (backend, ops, dst_dir, id, 2,
-					       name);
-  return path;
-}
-
 static gint
-summit_wavetable_upload (struct backend *backend, const gchar * path,
-			 GByteArray * input, struct job_control *control)
+summit_wavetable_upload (struct backend *backend, const gchar *path,
+			 struct idata *wavetable, struct job_control *control)
 {
   guint id;
+  GByteArray *input = wavetable->content;
 
-  if (common_slot_get_id_name_from_path (path, &id, NULL))
+  if (common_slot_get_id_from_path (path, &id))
     {
       return -EINVAL;
     }
@@ -876,17 +883,18 @@ summit_wavetable_upload (struct backend *backend, const gchar * path,
 		  + SUMMIT_WAVETABLE_ID_POS] = id;
     }
 
-  return common_data_upload (backend, input, control);
+  return common_data_tx (backend, input, control);
 }
 
 static gint
-summit_wavetable_rename (struct backend *backend, const gchar * src,
-			 const gchar * dst)
+summit_wavetable_rename (struct backend *backend, const gchar *src,
+			 const gchar *dst)
 {
   GByteArray *tx_msg;
   guint id, len;
+  gchar *sanitized;
 
-  if (common_slot_get_id_name_from_path (src, &id, NULL))
+  if (common_slot_get_id_from_path (src, &id))
     {
       return -EINVAL;
     }
@@ -895,7 +903,7 @@ summit_wavetable_rename (struct backend *backend, const gchar * src,
       return -EINVAL;
     }
 
-  debug_print (1, "Renaming from %s to %s...\n", src, dst);
+  debug_print (1, "Renaming from %s to %s...", src, dst);
 
   tx_msg = g_byte_array_sized_new (23);
   g_byte_array_append (tx_msg, SUMMIT_GENERIC_REQ,
@@ -903,56 +911,56 @@ summit_wavetable_rename (struct backend *backend, const gchar * src,
   tx_msg->data[SUMMIT_REQ_OP_POS] = 0x7;
   tx_msg->data[14] = id;
   g_byte_array_append (tx_msg, (guint8 *) "       \xf7", 8);
-  len = strlen (dst);
+  sanitized = common_get_sanitized_name (dst, SUMMIT_ALPHABET,
+					 SUMMIT_DEFAULT_CHAR);
+  len = strlen (sanitized);
   len = len > SUMMIT_WAVETABLE_NAME_LEN ? SUMMIT_WAVETABLE_NAME_LEN : len;
-  memcpy (&tx_msg->data[15], dst, len);
+  memcpy (&tx_msg->data[15], sanitized, len);
+  g_free (sanitized);
   return backend_tx (backend, tx_msg);
 }
 
 static const struct fs_operations FS_SUMMIT_WAVETABLE_OPERATIONS = {
-  .fs = FS_SUMMIT_WAVETABLE,
-  .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
-    FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID |
-    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN,
+  .id = FS_SUMMIT_WAVETABLE,
+  .options = FS_OPTION_SINGLE_OP | FS_OPTION_SLOT_STORAGE |
+    FS_OPTION_SHOW_ID_COLUMN | FS_OPTION_SHOW_SIZE_COLUMN,
   .name = "wavetable",
   .gui_name = "Wavetables",
-  .gui_icon = BE_FILE_ICON_WAVE,
-  .type_ext = "syx",
+  .gui_icon = FS_ICON_WAVETABLE,
+  .ext = BE_SYSEX_EXT,
   .max_name_len = SUMMIT_WAVETABLE_NAME_LEN,
   .readdir = summit_wavetable_read_dir,
   .print_item = common_print_item,
   .rename = summit_wavetable_rename,
   .download = summit_wavetable_download,
   .upload = summit_wavetable_upload,
-  .get_slot = summit_get_wavetable_id_as_slot,
-  .load = load_file,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_download_path = summit_get_wavetable_download_path,
+  .load = file_load,
+  .save = file_save,
+  .get_download_path = common_slot_get_download_path_n,
   .get_upload_path = common_slot_get_upload_path
 };
 
-static const struct fs_operations *FS_SUMMIT_OPERATIONS[] = {
-  &FS_SUMMIT_SINGLE_OPERATIONS, &FS_SUMMIT_MULTI_OPERATIONS,
-  &FS_SUMMIT_WAVETABLE_OPERATIONS, &FS_SUMMIT_SCALE_OPERATIONS,
-  &FS_SUMMIT_BULK_TUNING_OPERATIONS, NULL
-};
-
-gint
+static gint
 summit_handshake (struct backend *backend)
 {
-  backend_midi_handshake (backend);
-
   if (memcmp (backend->midi_info.company, NOVATION_ID, sizeof (NOVATION_ID))
       || memcmp (backend->midi_info.family, SUMMIT_ID, sizeof (SUMMIT_ID)))
     {
       return -ENODEV;
     }
 
-  backend->filesystems = FS_SUMMIT_SINGLE_PATCH | FS_SUMMIT_MULTI_PATCH |
-    FS_SUMMIT_WAVETABLE | FS_SUMMIT_SCALE | FS_SUMMIT_BULK_TUNING;
-  backend->fs_ops = FS_SUMMIT_OPERATIONS;
+  gslist_fill (&backend->fs_ops, &FS_SUMMIT_SINGLE_OPERATIONS,
+	       &FS_SUMMIT_MULTI_OPERATIONS,
+	       &FS_SUMMIT_WAVETABLE_OPERATIONS,
+	       &FS_SUMMIT_BULK_TUNING_OPERATIONS, NULL);
   snprintf (backend->name, LABEL_MAX, "Novation Summit");
 
   return 0;
 }
+
+const struct connector CONNECTOR_SUMMIT = {
+  .handshake = summit_handshake,
+  .name = "summit",
+  .standard = TRUE,
+  .regex = ".*(Peak|Summit).*"
+};
