@@ -20,6 +20,7 @@
 
 #include <errno.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #if defined(__linux__)
 #include <sys/statvfs.h>
 #include <mntent.h>
@@ -31,63 +32,31 @@
 struct system_iterator_data
 {
   GDir *dir;
-  gchar *path;
-  const gchar **extensions;
+  const GSList *extensions;
 };
 
 static gint
-system_download (struct backend *backend, const gchar * path,
-		 GByteArray * output, struct job_control *control)
+system_download (struct backend *backend, const gchar *path,
+		 struct idata *idata, struct job_control *control)
 {
   gint err;
-  gboolean active;
 
   control->parts = 1;
   control->part = 0;
-  set_job_control_progress (control, 0.0);
+  job_control_set_progress (control, 0.0);
 
-  err = load_file (path, output, control);
+  err = file_load (path, idata, control);
 
-  g_mutex_lock (&control->mutex);
-  active = control->active;
-  g_mutex_unlock (&control->mutex);
-  if (active)
+  if (!err)
     {
-      set_job_control_progress (control, 1.0);
-    }
-  else
-    {
-      err = -ECANCELED;
+      job_control_set_progress (control, 1.0);
     }
 
   return err;
 }
 
-static gchar *
-system_get_download_path (struct backend *backend,
-			  const struct fs_operations *ops,
-			  const gchar * dst_dir, const gchar * src_path,
-			  GByteArray * content)
-{
-  gchar *name = g_path_get_basename (src_path);
-  GString *name_with_ext = g_string_new (NULL);
-  remove_ext (name);
-  g_string_append_printf (name_with_ext, "%s.%s", name, ops->type_ext);
-  gchar *path = path_chain (PATH_SYSTEM, dst_dir, name_with_ext->str);
-  g_string_free (name_with_ext, TRUE);
-  return path;
-}
-
-static gchar *
-system_get_upload_path (struct backend *backend,
-			const struct fs_operations *ops,
-			const gchar * dst_dir, const gchar * src_path)
-{
-  return system_get_download_path (backend, ops, dst_dir, src_path, NULL);
-}
-
 gint
-system_mkdir (struct backend *backend, const gchar * name)
+system_mkdir (struct backend *backend, const gchar *name)
 {
   gint res = g_mkdir_with_parents (name, 0755);
   if (res == 0 || errno == EEXIST)
@@ -96,7 +65,7 @@ system_mkdir (struct backend *backend, const gchar * name)
     }
   else
     {
-      error_print ("Error while creating dir %s\n", name);
+      error_print ("Error while creating dir %s", name);
       res = -errno;
     }
 
@@ -104,14 +73,14 @@ system_mkdir (struct backend *backend, const gchar * name)
 }
 
 gint
-system_delete (struct backend *backend, const gchar * path)
+system_delete (struct backend *backend, const gchar *path)
 {
   GDir *dir;
   gchar *new_path;
 
   if ((dir = g_dir_open (path, 0, NULL)))
     {
-      debug_print (1, "Deleting local %s dir...\n", path);
+      debug_print (1, "Deleting local %s dir...", path);
 
       const gchar *name;
       while ((name = g_dir_read_name (dir)) != NULL)
@@ -127,15 +96,15 @@ system_delete (struct backend *backend, const gchar * path)
     }
   else
     {
-      debug_print (1, "Deleting local %s file...\n", path);
+      debug_print (1, "Deleting local %s file...", path);
       return g_unlink (path);
     }
 }
 
 gint
-system_rename (struct backend *backend, const gchar * old, const gchar * new)
+system_rename (struct backend *backend, const gchar *old, const gchar *new)
 {
-  debug_print (1, "Renaming locally from %s to %s...\n", old, new);
+  debug_print (1, "Renaming locally from %s to %s...", old, new);
   return rename (old, new);
 }
 
@@ -144,7 +113,6 @@ system_free_iterator_data (void *iter_data)
 {
   struct system_iterator_data *data = iter_data;
   g_dir_close (data->dir);
-  g_free (data->path);
   g_free (data);
 }
 
@@ -163,19 +131,19 @@ system_next_dentry (struct item_iterator *iter, gboolean sample_info)
 	  continue;
 	}
 
-      full_path = path_chain (PATH_SYSTEM, data->path, name);
+      full_path = path_chain (PATH_SYSTEM, iter->dir, name);
       enum item_type type;
       if (g_file_test (full_path, G_FILE_TEST_IS_DIR))
 	{
-	  type = ELEKTROID_DIR;
+	  type = ITEM_TYPE_DIR;
 	}
       else if (g_file_test (full_path, G_FILE_TEST_IS_REGULAR))
 	{
-	  type = ELEKTROID_FILE;
+	  type = ITEM_TYPE_FILE;
 	}
       else
 	{
-	  error_print ("'%s' is neither file nor directory\n", full_path);
+	  error_print ("'%s' is neither file nor directory", full_path);
 	  continue;
 	}
 
@@ -186,9 +154,10 @@ system_next_dentry (struct item_iterator *iter, gboolean sample_info)
 	  iter->item.size = st.st_size;
 	  iter->item.id = -1;
 
-	  if (iter_is_dir_or_matches_extensions (iter, data->extensions))
+	  if (item_iterator_is_dir_or_matches_extensions (iter,
+							  data->extensions))
 	    {
-	      if (iter->item.type == ELEKTROID_FILE && sample_info)
+	      if (iter->item.type == ITEM_TYPE_FILE && sample_info)
 		{
 		  sample_load_sample_info (full_path,
 					   &iter->item.sample_info);
@@ -218,63 +187,57 @@ system_next_dentry_with_sample_info (struct item_iterator *iter)
 
 static gint
 system_read_dir_opts (struct backend *backend, struct item_iterator *iter,
-		      const gchar * path, const gchar ** extensions,
+		      const gchar *dir, GSList *extensions,
 		      iterator_next next)
 {
-  GDir *dir;
+  GDir *gdir;
   struct system_iterator_data *data;
 
-  if (!(dir = g_dir_open (path, 0, NULL)))
+  if (!(gdir = g_dir_open (dir, 0, NULL)))
     {
       return -errno;
     }
 
   data = g_malloc (sizeof (struct system_iterator_data));
-  data->dir = dir;
-  data->path = strdup (path);
+  data->dir = gdir;
   data->extensions = extensions;
 
-  iter->data = data;
-  iter->next = next;
-  iter->free = system_free_iterator_data;
+  item_iterator_init (iter, dir, data, next, system_free_iterator_data);
 
   return 0;
 }
 
 gint
 system_read_dir (struct backend *backend, struct item_iterator *iter,
-		 const gchar * path, const gchar ** extensions)
+		 const gchar *dir, GSList *extensions)
 {
-  return system_read_dir_opts (backend, iter, path, extensions,
+  return system_read_dir_opts (backend, iter, dir, extensions,
 			       system_next_dentry_without_sample_info);
 }
 
 gint
 system_samples_read_dir (struct backend *backend, struct item_iterator *iter,
-			 const gchar * path, const gchar ** extensions)
+			 const gchar *dir, GSList *extensions)
 {
-  return system_read_dir_opts (backend, iter, path,
-			       sample_get_sample_extensions (),
+  return system_read_dir_opts (backend, iter, dir, extensions,
 			       system_next_dentry_with_sample_info);
 }
 
 static gint
-system_load_custom (const gchar * path, GByteArray * sample,
+system_load_custom (const gchar *path, struct idata *sample,
 		    struct job_control *control,
-		    struct sample_info *sample_info_dst)
+		    const struct sample_info *sample_info_req)
 {
+  struct sample_info sample_info_src;
+  //Typically, control parts are set not here but in this case makes more sense.
   control->parts = 1;
   control->part = 0;
-  gint res = sample_load_from_file (path, sample, control, sample_info_dst);
-  if (!res)
-    {
-      memcpy (control->data, sample_info_dst, sizeof (struct sample_info));
-    }
-  return res;
+  return sample_load_from_file (path, sample, control, sample_info_req,
+				&sample_info_src);
 }
 
 static gint
-system_load_48_16_stereo (const gchar * path, GByteArray * sample,
+system_load_48_16_stereo (const gchar *path, struct idata *sample,
 			  struct job_control *control)
 {
   struct sample_info sample_info_dst;
@@ -285,15 +248,16 @@ system_load_48_16_stereo (const gchar * path, GByteArray * sample,
 }
 
 gint
-system_upload (struct backend *backend, const gchar * path,
-	       GByteArray * input, struct job_control *control)
+system_upload (struct backend *backend, const gchar *path,
+	       struct idata *sample, struct job_control *control)
 {
-  return sample_save_to_file (path, input, control,
+  //Typically, control parts are set here but in this case makes more sense do it in the load functions.
+  return sample_save_to_file (path, sample, control,
 			      SF_FORMAT_WAV | SF_FORMAT_PCM_16);
 }
 
 static gint
-system_load_48_16_mono (const gchar * path, GByteArray * sample,
+system_load_48_16_mono (const gchar *path, struct idata *sample,
 			struct job_control *control)
 {
   struct sample_info sample_info_dst;
@@ -304,7 +268,7 @@ system_load_48_16_mono (const gchar * path, GByteArray * sample,
 }
 
 static gint
-system_load_441_16_stereo (const gchar * path, GByteArray * sample,
+system_load_441_16_stereo (const gchar *path, struct idata *sample,
 			   struct job_control *control)
 {
   struct sample_info sample_info_dst;
@@ -315,7 +279,7 @@ system_load_441_16_stereo (const gchar * path, GByteArray * sample,
 }
 
 static gint
-system_load_441_16_mono (const gchar * path, GByteArray * sample,
+system_load_441_16_mono (const gchar *path, struct idata *sample,
 			 struct job_control *control)
 {
   struct sample_info sample_info_dst;
@@ -326,7 +290,7 @@ system_load_441_16_mono (const gchar * path, GByteArray * sample,
 }
 
 static gint
-system_load_441_24_stereo (const gchar * path, GByteArray * sample,
+system_load_441_24_stereo (const gchar *path, struct idata *sample,
 			   struct job_control *control)
 {
   struct sample_info sample_info_dst;
@@ -337,15 +301,15 @@ system_load_441_24_stereo (const gchar * path, GByteArray * sample,
 }
 
 static gint
-system_upload_24_bits (struct backend *backend, const gchar * path,
-		       GByteArray * input, struct job_control *control)
+system_upload_24_bits (struct backend *backend, const gchar *path,
+		       struct idata *sample, struct job_control *control)
 {
-  return sample_save_to_file (path, input, control,
+  return sample_save_to_file (path, sample, control,
 			      SF_FORMAT_WAV | SF_FORMAT_PCM_24);
 }
 
 static gint
-system_load_441_24_mono (const gchar * path, GByteArray * sample,
+system_load_441_24_mono (const gchar *path, struct idata *sample,
 			 struct job_control *control)
 {
   struct sample_info sample_info_dst;
@@ -356,7 +320,7 @@ system_load_441_24_mono (const gchar * path, GByteArray * sample,
 }
 
 static gint
-system_load_441_8_stereo (const gchar * path, GByteArray * sample,
+system_load_441_8_stereo (const gchar *path, struct idata *sample,
 			  struct job_control *control)
 {
   struct sample_info sample_info_dst;
@@ -367,15 +331,15 @@ system_load_441_8_stereo (const gchar * path, GByteArray * sample,
 }
 
 static gint
-system_upload_8_bits (struct backend *backend, const gchar * path,
-		      GByteArray * input, struct job_control *control)
+system_upload_8_bits (struct backend *backend, const gchar *path,
+		      struct idata *sample, struct job_control *control)
 {
-  return sample_save_to_file (path, input, control,
+  return sample_save_to_file (path, sample, control,
 			      SF_FORMAT_WAV | SF_FORMAT_PCM_U8);
 }
 
 static gint
-system_load_441_8_mono (const gchar * path, GByteArray * sample,
+system_load_441_8_mono (const gchar *path, struct idata *sample,
 			struct job_control *control)
 {
   struct sample_info sample_info_dst;
@@ -385,31 +349,45 @@ system_load_441_8_mono (const gchar * path, GByteArray * sample,
   return system_load_custom (path, sample, control, &sample_info_dst);
 }
 
+static gint
+system_load_32_16_mono (const gchar *path, struct idata *sample,
+			struct job_control *control)
+{
+  struct sample_info sample_info_dst;
+  sample_info_dst.rate = 32000;
+  sample_info_dst.channels = 1;
+  sample_info_dst.format = SF_FORMAT_PCM_16;
+  return system_load_custom (path, sample, control, &sample_info_dst);
+}
+
 gboolean
-system_file_exists (struct backend *backend, const gchar * path)
+system_file_exists (struct backend *backend, const gchar *path)
 {
   return access (path, F_OK) == 0;
 }
 
 enum system_fs
 {
-  FS_SAMPLES_LOCAL_48_16_STEREO = 0x1,
-  FS_SAMPLES_LOCAL_48_16_MONO = 0x2,
-  FS_SAMPLES_LOCAL_441_16_STEREO = 0x4,
-  FS_SAMPLES_LOCAL_441_16_MONO = 0x8,
-  FS_SAMPLES_LOCAL_441_24_STEREO = 0x10,
-  FS_SAMPLES_LOCAL_441_24_MONO = 0x20,
-  FS_SAMPLES_LOCAL_441_8_STEREO = 0x40,
-  FS_SAMPLES_LOCAL_441_8_MONO = 0x80
+  FS_SAMPLES_LOCAL_48_16_STEREO,
+  FS_SAMPLES_LOCAL_48_16_MONO,
+  FS_SAMPLES_LOCAL_441_16_STEREO,
+  FS_SAMPLES_LOCAL_441_16_MONO,
+  FS_SAMPLES_LOCAL_441_24_STEREO,
+  FS_SAMPLES_LOCAL_441_24_MONO,
+  FS_SAMPLES_LOCAL_441_8_STEREO,
+  FS_SAMPLES_LOCAL_441_8_MONO,
+  FS_SAMPLES_LOCAL_32_16_MONO
 };
 
 const struct fs_operations FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS = {
-  .fs = FS_SAMPLES_LOCAL_48_16_STEREO,
-  .options = FS_OPTION_SORT_BY_NAME | FS_OPTION_SAMPLE_EDITOR |
-    FS_OPTION_STEREO | FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
+  .id = FS_SAMPLES_LOCAL_48_16_STEREO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_STEREO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "wav48k16b2c",
   .gui_name = "WAV 48 KHz 16 bits stereo",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
   .readdir = system_samples_read_dir,
   .print_item = common_print_item,
   .file_exists = system_file_exists,
@@ -420,22 +398,22 @@ const struct fs_operations FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS = {
   .download = system_download,
   .upload = system_upload,
   .load = system_load_48_16_stereo,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = system_get_upload_path,
-  .get_download_path = system_get_download_path,
-  .type_ext = "wav",
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
 
 const struct fs_operations FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS = {
-  .fs = FS_SAMPLES_LOCAL_48_16_MONO,
-  .options =
-    FS_OPTION_SORT_BY_NAME | FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
-    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
+  .id = FS_SAMPLES_LOCAL_48_16_MONO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "wav48k16b1c",
   .gui_name = "WAV 48 KHz 16 bits mono",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
   .readdir = system_samples_read_dir,
   .print_item = common_print_item,
   .file_exists = system_file_exists,
@@ -446,22 +424,22 @@ const struct fs_operations FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS = {
   .download = system_download,
   .upload = system_upload,
   .load = system_load_48_16_mono,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = system_get_upload_path,
-  .get_download_path = system_get_download_path,
-  .type_ext = "wav",
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
 
 const struct fs_operations FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS = {
-  .fs = FS_SAMPLES_LOCAL_441_16_STEREO,
-  .options =
-    FS_OPTION_SORT_BY_NAME | FS_OPTION_SAMPLE_EDITOR |
-    FS_OPTION_STEREO | FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
+  .id = FS_SAMPLES_LOCAL_441_16_STEREO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_STEREO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "wav44.1k16b2c",
   .gui_name = "WAV 44.1 KHz 16 bits stereo",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
   .readdir = system_samples_read_dir,
   .print_item = common_print_item,
   .file_exists = system_file_exists,
@@ -472,22 +450,22 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS = {
   .download = system_download,
   .upload = system_upload,
   .load = system_load_441_16_stereo,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = system_get_upload_path,
-  .get_download_path = system_get_download_path,
-  .type_ext = "wav",
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
 
 const struct fs_operations FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS = {
-  .fs = FS_SAMPLES_LOCAL_441_16_MONO,
-  .options =
-    FS_OPTION_SORT_BY_NAME | FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
-    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
+  .id = FS_SAMPLES_LOCAL_441_16_MONO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "wav44.1k16b1c",
   .gui_name = "WAV 44.1 KHz 16 bits mono",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
   .readdir = system_samples_read_dir,
   .print_item = common_print_item,
   .file_exists = system_file_exists,
@@ -501,22 +479,22 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS = {
   .download = system_download,
   .upload = system_upload,
   .load = system_load_441_16_mono,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = system_get_upload_path,
-  .get_download_path = system_get_download_path,
-  .type_ext = "wav",
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
 
 const struct fs_operations FS_SYSTEM_SAMPLES_441_24_STEREO_OPERATIONS = {
-  .fs = FS_SAMPLES_LOCAL_441_24_STEREO,
-  .options =
-    FS_OPTION_SORT_BY_NAME | FS_OPTION_SAMPLE_EDITOR |
-    FS_OPTION_STEREO | FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
+  .id = FS_SAMPLES_LOCAL_441_24_STEREO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_STEREO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "wav44.1k24b2c",
   .gui_name = "WAV 44.1 KHz 24 bits stereo",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
   .readdir = system_samples_read_dir,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
@@ -526,22 +504,22 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_24_STEREO_OPERATIONS = {
   .download = system_download,
   .upload = system_upload_24_bits,
   .load = system_load_441_24_stereo,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = system_get_upload_path,
-  .get_download_path = system_get_download_path,
-  .type_ext = "wav",
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
 
 const struct fs_operations FS_SYSTEM_SAMPLES_441_24_MONO_OPERATIONS = {
-  .fs = FS_SAMPLES_LOCAL_441_24_MONO,
-  .options =
-    FS_OPTION_SORT_BY_NAME | FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
-    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
+  .id = FS_SAMPLES_LOCAL_441_24_MONO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "wav44.1k24b1c",
   .gui_name = "WAV 44.1 KHz 24 bits mono",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
   .readdir = system_samples_read_dir,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
@@ -554,22 +532,22 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_24_MONO_OPERATIONS = {
   .download = system_download,
   .upload = system_upload_24_bits,
   .load = system_load_441_24_mono,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = system_get_upload_path,
-  .get_download_path = system_get_download_path,
-  .type_ext = "wav",
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
 
 const struct fs_operations FS_SYSTEM_SAMPLES_441_8_STEREO_OPERATIONS = {
-  .fs = FS_SAMPLES_LOCAL_441_8_STEREO,
-  .options =
-    FS_OPTION_SORT_BY_NAME | FS_OPTION_SAMPLE_EDITOR |
-    FS_OPTION_STEREO | FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
+  .id = FS_SAMPLES_LOCAL_441_8_STEREO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_STEREO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "wav44.1k8b2c",
   .gui_name = "WAV 44.1 KHz 8 bits stereo",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
   .readdir = system_samples_read_dir,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
@@ -579,22 +557,22 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_8_STEREO_OPERATIONS = {
   .download = system_download,
   .upload = system_upload_8_bits,
   .load = system_load_441_8_stereo,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = system_get_upload_path,
-  .get_download_path = system_get_download_path,
-  .type_ext = "wav",
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
 
 const struct fs_operations FS_SYSTEM_SAMPLES_441_8_MONO_OPERATIONS = {
-  .fs = FS_SAMPLES_LOCAL_441_8_MONO,
-  .options =
-    FS_OPTION_SORT_BY_NAME | FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
-    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
+  .id = FS_SAMPLES_LOCAL_441_8_MONO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "wav44.1k8b1c",
   .gui_name = "WAV 44.1 KHz 8 bits mono",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
   .readdir = system_samples_read_dir,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
@@ -607,31 +585,46 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_8_MONO_OPERATIONS = {
   .download = system_download,
   .upload = system_upload_8_bits,
   .load = system_load_441_8_mono,
-  .save = save_file,
-  .get_ext = backend_get_fs_ext,
-  .get_upload_path = system_get_upload_path,
-  .get_download_path = system_get_download_path,
-  .type_ext = "wav",
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
 
-static const struct fs_operations *FS_SYSTEM_OPERATIONS[] = {
-  &FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS,
-  &FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS,
-  &FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS,
-  &FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS,
-  &FS_SYSTEM_SAMPLES_441_24_STEREO_OPERATIONS,
-  &FS_SYSTEM_SAMPLES_441_24_MONO_OPERATIONS,
-  &FS_SYSTEM_SAMPLES_441_8_STEREO_OPERATIONS,
-  &FS_SYSTEM_SAMPLES_441_8_MONO_OPERATIONS,
-  NULL
+const struct fs_operations FS_SYSTEM_SAMPLES_32_16_MONO_OPERATIONS = {
+  .id = FS_SAMPLES_LOCAL_32_16_MONO,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
+  .name = "wav32k8b1c",
+  .gui_name = "WAV 32 KHz 16 bits mono",
+  .gui_icon = FS_ICON_WAVE,
+  .ext = "wav",
+  .readdir = system_samples_read_dir,
+  .file_exists = system_file_exists,
+  .mkdir = system_mkdir,
+  .delete = system_delete,
+  .rename = system_rename,
+  .move = system_rename,
+  .copy = NULL,
+  .clear = NULL,
+  .swap = NULL,
+  .download = system_download,
+  .upload = system_upload,
+  .load = system_load_32_16_mono,
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
+  .max_name_len = 255
 };
 
 #if defined (__linux__)
 static gint
-system_get_storage_stats (struct backend *backend, gint type,
+system_get_storage_stats (struct backend *backend, guint8 type,
 			  struct backend_storage_stats *statfs,
-			  const gchar * path)
+			  const gchar *path)
 {
   gint err;
   FILE *f;
@@ -673,33 +666,33 @@ system_get_storage_stats (struct backend *backend, gint type,
 }
 #endif
 
-gint
-system_init_backend (struct backend *backend, const gchar * id)
+static gint
+system_handshake (struct backend *backend)
 {
-  if (strcmp (id, BE_SYSTEM_ID))
-    {
-      return -ENODEV;
-    }
-
-  backend->type = BE_TYPE_SYSTEM;
-  backend->filesystems =
-    FS_SAMPLES_LOCAL_48_16_STEREO | FS_SAMPLES_LOCAL_48_16_MONO |
-    FS_SAMPLES_LOCAL_441_16_STEREO | FS_SAMPLES_LOCAL_441_16_MONO |
-    FS_SAMPLES_LOCAL_441_24_STEREO | FS_SAMPLES_LOCAL_441_24_MONO |
-    FS_SAMPLES_LOCAL_441_8_STEREO | FS_SAMPLES_LOCAL_441_8_MONO;
-  backend->fs_ops = FS_SYSTEM_OPERATIONS;
+  gslist_fill (&backend->fs_ops, &FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_441_24_STEREO_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_441_24_MONO_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_441_8_STEREO_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_441_8_MONO_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_32_16_MONO_OPERATIONS, NULL);
   snprintf (backend->name, LABEL_MAX, "%s", _("System"));
-  *backend->version = 0;
-  *backend->description = 0;
-  backend->conn_name = "system";
-  backend->destroy_data = NULL;
-  backend->upgrade_os = NULL;
+
+  backend->get_storage_stats =
 #if defined (__linux__)
-  backend->get_storage_stats = system_get_storage_stats;
-  backend->storage = 1;
+    system_get_storage_stats;
 #else
-  backend->get_storage_stats = NULL;
-  backend->storage = 0;
+    NULL;
 #endif
+
   return 0;
 }
+
+const struct connector CONNECTOR_SYSTEM = {
+  .handshake = system_handshake,
+  .name = "system",
+  .standard = FALSE,
+  .regex = NULL
+};
