@@ -18,14 +18,17 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "backend.h"
-#include "utils.h"
+#include "connector.h"
+#include "sample.h"
 
-gint system_read_dir (struct backend *, struct item_iterator *, const gchar *,
-		      const gchar **);
+extern struct connector CONNECTOR_SYSTEM;
 
-gint system_samples_read_dir (struct backend *, struct item_iterator *,
-			      const gchar *, const gchar **);
+gint system_read_dir (struct backend *backend, struct item_iterator *iter,
+		      const gchar * dir, GSList * extensions);
+
+gint system_samples_read_dir (struct backend *backend,
+			      struct item_iterator *iter, const gchar * dir,
+			      GSList * extensions);
 
 gboolean system_file_exists (struct backend *, const gchar *);
 
@@ -37,5 +40,3 @@ gint system_rename (struct backend *, const gchar *, const gchar *);
 
 gint system_upload (struct backend *, const gchar *, GByteArray *,
 		    struct job_control *);
-
-gint system_init_backend (struct backend *, const gchar *);
