@@ -28,10 +28,11 @@
 #include <inttypes.h>
 #include <stddef.h>
 #include "backend.h"
-#include "connector.h"
+#include "regconn.h"
+#include "regpref.h"
 #include "utils.h"
 
-#define COMMAND_NOT_IN_SYSTEM_FS "Command not available in system backend\n"
+#define COMMAND_NOT_IN_SYSTEM_FS "Command not available in system backend"
 
 #define GET_FS_OPS_OFFSET(member) offsetof(struct fs_operations, member)
 #define GET_FS_OPS_FUNC(type,fs,offset) (*(((type *) (((gchar *) fs) + offset))))
@@ -44,10 +45,10 @@ static gchar *connector, *fs, *op;
 const struct fs_operations *fs_ops;
 
 static const gchar *
-cli_get_path (gchar * device_path)
+cli_get_path (const gchar *device_path)
 {
   gint len = strlen (device_path);
-  gchar *path = device_path;
+  const gchar *path = device_path;
   gint i = 0;
 
   while (*path != G_SEARCHPATH_SEPARATOR && i < len)
@@ -79,7 +80,7 @@ cli_ld ()
 }
 
 static gint
-cli_connect (const gchar * device_path)
+cli_connect (const gchar *device_path)
 {
   gint err, id = (gint) atoi (device_path);
   struct backend_device device;
@@ -87,20 +88,20 @@ cli_connect (const gchar * device_path)
 
   if (!devices->len || id >= devices->len)
     {
-      error_print ("Invalid device %d\n", id);
+      error_print ("Invalid device %d", id);
       return -ENODEV;
     }
 
   device = g_array_index (devices, struct backend_device, id);
-  err = connector_init_backend (&backend, &device, connector, NULL);
+  err = backend_init_connector (&backend, &device, connector, NULL);
   g_array_free (devices, TRUE);
 
   if (!err && fs)
     {
-      fs_ops = backend_get_fs_operations (&backend, 0, fs);
+      fs_ops = backend_get_fs_operations_by_name (&backend, fs);
       if (!fs_ops)
 	{
-	  error_print ("Invalid filesystem '%s'\n", fs);
+	  error_print ("Invalid filesystem '%s'", fs);
 	  return -EINVAL;
 	}
     }
@@ -109,16 +110,17 @@ cli_connect (const gchar * device_path)
 }
 
 static gint
-cli_list (int argc, gchar * argv[], int *optind)
+cli_list (int argc, gchar *argv[], int *optind)
 {
   gint err;
   const gchar *path;
   struct item_iterator iter;
-  gchar *device_path;
+  const gchar *device_path;
+  gboolean active;
 
   if (*optind == argc)
     {
-      error_print ("Remote path missing\n");
+      error_print ("Remote path missing");
       return -EINVAL;
     }
   else
@@ -137,34 +139,40 @@ cli_list (int argc, gchar * argv[], int *optind)
 
   CHECK_FS_OPS_FUNC (fs_ops->readdir);
 
+  active = TRUE;
+  sysex_transfer.active = TRUE;
+
   err = fs_ops->readdir (&backend, &iter, path, NULL);
   if (err)
     {
       return err;
     }
 
-  while (!next_item_iterator (&iter))
+  while (!item_iterator_next (&iter) && active)
     {
       fs_ops->print_item (&iter, &backend, fs_ops);
+
+      g_mutex_lock (&sysex_transfer.mutex);
+      active = sysex_transfer.active;
+      g_mutex_unlock (&sysex_transfer.mutex);
     }
 
-  free_item_iterator (&iter);
+  item_iterator_free (&iter);
 
   return EXIT_SUCCESS;
 }
 
 static int
-cli_command_path (int argc, gchar * argv[], int *optind,
-		  ssize_t member_offset)
+cli_command_path (int argc, gchar *argv[], int *optind, ssize_t member_offset)
 {
   const gchar *path;
-  gchar *device_path;
+  const gchar *device_path;
   gint err;
   fs_path_func f;
 
   if (*optind == argc)
     {
-      error_print ("Remote path missing\n");
+      error_print ("Remote path missing");
       return -EINVAL;
     }
   else
@@ -188,7 +196,7 @@ cli_command_path (int argc, gchar * argv[], int *optind,
 }
 
 static gint
-cli_command_src_dst (int argc, gchar * argv[], int *optind,
+cli_command_src_dst (int argc, gchar *argv[], int *optind,
 		     ssize_t member_offset)
 {
   const gchar *src_path, *dst_path;
@@ -198,7 +206,7 @@ cli_command_src_dst (int argc, gchar * argv[], int *optind,
 
   if (*optind == argc)
     {
-      error_print ("Remote path source missing\n");
+      error_print ("Remote path source missing");
       return -EINVAL;
     }
   else
@@ -209,7 +217,7 @@ cli_command_src_dst (int argc, gchar * argv[], int *optind,
 
   if (*optind == argc)
     {
-      error_print ("Remote path destination missing\n");
+      error_print ("Remote path destination missing");
       return -EINVAL;
     }
   else
@@ -222,7 +230,7 @@ cli_command_src_dst (int argc, gchar * argv[], int *optind,
   dst_card = atoi (device_dst_path);
   if (src_card != dst_card)
     {
-      error_print ("Source and destination device must be the same\n");
+      error_print ("Source and destination device must be the same");
       return -EINVAL;
     }
 
@@ -241,7 +249,7 @@ cli_command_src_dst (int argc, gchar * argv[], int *optind,
 }
 
 static gint
-cli_command_mv_rename (int argc, gchar * argv[], int *optind)
+cli_command_mv_rename (int argc, gchar *argv[], int *optind)
 {
   const gchar *src_path, *dst_path;
   gchar *device_src_path, *device_dst_path;
@@ -250,7 +258,7 @@ cli_command_mv_rename (int argc, gchar * argv[], int *optind)
 
   if (*optind == argc)
     {
-      error_print ("Remote path source missing\n");
+      error_print ("Remote path source missing");
       return -EINVAL;
     }
   else
@@ -261,7 +269,7 @@ cli_command_mv_rename (int argc, gchar * argv[], int *optind)
 
   if (*optind == argc)
     {
-      error_print ("Remote path destination missing\n");
+      error_print ("Remote path destination missing");
       return -EINVAL;
     }
   else
@@ -286,7 +294,7 @@ cli_command_mv_rename (int argc, gchar * argv[], int *optind)
       dst_card = atoi (device_dst_path);
       if (src_card != dst_card)
 	{
-	  error_print ("Source and destination device must be the same\n");
+	  error_print ("Source and destination device must be the same");
 	  return -EINVAL;
 	}
       dst_path = cli_get_path (device_dst_path);
@@ -303,15 +311,16 @@ cli_command_mv_rename (int argc, gchar * argv[], int *optind)
 }
 
 static int
-cli_info (int argc, gchar * argv[], int *optind)
+cli_info (int argc, gchar *argv[], int *optind)
 {
-  gchar *device_path;
-  const gchar *name;
+  const gchar *device_path;
   gint err;
+  gboolean first;
+  GSList *e;
 
   if (*optind == argc)
     {
-      error_print ("Device missing\n");
+      error_print ("Device missing");
       return -EINVAL;
     }
   else
@@ -333,13 +342,17 @@ cli_info (int argc, gchar * argv[], int *optind)
   printf ("Connector name: %s\n", backend.conn_name);
   printf ("Filesystems: ");
 
-  for (guint32 fs = 1, i = 0; i < MAX_BACKEND_FSS; fs <<= 1, i++)
+  e = backend.fs_ops;
+  first = TRUE;
+  while (e)
     {
-      if (backend.filesystems & fs)
-	{
-	  name = backend_get_fs_operations (&backend, fs, NULL)->name;
-	  printf ("%s%s", fs == 1 ? "" : ",", name);
-	}
+      const struct fs_operations *fs_ops = e->data;
+      const gchar *name = fs_ops->name;
+      gboolean cli_only = fs_ops->gui_name == NULL;
+      printf ("%s%s%s", first ? "" : ", ", name,
+	      cli_only ? " (CLI only)" : "");
+      first = FALSE;
+      e = e->next;
     }
   printf ("\n");
 
@@ -347,19 +360,19 @@ cli_info (int argc, gchar * argv[], int *optind)
 }
 
 static int
-cli_df (int argc, gchar * argv[], int *optind)
+cli_df (int argc, gchar *argv[], int *optind)
 {
-  gchar *device_path;
+  const gchar *device_path;
   const gchar *path;
   gchar *size;
   gchar *diff;
   gchar *free;
-  gint err, storage;
+  gint err;
   struct backend_storage_stats statfs;
 
   if (*optind == argc)
     {
-      error_print ("Device missing\n");
+      error_print ("Device missing");
       return -EINVAL;
     }
   else
@@ -374,7 +387,7 @@ cli_df (int argc, gchar * argv[], int *optind)
       return err;
     }
 
-  if (!backend.storage || !backend.get_storage_stats)
+  if (!backend.get_storage_stats)
     {
       return -ENOSYS;
     }
@@ -389,15 +402,11 @@ cli_df (int argc, gchar * argv[], int *optind)
 	  "Used", "Available", "Use%");
 
   err = 0;
-  for (storage = 1; storage < MAX_BACKEND_STORAGE; storage <<= 1)
+  for (guint8 i = 0; i < G_MAXUINT8; i++)
     {
-      if (backend.storage & storage)
+      gint v = backend.get_storage_stats (&backend, i, &statfs, path);
+      if (v >= 0)
 	{
-	  err |= backend.get_storage_stats (&backend, storage, &statfs, path);
-	  if (err)
-	    {
-	      continue;
-	    }
 	  size = get_human_size (statfs.bsize, FALSE);
 	  diff = get_human_size (statfs.bsize - statfs.bfree, FALSE);
 	  free = get_human_size (statfs.bfree, FALSE);
@@ -409,22 +418,26 @@ cli_df (int argc, gchar * argv[], int *optind)
 	  g_free (free);
 	}
 
-      usleep (BE_REST_TIME_US);
+      if (!v)
+	{
+	  break;
+	}
     }
 
   return err;
 }
 
 static int
-cli_upgrade_os (int argc, gchar * argv[], int *optind)
+cli_upgrade_os (int argc, gchar *argv[], int *optind)
 {
   gint err;
   const gchar *src_path;
   const gchar *device_path;
+  struct idata idata;
 
   if (*optind == argc)
     {
-      error_print ("Local path missing\n");
+      error_print ("Local path missing");
       return EXIT_FAILURE;
     }
   else
@@ -435,7 +448,7 @@ cli_upgrade_os (int argc, gchar * argv[], int *optind)
 
   if (*optind == argc)
     {
-      error_print ("Remote path missing\n");
+      error_print ("Remote path missing");
       return EXIT_FAILURE;
     }
   else
@@ -455,35 +468,35 @@ cli_upgrade_os (int argc, gchar * argv[], int *optind)
       return EXIT_FAILURE;
     }
 
-  sysex_transfer.raw = g_byte_array_new ();
-  err = load_file (src_path, sysex_transfer.raw, NULL);
+  err = file_load (src_path, &idata, NULL);
   if (err)
     {
-      error_print ("Error while loading '%s'.\n", src_path);
+      error_print ("Error while loading '%s'.", src_path);
     }
   else
     {
+      sysex_transfer.raw = idata.content;
       sysex_transfer.active = TRUE;
       sysex_transfer.timeout = BE_SYSEX_TIMEOUT_MS;
       CHECK_FS_OPS_FUNC (backend.upgrade_os);
       err = backend.upgrade_os (&backend, &sysex_transfer);
+      idata_free (&idata);
     }
 
-  g_byte_array_free (sysex_transfer.raw, TRUE);
   return err;
 }
 
 static int
-cli_download (int argc, gchar * argv[], int *optind)
+cli_download (int argc, gchar *argv[], int *optind)
 {
   const gchar *src_path;
   gchar *device_src_path, *download_path;
   gint err;
-  GByteArray *array;
+  struct idata idata;
 
   if (*optind == argc)
     {
-      error_print ("Remote path missing\n");
+      error_print ("Remote path missing");
       return EXIT_FAILURE;
     }
   else
@@ -501,42 +514,40 @@ cli_download (int argc, gchar * argv[], int *optind)
   src_path = cli_get_path (device_src_path);
 
   control.active = TRUE;
-  array = g_byte_array_new ();
   CHECK_FS_OPS_FUNC (fs_ops->download);
-  err = fs_ops->download (&backend, src_path, array, &control);
+  err = fs_ops->download (&backend, src_path, &idata, &control);
   if (err)
     {
-      goto end;
+      return err;
     }
 
   download_path = fs_ops->get_download_path (&backend, fs_ops, ".", src_path,
-					     array);
+					     &idata);
   if (!download_path)
     {
       err = -EINVAL;
-      goto end;
+      goto cleanup;
     }
 
-  err = fs_ops->save (download_path, array, &control);
+  err = fs_ops->save (download_path, &idata, &control);
   g_free (download_path);
-  g_free (control.data);
 
-end:
-  g_byte_array_free (array, TRUE);
+cleanup:
+  idata_free (&idata);
   return err;
 }
 
 static int
-cli_upload (int argc, gchar * argv[], int *optind)
+cli_upload (int argc, gchar *argv[], int *optind)
 {
   const gchar *dst_path;
   gchar *src_path, *device_dst_path, *upload_path;
   gint err;
-  GByteArray *array;
+  struct idata idata;
 
   if (*optind == argc)
     {
-      error_print ("Local path missing\n");
+      error_print ("Local path missing");
       return EXIT_FAILURE;
     }
   else
@@ -547,7 +558,7 @@ cli_upload (int argc, gchar * argv[], int *optind)
 
   if (*optind == argc)
     {
-      error_print ("Remote path missing\n");
+      error_print ("Remote path missing");
       return EXIT_FAILURE;
     }
   else
@@ -564,36 +575,35 @@ cli_upload (int argc, gchar * argv[], int *optind)
 
   dst_path = cli_get_path (device_dst_path);
 
-  upload_path = fs_ops->get_upload_path (&backend, fs_ops, dst_path,
-					 src_path);
-
-  array = g_byte_array_new ();
   control.active = TRUE;
-  err = fs_ops->load (src_path, array, &control);
+  err = fs_ops->load (src_path, &idata, &control);
   if (err)
     {
-      goto cleanup;
+      return err;
     }
 
+  upload_path = fs_ops->get_upload_path (&backend, fs_ops, dst_path,
+					 src_path, &idata);
+
   CHECK_FS_OPS_FUNC (fs_ops->upload);
-  err = fs_ops->upload (&backend, upload_path, array, &control);
-  g_free (control.data);
+  err = fs_ops->upload (&backend, upload_path, &idata, &control);
+  idata_free (&idata);
 
-cleanup:
   g_free (upload_path);
-  g_byte_array_free (array, TRUE);
+
   return err;
 }
 
 static int
-cli_send (int argc, gchar * argv[], int *optind)
+cli_send (int argc, gchar *argv[], int *optind)
 {
   gint err;
   const gchar *device_dst_path, *src_file;
+  struct idata idata;
 
   if (*optind == argc)
     {
-      error_print ("Source file missing\n");
+      error_print ("Source file missing");
       return -EINVAL;
     }
   else
@@ -604,7 +614,7 @@ cli_send (int argc, gchar * argv[], int *optind)
 
   if (*optind == argc)
     {
-      error_print ("Remote device missing\n");
+      error_print ("Remote device missing");
       return -EINVAL;
     }
   else
@@ -625,30 +635,28 @@ cli_send (int argc, gchar * argv[], int *optind)
       return EXIT_FAILURE;
     }
 
-  sysex_transfer.active = TRUE;
-  sysex_transfer.timeout = BE_DUMP_TIMEOUT;
-  sysex_transfer.raw = g_byte_array_new ();
-
-  err = load_file (src_file, sysex_transfer.raw, NULL);
-
+  err = file_load (src_file, &idata, NULL);
   if (!err)
     {
+      sysex_transfer.active = TRUE;
+      sysex_transfer.timeout = BE_SYSEX_TIMEOUT_MS;
+      sysex_transfer.raw = idata.content;
       err = backend_tx_sysex (&backend, &sysex_transfer);
+      idata_free (&idata);
     }
 
-  free_msg (sysex_transfer.raw);
   return err;
 }
 
 static int
-cli_receive (int argc, gchar * argv[], int *optind)
+cli_receive (int argc, gchar *argv[], int *optind)
 {
   gint err;
   const gchar *device_src_path, *dst_file;
 
   if (*optind == argc)
     {
-      error_print ("Remote device missing\n");
+      error_print ("Remote device missing");
       return -EINVAL;
     }
   else
@@ -659,7 +667,7 @@ cli_receive (int argc, gchar * argv[], int *optind)
 
   if (*optind == argc)
     {
-      error_print ("Destination file missing\n");
+      error_print ("Destination file missing");
       return -EINVAL;
     }
   else
@@ -680,7 +688,7 @@ cli_receive (int argc, gchar * argv[], int *optind)
       return EXIT_FAILURE;
     }
 
-  sysex_transfer.timeout = BE_DUMP_TIMEOUT;
+  sysex_transfer.timeout = BE_SYSEX_TIMEOUT_MS;
   sysex_transfer.batch = TRUE;
 
   backend_rx_drain (&backend);
@@ -688,7 +696,9 @@ cli_receive (int argc, gchar * argv[], int *optind)
   err = backend_rx_sysex (&backend, &sysex_transfer);
   if (!err)
     {
-      err = save_file (dst_file, sysex_transfer.raw, NULL);
+      struct idata idata;
+      idata.content = sysex_transfer.raw;
+      err = file_save (dst_file, &idata, NULL);
     }
 
   free_msg (sysex_transfer.raw);
@@ -696,7 +706,7 @@ cli_receive (int argc, gchar * argv[], int *optind)
 }
 
 static gint
-set_conn_fs_op_from_command (const gchar * cmd)
+set_conn_fs_op_from_command (const gchar *cmd)
 {
   gchar *aux;
 
@@ -731,9 +741,7 @@ set_conn_fs_op_from_command (const gchar * cmd)
 static void
 cli_end (int sig)
 {
-  g_mutex_lock (&control.mutex);
-  control.active = FALSE;
-  g_mutex_unlock (&control.mutex);
+  job_control_set_active_lock (&control, FALSE);
 
   g_mutex_lock (&sysex_transfer.mutex);
   sysex_transfer.active = FALSE;
@@ -742,7 +750,7 @@ cli_end (int sig)
 #endif
 
 int
-main (int argc, gchar * argv[])
+main (int argc, gchar *argv[])
 {
   gint c;
   gint err;
@@ -795,6 +803,10 @@ main (int argc, gchar * argv[])
       exit (EXIT_FAILURE);
     }
 
+  regconn_register ();
+  regpref_register ();
+  preferences_load ();
+
   if (!strcmp (command, "ld") || !strcmp (command, "list-devices"))
     {
       err = cli_ld ();
@@ -828,7 +840,7 @@ main (int argc, gchar * argv[])
 	}
 
       debug_print (1,
-		   "Connector: \"%s\"; filesystem: \"%s\"; operation: \"%s\"\n",
+		   "Connector: \"%s\"; filesystem: \"%s\"; operation: \"%s\"",
 		   connector, fs, op);
 
       if (!strcmp (op, "ls") || !strcmp (op, "list"))
@@ -874,7 +886,7 @@ main (int argc, gchar * argv[])
 	}
       else
 	{
-	  error_print ("Command '%s' not recognized\n", command);
+	  error_print ("Command '%s' not recognized", command);
 	  err = EXIT_FAILURE;
 	}
 
@@ -891,9 +903,12 @@ main (int argc, gchar * argv[])
 end:
   if (err && err != EXIT_FAILURE)
     {
-      error_print ("Error: %s\n", g_strerror (-err));
+      error_print ("Error: %s", g_strerror (-err));
     }
 
+  regconn_unregister ();
+  regpref_unregister ();
+
   usleep (BE_REST_TIME_US * 2);
   return err ? EXIT_FAILURE : EXIT_SUCCESS;
 }
