@@ -19,9 +19,31 @@
  */
 
 #include <gtk/gtk.h>
-#include "utils.h"
+#include "connector.h"
 
-extern struct sysex_transfer sysex_transfer;
+#ifndef PROGRESS_H
+#define PROGRESS_H
+
+enum progress_type
+{
+  PROGRESS_TYPE_NONE,
+  PROGRESS_TYPE_PULSE,
+  PROGRESS_TYPE_UPDATE
+};
+
+struct progress
+{
+  struct sysex_transfer sysex_transfer;
+  GtkDialog *dialog;
+  GtkWidget *bar;
+  GtkWidget *label;
+  GtkWidget *cancel_button;
+  GThread *thread;
+  gint64 start;
+  enum progress_type type;
+};
+
+extern struct progress progress;
 
 void progress_stop_thread ();
 
@@ -29,13 +51,14 @@ void progress_dialog_close (gpointer data);
 
 void progress_set_fraction (gdouble fraction);
 
-gboolean progress_pulse (gpointer data);
+gboolean progress_is_active ();
 
-gboolean progress_update (gpointer data);
-
-gpointer progress_run (GThreadFunc f, gpointer user_data, const gchar * name,
-		       const gchar * text, gint * res);
+gpointer progress_run (GThreadFunc f, enum progress_type type,
+		       gpointer user_data, const gchar * name,
+		       const gchar * text, gboolean cancellable, gint * res);
 
 void progress_init (GtkBuilder * builder);
 
 void progress_response (gint response);
+
+#endif
