@@ -22,18 +22,24 @@
 #include <math.h>
 #include <errno.h>
 #include "sample.h"
+#include "connectors/microfreak_sample.h"
 
-#define LOAD_BUFFER_LEN (32 * 1024)
+#define LOAD_BUFFER_LEN (32 * KI)
 
 #define JUNK_CHUNK_ID "JUNK"
 #define SMPL_CHUNK_ID "smpl"
 
+#define HEADERS_SPACE 4096	//Gross estimation for the sample (WAV) headers
+
 static const gchar *ELEKTROID_AUDIO_LOCAL_EXTS[] =
+  { "wav", "ogg", "aiff", "flac", MICROFREAK_PWAVETABLE_EXT,
+  MICROFREAK_ZWAVETABLE_EXT, MICROFREAK_PSAMPLE_EXT,
+  MICROFREAK_ZSAMPLE_EXT,
 #if !defined(__linux__) || HAVE_SNDFILE_MP3
-{ "wav", "ogg", "aiff", "flac", "mp3", NULL };
-#else
-{ "wav", "ogg", "aiff", "flac", NULL };
+  "mp3",
 #endif
+  NULL
+};
 
 struct smpl_chunk_data
 {
@@ -201,8 +207,9 @@ static SF_VIRTUAL_IO FILE_IO = {
 };
 
 static gint
-sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
-			    struct g_byte_array_io_data *wave, guint32 format)
+sample_write_audio_file_data (struct idata *idata,
+			      struct g_byte_array_io_data *wave,
+			      struct job_control *control, guint32 format)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
@@ -210,15 +217,13 @@ sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
   struct SF_CHUNK_INFO junk_chunk_info;
   struct SF_CHUNK_INFO smpl_chunk_info;
   struct smpl_chunk_data smpl_chunk_data;
-  struct sample_info *sample_info = control->data;
-
-  g_byte_array_set_size (wave->array, sample->len + 4096);	//We need space for the headers.
-  wave->array->len = 0;
+  GByteArray *sample = idata->content;
+  struct sample_info *sample_info = idata->info;
 
   frames = sample->len / SAMPLE_INFO_FRAME_SIZE (sample_info);
-  debug_print (1, "Frames: %" PRIu64 "; sample rate: %d; channels: %d\n",
+  debug_print (1, "Frames: %" PRIu64 "; sample rate: %d; channels: %d",
 	       frames, sample_info->rate, sample_info->channels);
-  debug_print (1, "Loop start at %d; loop end at %d\n",
+  debug_print (1, "Loop start at %d; loop end at %d",
 	       sample_info->loop_start, sample_info->loop_end);
 
   memset (&sf_info, 0, sizeof (sf_info));
@@ -229,7 +234,7 @@ sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
   sndfile = sf_open_virtual (&G_BYTE_ARRAY_IO, SFM_WRITE, &sf_info, wave);
   if (!sndfile)
     {
-      error_print ("%s\n", sf_strerror (sndfile));
+      error_print ("%s", sf_strerror (sndfile));
       return -1;
     }
 
@@ -239,22 +244,22 @@ sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
   junk_chunk_info.data = (void *) JUNK_CHUNK_DATA;
   if (sf_set_chunk (sndfile, &junk_chunk_info) != SF_ERR_NO_ERROR)
     {
-      error_print ("%s\n", sf_strerror (sndfile));
+      error_print ("%s", sf_strerror (sndfile));
     }
 
   smpl_chunk_data.manufacturer = 0;
   smpl_chunk_data.product = 0;
   smpl_chunk_data.sample_period = 1e9 / sample_info->rate;
-  smpl_chunk_data.midi_unity_note = sample_info->midi_note;
+  smpl_chunk_data.midi_unity_note = GUINT32_TO_LE (sample_info->midi_note);
   smpl_chunk_data.midi_pitch_fraction = 0;
   smpl_chunk_data.smpte_format = 0;
   smpl_chunk_data.smpte_offset = 0;
   smpl_chunk_data.num_sampler_loops = 1;
   smpl_chunk_data.sampler_data = 0;
   smpl_chunk_data.sample_loop.cue_point_id = 0;
-  smpl_chunk_data.sample_loop.type = sample_info->loop_type;
-  smpl_chunk_data.sample_loop.start = sample_info->loop_start;
-  smpl_chunk_data.sample_loop.end = sample_info->loop_end;
+  smpl_chunk_data.sample_loop.type = GUINT32_TO_LE (sample_info->loop_type);
+  smpl_chunk_data.sample_loop.start = GUINT32_TO_LE (sample_info->loop_start);
+  smpl_chunk_data.sample_loop.end = GUINT32_TO_LE (sample_info->loop_end);
   smpl_chunk_data.sample_loop.fraction = 0;
   smpl_chunk_data.sample_loop.play_count = 0;
 
@@ -264,7 +269,7 @@ sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
   smpl_chunk_info.data = &smpl_chunk_data;
   if (sf_set_chunk (sndfile, &smpl_chunk_info) != SF_ERR_NO_ERROR)
     {
-      error_print ("%s\n", sf_strerror (sndfile));
+      error_print ("%s", sf_strerror (sndfile));
     }
 
   if ((sample_info->format & SF_FORMAT_SUBMASK) == SF_FORMAT_PCM_16)
@@ -281,7 +286,7 @@ sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
     }
   else
     {
-      error_print ("Invalid sample format. Using short...\n");
+      error_print ("Invalid sample format. Using short...");
       total = sf_writef_short (sndfile, (gint16 *) sample->data, frames);
     }
 
@@ -290,7 +295,7 @@ sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
   if (total != frames)
     {
       error_print ("Unexpected frames while writing to file (%" PRIu64 " != %"
-		   PRIu64 ")\n", total, frames);
+		   PRIu64 ")", total, frames);
       return -1;
     }
 
@@ -298,38 +303,58 @@ sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
 }
 
 gint
-sample_get_audio_file_data_from_array (GByteArray * sample, GByteArray * wave,
-				       struct job_control *control,
-				       guint32 format)
+sample_get_memfile_from_sample (struct idata *sample, struct idata *memfile,
+				struct job_control *control, guint32 format)
 {
+  gint err;
+  GByteArray *content;
   struct g_byte_array_io_data data;
+  struct sample_info *sample_info = sample->info;
+  guint frame_size = FRAME_SIZE (sample_info->channels,
+				 format & SF_FORMAT_SUBMASK);
+
+  content = g_byte_array_sized_new (sample->content->len * frame_size +
+				    HEADERS_SPACE);
+  idata_init (memfile, content, sample->name ? strdup (sample->name) : NULL,
+	      NULL);
   data.pos = 0;
-  data.array = wave;
-  return sample_get_audio_file_data (sample, control, &data, format);
+  data.array = content;
+
+  err = sample_write_audio_file_data (sample, &data, control, format);
+  if (err)
+    {
+      idata_free (memfile);
+    }
+
+  return err;
 }
 
 gint
-sample_save_to_file (const gchar * path, GByteArray * sample,
+sample_save_to_file (const gchar *path, struct idata *sample,
 		     struct job_control *control, guint32 format)
 {
-  GByteArray *wave = g_byte_array_new ();
-  gint ret = sample_get_audio_file_data_from_array (sample, wave, control,
-						    format);
-  if (!ret)
+  gint err;
+  struct idata file;
+
+  err = sample_get_memfile_from_sample (sample, &file, control, format);
+  if (err)
     {
-      ret = save_file (path, wave, control);
+      return err;
     }
-  g_byte_array_free (wave, TRUE);
-  return ret;
+
+  err = file_save (path, &file, control);
+  idata_free (&file);
+
+  return err;
 }
 
 static void
-audio_multichannel_to_mono_short (gshort * input, gshort * output, gint size,
+audio_multichannel_to_mono_short (gshort *input, gshort *output, gint size,
 				  gint channels)
 {
   gint32 i, j, v;
 
-  debug_print (2, "Converting short values to mono...\n");
+  debug_print (2, "Converting short values to mono...");
 
   for (i = 0; i < size; i++)
     {
@@ -338,19 +363,19 @@ audio_multichannel_to_mono_short (gshort * input, gshort * output, gint size,
 	{
 	  v += input[i * channels + j];
 	}
-      v *= MULTICHANNEL_MIX_GAIN (channels);
+      v *= MONO_MIX_GAIN (channels);
       output[i] = v;
     }
 }
 
 static void
-audio_multichannel_to_mono_float (gfloat * input, gfloat * output, gint size,
+audio_multichannel_to_mono_float (gfloat *input, gfloat *output, gint size,
 				  gint channels)
 {
   gfloat v;
   gint i, j;
 
-  debug_print (2, "Converting float values to mono...\n");
+  debug_print (2, "Converting float values to mono...");
 
   for (i = 0; i < size; i++)
     {
@@ -359,19 +384,19 @@ audio_multichannel_to_mono_float (gfloat * input, gfloat * output, gint size,
 	{
 	  v += input[i * channels + j];
 	}
-      v *= MULTICHANNEL_MIX_GAIN (channels);
+      v *= MONO_MIX_GAIN (channels);
       output[i] = v;
     }
 }
 
 static void
-audio_multichannel_to_mono_int (gint32 * input, gint32 * output, gint size,
+audio_multichannel_to_mono_int (gint32 *input, gint32 *output, gint size,
 				gint channels)
 {
   gint32 v;
   gint i, j;
 
-  debug_print (2, "Converting int values to mono...\n");
+  debug_print (2, "Converting int values to mono...");
 
   for (i = 0; i < size; i++)
     {
@@ -380,15 +405,15 @@ audio_multichannel_to_mono_int (gint32 * input, gint32 * output, gint size,
 	{
 	  v += input[i * channels + j];
 	}
-      v *= MULTICHANNEL_MIX_GAIN (channels);
+      v *= MONO_MIX_GAIN (channels);
       output[i] = v;
     }
 }
 
 static void
-audio_mono_to_stereo_short (gshort * input, gshort * output, gint size)
+audio_mono_to_stereo_short (gshort *input, gshort *output, gint size)
 {
-  debug_print (2, "Converting short values to stereo...\n");
+  debug_print (2, "Converting short values to stereo...");
 
   for (gint i = 0; i < size; i++, input++)
     {
@@ -400,9 +425,9 @@ audio_mono_to_stereo_short (gshort * input, gshort * output, gint size)
 }
 
 static void
-audio_mono_to_stereo_float (gfloat * input, gfloat * output, gint size)
+audio_mono_to_stereo_float (gfloat *input, gfloat *output, gint size)
 {
-  debug_print (2, "Converting float values to stereo...\n");
+  debug_print (2, "Converting float values to stereo...");
 
   for (gint i = 0; i < size; i++, input++)
     {
@@ -414,9 +439,9 @@ audio_mono_to_stereo_float (gfloat * input, gfloat * output, gint size)
 }
 
 static void
-audio_mono_to_stereo_int (gint32 * input, gint32 * output, gint size)
+audio_mono_to_stereo_int (gint32 *input, gint32 *output, gint size)
 {
-  debug_print (2, "Converting int values to stereo...\n");
+  debug_print (2, "Converting int values to stereo...");
 
   for (gint i = 0; i < size; i++, input++)
     {
@@ -428,8 +453,8 @@ audio_mono_to_stereo_int (gint32 * input, gint32 * output, gint size)
 }
 
 static void
-sample_set_sample_info (struct sample_info *sample_info, SNDFILE * sndfile,
-			SF_INFO * sf_info)
+sample_set_sample_info (struct sample_info *sample_info, SNDFILE *sndfile,
+			SF_INFO *sf_info)
 {
   struct SF_CHUNK_INFO chunk_info;
   SF_CHUNK_ITERATOR *chunk_iter;
@@ -449,7 +474,7 @@ sample_set_sample_info (struct sample_info *sample_info, SNDFILE * sndfile,
     {
       chunk_info.datalen = sizeof (struct smpl_chunk_data);
       memset (&smpl_chunk_data, 0, chunk_info.datalen);
-      debug_print (2, "%s chunk found (%d B)\n", SMPL_CHUNK_ID,
+      debug_print (2, "%s chunk found (%d B)", SMPL_CHUNK_ID,
 		   chunk_info.datalen);
       chunk_info.data = &smpl_chunk_data;
       sf_get_chunk_data (chunk_iter, &chunk_info);
@@ -463,12 +488,12 @@ sample_set_sample_info (struct sample_info *sample_info, SNDFILE * sndfile,
 	GUINT32_FROM_LE (smpl_chunk_data.midi_unity_note);
       if (sample_info->loop_start >= sample_info->frames)
 	{
-	  debug_print (2, "Bad loop start\n");
+	  debug_print (2, "Bad loop start");
 	  disable_loop = TRUE;
 	}
       if (sample_info->loop_end >= sample_info->frames)
 	{
-	  debug_print (2, "Bad loop end\n");
+	  debug_print (2, "Bad loop end");
 	  disable_loop = TRUE;
 	}
 
@@ -489,18 +514,18 @@ sample_set_sample_info (struct sample_info *sample_info, SNDFILE * sndfile,
       sample_info->loop_type = 0;
     }
 
-  debug_print (2, "Loop start at %d, loop end at %d\n",
+  debug_print (2, "Loop start at %d, loop end at %d",
 	       sample_info->loop_start, sample_info->loop_end);
 }
 
-gint
-sample_load_sample_info (const gchar * path, struct sample_info *sample_info)
+static gint
+sample_load_sample_info_libsndfile (const gchar *path,
+				    struct sample_info *sample_info)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
   FILE *file;
-
-  memset (sample_info, 0, sizeof (struct sample_info));
+  gint err = 0;
 
   file = fopen (path, "rb");
   if (!file)
@@ -511,19 +536,94 @@ sample_load_sample_info (const gchar * path, struct sample_info *sample_info)
   sndfile = sf_open_virtual (&FILE_IO, SFM_READ, &sf_info, file);
   if (!sndfile)
     {
-      error_print ("Error while reading %s: %s\n", path,
-		   sf_strerror (sndfile));
-      return -1;
+      error_print ("Error while reading %s: %s", path, sf_strerror (sndfile));
+      err = -1;
+      goto end;
     }
 
   sample_set_sample_info (sample_info, sndfile, &sf_info);
 
+end:
   fclose (file);
-  return 0;
+  return err;
 }
 
-void
-sample_check_and_fix_loop_points (struct sample_info *sample_info)
+static gint
+sample_load_microfreak_sample (const gchar *path, struct idata *sample,
+			       struct job_control *control)
+{
+  gint err;
+  const gchar *ext = filename_get_ext (path);
+
+  if (!strcmp (MICROFREAK_PWAVETABLE_EXT, ext))
+    {
+      err = microfreak_pwavetable_load (path, sample, control);
+    }
+  else if (!strcmp (MICROFREAK_ZWAVETABLE_EXT, ext))
+    {
+      err = microfreak_zwavetable_load (path, sample, control);
+    }
+  else if (!strcmp (MICROFREAK_PSAMPLE_EXT, ext))
+    {
+      err = microfreak_psample_load (path, sample, control);
+    }
+  else if (!strcmp (MICROFREAK_ZSAMPLE_EXT, ext))
+    {
+      err = microfreak_zsample_load (path, sample, control);
+    }
+  else
+    {
+      err = -1;
+    }
+
+  return err;
+}
+
+static gboolean
+sample_microfreak_filename (const gchar *path)
+{
+  const gchar *ext = filename_get_ext (path);
+
+  return !strcmp (MICROFREAK_PWAVETABLE_EXT, ext) ||
+    !strcmp (MICROFREAK_ZWAVETABLE_EXT, ext) ||
+    !strcmp (MICROFREAK_PSAMPLE_EXT, ext) ||
+    !strcmp (MICROFREAK_ZSAMPLE_EXT, ext);
+}
+
+gint
+sample_load_sample_info (const gchar *path, struct sample_info *sample_info)
+{
+  gint err;
+
+  memset (sample_info, 0, sizeof (struct sample_info));
+
+  if (sample_microfreak_filename (path))
+    {
+      struct idata aux;
+      struct job_control control;
+
+      control.active = TRUE;
+      g_mutex_init (&control.mutex);
+
+      err = sample_load_microfreak_sample (path, &aux, &control);
+      if (err)
+	{
+	  return err;
+	}
+
+      memcpy (sample_info, aux.info, sizeof (struct sample_info));
+      idata_free (&aux);
+    }
+  else
+    {
+      err = sample_load_sample_info_libsndfile (path, sample_info);
+    }
+
+  return err;
+}
+
+static void
+sample_info_fix_frame_values (struct sample_info *sample_info)
 {
   if (sample_info->loop_start >= sample_info->frames)
     {
@@ -535,14 +635,13 @@ sample_check_and_fix_loop_points (struct sample_info *sample_info)
     }
 }
 
-// If control->data is NULL, then a new struct sample_info * is created and control->data points to it.
-// In case of failure, if control->data is NULL is freed.
-
 static gint
-sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
-		 struct job_control *control, GByteArray * sample,
-		 struct sample_info *sample_info_dst, sample_load_cb cb,
-		 gpointer cb_data)
+sample_load_libsndfile (void *data, SF_VIRTUAL_IO *sf_virtual_io,
+			struct job_control *control, struct idata *idata,
+			const struct sample_info *sample_info_req,
+			struct sample_info *sample_info_src,
+			sample_load_cb cb, gpointer cb_data,
+			const gchar *name)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
@@ -555,95 +654,72 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
   void *buffer_i;		//For gint16 or gint32
   gfloat *buffer_f;
   void *buffer_output;
-  gint err, resampled_buffer_len, frames_read;
-  gboolean active;
+  gint err = 0, resampled_buffer_len, frames;
+  gboolean active, rounding_fix;
   gdouble ratio;
-  struct sample_info *sample_info_src;
   guint bytes_per_sample, bytes_per_frame;
-  guint32 f, actual_frames = 0;
-
-  if (control)
-    {
-      g_mutex_lock (&control->mutex);
-    }
-  g_byte_array_set_size (sample, 0);
-  if (control)
-    {
-      g_mutex_unlock (&control->mutex);
-    }
+  guint32 read_frames, actual_frames = 0;
+  GByteArray *sample = NULL;
+  struct sample_info *sample_info;
 
   sf_info.format = 0;
   sndfile = sf_open_virtual (sf_virtual_io, SFM_READ, &sf_info, data);
   if (!sndfile)
     {
-      error_print ("%s\n", sf_strerror (sndfile));
+      error_print ("%s", sf_strerror (sndfile));
       return -1;
     }
 
-  sample_info_src = control->data;
-  if (!sample_info_src)
-    {
-      sample_info_src = g_malloc (sizeof (struct sample_info));
-    }
-
-  if (control)
-    {
-      g_mutex_lock (&control->mutex);
-    }
-
-  sample_info_dst->channels = sample_info_dst->channels ?
-    sample_info_dst->channels : sf_info.channels;
-  sample_info_dst->rate = sample_info_dst->rate ? sample_info_dst->rate :
-    sf_info.samplerate;
-  sample_info_dst->format =
-    sample_info_dst->format ? sample_info_dst->format : SF_FORMAT_PCM_16;
-
-  if (sample_info_dst->format != SF_FORMAT_PCM_16 &&
-      sample_info_dst->format != SF_FORMAT_PCM_32 &&
-      sample_info_dst->format != SF_FORMAT_FLOAT)
-    {
-      error_print ("Invalid sample format. Using short...\n");
-      sample_info_dst->format = SF_FORMAT_PCM_16;
-    }
-
-  bytes_per_frame = SAMPLE_INFO_FRAME_SIZE (sample_info_dst);
-  bytes_per_sample = SAMPLE_SIZE (sample_info_dst->format);
-
   sample_set_sample_info (sample_info_src, sndfile, &sf_info);
-  sample_info_dst->midi_note = sample_info_src->midi_note;
-  sample_info_dst->loop_type = sample_info_src->loop_type;
 
-  if (control)
+  sample_info = g_malloc (sizeof (struct sample_info));
+
+  sample_info->midi_note = sample_info_src->midi_note;
+  sample_info->loop_type = sample_info_src->loop_type;
+  sample_info->channels = sample_info_req->channels ?
+    sample_info_req->channels : sample_info_src->channels;
+  sample_info->rate = sample_info_req->rate ? sample_info_req->rate :
+    sample_info_src->rate;
+  sample_info->format = sample_info_req->format ? sample_info_req->format :
+    SF_FORMAT_PCM_16;
+  if (sample_info->format != SF_FORMAT_PCM_16 &&
+      sample_info->format != SF_FORMAT_PCM_32 &&
+      sample_info->format != SF_FORMAT_FLOAT)
     {
-      g_mutex_unlock (&control->mutex);
+      error_print ("Invalid sample format. Using short...");
+      sample_info->format = SF_FORMAT_PCM_16;
     }
 
+  bytes_per_frame = SAMPLE_INFO_FRAME_SIZE (sample_info);
+  bytes_per_sample = SAMPLE_SIZE (sample_info->format);
+
   //Set scale factor. See http://www.mega-nerd.com/libsndfile/api.html#note2
-  if ((sf_info.format & SF_FORMAT_FLOAT) == SF_FORMAT_FLOAT ||
-      (sf_info.format & SF_FORMAT_DOUBLE) == SF_FORMAT_DOUBLE)
+  if ((sample_info_src->format & SF_FORMAT_FLOAT) == SF_FORMAT_FLOAT ||
+      (sample_info_src->format & SF_FORMAT_DOUBLE) == SF_FORMAT_DOUBLE)
     {
       debug_print (2,
-		   "Setting scale factor to ensure correct integer readings...\n");
+		   "Setting scale factor to ensure correct integer readings...");
       sf_command (sndfile, SFC_SET_SCALE_FLOAT_INT_READ, NULL, SF_TRUE);
     }
 
   buffer_input_multi = g_malloc (LOAD_BUFFER_LEN *
 				 FRAME_SIZE (sample_info_src->channels,
-					     sample_info_dst->format));
+					     sample_info->format));
   buffer_input_mono = g_malloc (LOAD_BUFFER_LEN * bytes_per_sample);
   buffer_input_stereo = g_malloc (LOAD_BUFFER_LEN * 2 * bytes_per_sample);
 
-  ratio = sample_info_dst->rate / (double) sample_info_src->rate;
+  ratio = sample_info->rate / (double) sample_info_src->rate;
   src_data.src_ratio = ratio;
 
   src_data.output_frames = ceil (LOAD_BUFFER_LEN * src_data.src_ratio);
-  resampled_buffer_len = src_data.output_frames * sample_info_dst->channels;
+  resampled_buffer_len = src_data.output_frames * sample_info->channels;
   buffer_i = g_malloc (resampled_buffer_len * bytes_per_sample);
   src_data.data_out = g_malloc (resampled_buffer_len * sizeof (gfloat));
-  if (sample_info_dst->format == SF_FORMAT_PCM_16 ||
-      sample_info_dst->format == SF_FORMAT_PCM_32)
+
+  if (sample_info->format == SF_FORMAT_PCM_16 ||
+      sample_info->format == SF_FORMAT_PCM_32)
     {
-      buffer_f = g_malloc (LOAD_BUFFER_LEN * sample_info_dst->channels *
+      buffer_f = g_malloc (LOAD_BUFFER_LEN * sample_info->channels *
 			   sizeof (gfloat));
       src_data.data_in = buffer_f;
       buffer_output = buffer_i;
@@ -654,128 +730,118 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
       buffer_output = src_data.data_out;
     }
 
-  src_state = src_new (SRC_SINC_BEST_QUALITY, sample_info_dst->channels,
-		       &err);
+  src_state = src_new (SRC_SINC_BEST_QUALITY, sample_info->channels, &err);
   if (err)
     {
+      error_print ("Error while creating the resampler: %s",
+		   src_strerror (err));
       goto cleanup;
     }
 
-  if (control)
-    {
-      g_mutex_lock (&control->mutex);
-    }
-  sample_info_dst->frames = floor (sample_info_src->frames * ratio);	//Lower bound estimation. The actual amount is updated later.
-  sample_info_dst->loop_start = round (sample_info_src->loop_start * ratio);
-  sample_info_dst->loop_end = round (sample_info_src->loop_end * ratio);
-  sample_check_and_fix_loop_points (sample_info_dst);
-  if (control)
-    {
-      g_mutex_unlock (&control->mutex);
-    }
-
-  debug_print (2, "Loading sample (%d frames)...\n", sample_info_src->frames);
-
+  active = TRUE;
   if (control)
     {
       g_mutex_lock (&control->mutex);
       active = control->active;
+    }
+  sample_info->frames = ceil (sample_info_src->frames * ratio);	//Upper bound estimation. The actual amount is updated later.
+  sample_info->loop_start = round (sample_info_src->loop_start * ratio);
+  sample_info->loop_end = round (sample_info_src->loop_end * ratio);
+  sample_info_fix_frame_values (sample_info);
+
+  sample = g_byte_array_sized_new (sample_info->frames * bytes_per_frame);
+  idata_init (idata, sample, strdup (name), sample_info);
+  if (control)
+    {
       g_mutex_unlock (&control->mutex);
     }
-  else
-    {
-      active = TRUE;
-    }
 
-  f = 0;
-  while (f < sample_info_src->frames && active)
-    {
-      debug_print (2, "Loading %d channels buffer...\n",
-		   sample_info_dst->channels);
+  debug_print (2, "Loading sample (%d frames)...", sample_info_src->frames);
 
-      if (sample_info_dst->format == SF_FORMAT_FLOAT)
+  read_frames = 0;
+  while (read_frames < sample_info_src->frames && active)
+    {
+      debug_print (2, "Loading %d channels buffer...", sample_info->channels);
+
+      if (sample_info->format == SF_FORMAT_FLOAT)
 	{
-	  frames_read = sf_readf_float (sndfile,
-					(gfloat *) buffer_input_multi,
-					LOAD_BUFFER_LEN);
+	  frames = sf_readf_float (sndfile, (gfloat *) buffer_input_multi,
+				   LOAD_BUFFER_LEN);
 	}
-      else if (sample_info_dst->format == SF_FORMAT_PCM_32)
+      else if (sample_info->format == SF_FORMAT_PCM_32)
 	{
-	  frames_read = sf_readf_int (sndfile,
-				      (gint32 *) buffer_input_multi,
-				      LOAD_BUFFER_LEN);
+	  frames = sf_readf_int (sndfile, (gint32 *) buffer_input_multi,
+				 LOAD_BUFFER_LEN);
 	}
       else
 	{
-	  frames_read = sf_readf_short (sndfile,
-					(gint16 *) buffer_input_multi,
-					LOAD_BUFFER_LEN);
+	  frames = sf_readf_short (sndfile, (gint16 *) buffer_input_multi,
+				   LOAD_BUFFER_LEN);
 	}
-      f += frames_read;
+      read_frames += frames;
 
-      if (sample_info_dst->channels == sample_info_src->channels)
+      if (sample_info->channels == sample_info_src->channels)
 	{
 	  buffer_input = buffer_input_multi;
 	}
       else
 	{
-	  if (sample_info_dst->format == SF_FORMAT_FLOAT)
+	  if (sample_info->format == SF_FORMAT_FLOAT)
 	    {
 	      audio_multichannel_to_mono_float (buffer_input_multi,
 						buffer_input_mono,
-						frames_read,
+						frames,
 						sample_info_src->channels);
 	    }
-	  else if (sample_info_dst->format == SF_FORMAT_PCM_32)
+	  else if (sample_info->format == SF_FORMAT_PCM_32)
 	    {
 	      audio_multichannel_to_mono_int (buffer_input_multi,
 					      buffer_input_mono,
-					      frames_read,
+					      frames,
 					      sample_info_src->channels);
 	    }
 	  else
 	    {
 	      audio_multichannel_to_mono_short (buffer_input_multi,
 						buffer_input_mono,
-						frames_read,
+						frames,
 						sample_info_src->channels);
 	    }
-	  if (sample_info_dst->channels == 1)
+
+	  if (sample_info->channels == 1)
 	    {
 	      buffer_input = buffer_input_mono;
 	    }
 	  else
 	    {
-	      if (sample_info_dst->format == SF_FORMAT_FLOAT)
+	      if (sample_info->format == SF_FORMAT_FLOAT)
 		{
 		  audio_mono_to_stereo_float (buffer_input_mono,
-					      buffer_input_stereo,
-					      frames_read);
+					      buffer_input_stereo, frames);
 		}
-	      else if (sample_info_dst->format == SF_FORMAT_PCM_32)
+	      else if (sample_info->format == SF_FORMAT_PCM_32)
 		{
 		  audio_mono_to_stereo_int (buffer_input_mono,
-					    buffer_input_stereo, frames_read);
+					    buffer_input_stereo, frames);
 		}
 	      else
 		{
 		  audio_mono_to_stereo_short (buffer_input_mono,
-					      buffer_input_stereo,
-					      frames_read);
+					      buffer_input_stereo, frames);
 		}
 	      buffer_input = buffer_input_stereo;
 	    }
 	}
 
-      if (sample_info_dst->rate == sample_info_src->rate)
+      if (sample_info->rate == sample_info_src->rate)
 	{
 	  if (control)
 	    {
 	      g_mutex_lock (&control->mutex);
 	    }
 	  g_byte_array_append (sample, (guint8 *) buffer_input,
-			       frames_read * bytes_per_frame);
-	  actual_frames += frames_read;
+			       frames * bytes_per_frame);
+	  actual_frames += frames;
 	  if (control)
 	    {
 	      g_mutex_unlock (&control->mutex);
@@ -783,32 +849,31 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 	}
       else
 	{
-	  src_data.end_of_input = frames_read < LOAD_BUFFER_LEN ? SF_TRUE : 0;
-	  src_data.input_frames = frames_read;
+	  debug_print (2, "Resampling %d channels with ratio %f...",
+		       sample_info->channels, src_data.src_ratio);
 
-	  if (sample_info_dst->format == SF_FORMAT_FLOAT)
+	  src_data.end_of_input = frames < LOAD_BUFFER_LEN ? SF_TRUE : 0;
+	  src_data.input_frames = frames;
+
+	  if (sample_info->format == SF_FORMAT_FLOAT)
 	    {
 	      src_data.data_in = buffer_input;
 	    }
-	  else if (sample_info_dst->format == SF_FORMAT_PCM_32)
+	  else if (sample_info->format == SF_FORMAT_PCM_32)
 	    {
-	      src_int_to_float_array (buffer_input, buffer_f, frames_read *
-				      sample_info_dst->channels);
+	      src_int_to_float_array (buffer_input, buffer_f, frames *
+				      sample_info->channels);
 	    }
 	  else
 	    {
-	      src_short_to_float_array (buffer_input, buffer_f, frames_read *
-					sample_info_dst->channels);
+	      src_short_to_float_array (buffer_input, buffer_f, frames *
+					sample_info->channels);
 	    }
 
-	  debug_print (2, "Resampling %d channels with ratio %f...\n",
-		       sample_info_dst->channels, src_data.src_ratio);
 	  err = src_process (src_state, &src_data);
 	  if (err)
 	    {
-	      g_byte_array_set_size (sample, 0);
-	      error_print ("Error while resampling: %s\n",
-			   src_strerror (err));
+	      error_print ("Error while resampling: %s", src_strerror (err));
 	      break;
 	    }
 
@@ -817,17 +882,17 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 	      g_mutex_lock (&control->mutex);
 	    }
 
-	  if (sample_info_dst->format == SF_FORMAT_PCM_32)
+	  if (sample_info->format == SF_FORMAT_PCM_32)
 	    {
 	      src_float_to_int_array (src_data.data_out, buffer_i,
 				      src_data.output_frames_gen *
-				      sample_info_dst->channels);
+				      sample_info->channels);
 	    }
-	  if (sample_info_dst->format == SF_FORMAT_PCM_16)
+	  if (sample_info->format == SF_FORMAT_PCM_16)
 	    {
 	      src_float_to_short_array (src_data.data_out, buffer_i,
 					src_data.output_frames_gen *
-					sample_info_dst->channels);
+					sample_info->channels);
 	    }
 	  g_byte_array_append (sample, (guint8 *) buffer_output,
 			       src_data.output_frames_gen * bytes_per_frame);
@@ -840,32 +905,20 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 
       if (control)
 	{
-	  g_mutex_lock (&control->mutex);
-	  cb (control, f * 1.0 / sample_info_src->frames, cb_data);
-	  active = control->active;
-	  g_mutex_unlock (&control->mutex);
+	  cb (control, read_frames * 1.0 / sample_info_src->frames, cb_data);
+	  active = job_control_get_active_lock (control);
 	}
     }
 
   src_delete (src_state);
 
-  if (control)
-    {
-      g_mutex_lock (&control->mutex);
-      if (!control->active)
-	{
-	  g_byte_array_set_size (sample, 0);
-	}
-      g_mutex_unlock (&control->mutex);
-    }
-
 cleanup:
   g_free (buffer_input_multi);
   g_free (buffer_input_mono);
   g_free (buffer_input_stereo);
   g_free (buffer_i);
-  if (sample_info_dst->format == SF_FORMAT_PCM_16 ||
-      sample_info_dst->format == SF_FORMAT_PCM_32)
+  if (sample_info->format == SF_FORMAT_PCM_16 ||
+      sample_info->format == SF_FORMAT_PCM_32)
     {
       g_free (buffer_f);
     }
@@ -873,83 +926,155 @@ cleanup:
 
   sf_close (sndfile);
 
-  if (sample->len)
+  if (!sample)
     {
-      if (!control->data)
-	{
-	  control->data = sample_info_src;
-	}
-      if (control)
-	{
-	  g_mutex_lock (&control->mutex);
-	}
-      // This removes the additional samples added by the resampler due to rounding.
-      sample_info_dst->frames = actual_frames;
-      sample_check_and_fix_loop_points (sample_info_dst);
-      g_byte_array_set_size (sample,
-			     sample_info_dst->frames * bytes_per_frame);
-      if (control)
-	{
-	  cb (control, 1.0, cb_data);
-	  g_mutex_unlock (&control->mutex);
-	}
-      return 0;
+      g_free (sample_info);
+      return err;
     }
-  else
+
+  if (control)
     {
-      if (!control->data)
-	{
-	  g_free (sample_info_src);
-	}
+      g_mutex_lock (&control->mutex);
+    }
+  if (!active || !sample_info->frames || err)
+    {
+      idata_free (idata);
+      cb (control, 0, cb_data);
+      g_mutex_unlock (&control->mutex);
       return -1;
     }
+  // This removes the additional samples added by the estimation above.
+  if (sample_info->frames > actual_frames)
+    {
+      rounding_fix = TRUE;
+      sample_info->frames = actual_frames;
+      sample_info_fix_frame_values (sample_info);
+      sample->len = sample_info->frames * bytes_per_frame;
+    }
+  if (control)
+    {
+      //It there was a rounding fix in the previous lines, the call is needed to detect the end of the loading process.
+      if (rounding_fix)
+	{
+	  cb (control, 1.0, cb_data);
+	}
+      g_mutex_unlock (&control->mutex);
+    }
+
+  return 0;
 }
 
 gint
-sample_load_from_array (GByteArray * wave, GByteArray * sample,
-			struct job_control *control,
-			struct sample_info *sample_info_dst)
+sample_load_from_memfile (struct idata *memfile, struct idata *sample,
+			  struct job_control *control,
+			  const struct sample_info *sample_info_req,
+			  struct sample_info *sample_info_src)
 {
   struct g_byte_array_io_data data;
   data.pos = 0;
-  data.array = wave;
-  return sample_load_raw (&data, &G_BYTE_ARRAY_IO, control, sample,
-			  sample_info_dst, set_job_control_progress_no_sync,
-			  NULL);
+  data.array = memfile->content;
+  return sample_load_libsndfile (&data, &G_BYTE_ARRAY_IO, control, sample,
+				 sample_info_req, sample_info_src,
+				 job_control_set_sample_progress_no_sync,
+				 NULL, memfile->name);
 }
 
+// Reloads the input into the output following all the requirements.
+
 gint
-sample_load_from_file_with_cb (const gchar * path, GByteArray * sample,
-			       struct job_control *control,
-			       struct sample_info *sample_info_dst,
-			       sample_load_cb cb, gpointer cb_data)
+sample_reload (struct idata *input, struct idata *output,
+	       struct job_control *control,
+	       const struct sample_info *sample_info_req, sample_load_cb cb,
+	       gpointer cb_data)
 {
-  FILE *file = fopen (path, "rb");
-  if (!file)
+  gint err;
+  struct idata aux;
+  struct sample_info sample_info_src;
+  struct g_byte_array_io_data data;
+
+  err = sample_get_memfile_from_sample (input, &aux, NULL,
+					SF_FORMAT_WAV | SF_FORMAT_PCM_16);
+  if (err)
     {
-      return -errno;
+      return err;
     }
-  gint err = sample_load_raw (file, &FILE_IO, control, sample,
-			      sample_info_dst, cb, cb_data);
-  fclose (file);
+
+  data.pos = 0;
+  data.array = aux.content;
+  err = sample_load_libsndfile (&data, &G_BYTE_ARRAY_IO, control, output,
+				sample_info_req, &sample_info_src, cb,
+				cb_data, input->name);
+  idata_free (&aux);
+
   return err;
 }
 
 gint
-sample_load_from_file (const gchar * path, GByteArray * sample,
-		       struct job_control *control,
-		       struct sample_info *sample_info_dst)
+sample_load_from_file_full (const gchar *path, struct idata *sample,
+			    struct job_control *control,
+			    const struct sample_info *sample_info_req,
+			    struct sample_info *sample_info_src,
+			    sample_load_cb cb, gpointer cb_data)
 {
-  return sample_load_from_file_with_cb (path, sample, control,
-					sample_info_dst,
-					set_job_control_progress_no_sync,
-					NULL);
+  gint err;
+  if (sample_microfreak_filename (path))
+    {
+      struct idata aux;
+
+      err = sample_load_microfreak_sample (path, &aux, control);
+      if (err)
+	{
+	  return err;
+	}
+
+      memcpy (sample_info_src, aux.info, sizeof (struct sample_info));
+      err = sample_reload (&aux, sample, control, sample_info_req, cb,
+			   cb_data);
+      idata_free (&aux);
+    }
+  else
+    {
+      gchar *name;
+      FILE *file = fopen (path, "rb");
+      if (!file)
+	{
+	  return -errno;
+	}
+      name = g_path_get_basename (path);
+      filename_remove_ext (name);
+      err = sample_load_libsndfile (file, &FILE_IO, control, sample,
+				    sample_info_req, sample_info_src, cb,
+				    cb_data, name);
+      g_free (name);
+      fclose (file);
+    }
+
+  return err;
 }
 
-const gchar **
+gint
+sample_load_from_file (const gchar *path, struct idata *sample,
+		       struct job_control *control,
+		       const struct sample_info *sample_info_req,
+		       struct sample_info *sample_info_src)
+{
+  return sample_load_from_file_full (path, sample, control,
+				     sample_info_req, sample_info_src,
+				     job_control_set_sample_progress_no_sync,
+				     NULL);
+}
+
+GSList *
 sample_get_sample_extensions ()
 {
-  return ELEKTROID_AUDIO_LOCAL_EXTS;
+  GSList *exts = NULL;
+  const gchar **e = ELEKTROID_AUDIO_LOCAL_EXTS;
+  while (*e)
+    {
+      exts = g_slist_append (exts, strdup (*e));
+      e++;
+    }
+  return exts;
 }
 
 const gchar *
