@@ -28,8 +28,6 @@
 #define DEBUG_SHORT_HEX_LEN 64
 #define DEBUG_FULL_HEX_THRES 5
 
-#define KIB 1024
-
 gint debug_level;
 
 static guint
@@ -50,7 +48,7 @@ get_max_message_length (guint msg_len)
 }
 
 gchar *
-debug_get_hex_data (gint level, guint8 * data, guint len)
+debug_get_hex_data (gint level, guint8 *data, guint len)
 {
   gint i;
   guint8 *b;
@@ -111,13 +109,13 @@ debug_get_hex_data (gint level, guint8 * data, guint len)
 }
 
 gchar *
-debug_get_hex_msg (const GByteArray * msg)
+debug_get_hex_msg (const GByteArray *msg)
 {
   return debug_get_hex_data (debug_level, msg->data, msg->len);
 }
 
 void
-remove_ext (char *name)
+filename_remove_ext (char *name)
 {
   gint namelen = strlen (name);
   gchar *dot = &name[namelen - 1];
@@ -136,41 +134,46 @@ remove_ext (char *name)
 }
 
 const gchar *
-get_ext (const gchar * name)
+filename_get_ext (const gchar *name)
 {
-  int namelen = strlen (name) - 1;
-  int i = namelen;
-  const gchar *ext = &name[namelen];
+  int namelen = strlen (name);
+  const gchar *ext = &name[namelen], *p = name;
 
-  while (i > 0 && *(ext - 1) != '.')
+  for (guint i = 0; i < namelen; i++, p++)
     {
-      ext--;
-      i--;
+      if (*p == '.')
+	{
+	  i++;
+	  p++;
+	  ext = p;
+	}
     }
 
-  if (i == 0 && name[0] != '.')
+  return ext;
+}
+
+//The returned value is owned by the caller.
+//As this is used from the code, rel_dir uses '/' always and needs to be converted.
+
+gchar *
+get_user_dir (const char *rel_dir)
+{
+  const gchar *home = g_get_home_dir ();
+  if (rel_dir)
     {
-      return NULL;
+      gchar *rel_dir_conv = path_translate (PATH_SYSTEM, rel_dir);
+      gchar *dir = path_chain (PATH_SYSTEM, home, rel_dir_conv);
+      g_free (rel_dir_conv);
+      return dir;
     }
   else
     {
-      return ext;
+      return strdup (home);
     }
 }
 
-gchar *
-get_user_dir (const char *rel_conf_path)
-{
-  const gchar *home = getenv ("HOME");
-  gchar *input = rel_conf_path ? g_strconcat (home, rel_conf_path, NULL) :
-    strdup (home);
-  gchar *output = path_translate (PATH_SYSTEM, input);
-  g_free (input);
-  return output;
-}
-
 char *
-get_system_startup_path (const gchar * local_dir)
+get_system_startup_path (const gchar *local_dir)
 {
   DIR *dir;
   gchar *startup_path = NULL;
@@ -184,7 +187,7 @@ get_system_startup_path (const gchar * local_dir)
 	}
       else
 	{
-	  error_print ("Unable to open dir '%s'\n", local_dir);
+	  error_print ("Unable to open dir '%s'", local_dir);
 	}
       closedir (dir);
     }
@@ -194,7 +197,7 @@ get_system_startup_path (const gchar * local_dir)
       startup_path = get_user_dir (NULL);
     }
 
-  debug_print (1, "Using '%s' as local dir...\n", startup_path);
+  debug_print (1, "Using '%s' as local dir...", startup_path);
 
   return startup_path;
 }
@@ -205,89 +208,58 @@ free_msg (gpointer msg)
   g_byte_array_free ((GByteArray *) msg, TRUE);
 }
 
-gchar *
-get_filename (guint32 fs_options, struct item *item)
-{
-  if (fs_options & FS_OPTION_ID_AS_FILENAME && item->type == ELEKTROID_FILE)
-    {
-      gchar *dir = g_path_get_dirname (item->name);
-      gboolean no_dir = strcmp (dir, ".") == 0;
-      gchar *id = g_malloc (LABEL_MAX);
-      if (no_dir)
-	{
-	  snprintf (id, LABEL_MAX, "%d", item->id);
-	}
-      else
-	{
-	  // If FS_OPTION_ID_AS_FILENAME is used, then the path separator is always '/'.
-	  snprintf (id, LABEL_MAX, "%s/%d", dir, item->id);
-	}
-      g_free (dir);
-      return id;
-    }
-  return strdup (item->name);
-}
-
 gint
-next_item_iterator (struct item_iterator *iter)
+file_load (const char *path, struct idata *idata, struct job_control *control)
 {
-  return iter->next (iter);
-}
-
-void
-free_item_iterator (struct item_iterator *iter)
-{
-  if (iter->free)
-    {
-      iter->free (iter->data);
-    }
-}
-
-gint
-load_file (const char *path, GByteArray * array, struct job_control *control)
-{
-  FILE *file;
+  FILE *f;
   size_t size;
   gint res;
+  GByteArray *array;
 
-  file = fopen (path, "rb");
+  f = fopen (path, "rb");
 
-  if (!file)
+  if (!f)
     {
       return -errno;
     }
 
   res = 0;
 
-  if (fseek (file, 0, SEEK_END))
+  if (fseek (f, 0, SEEK_END))
     {
-      error_print ("Unexpected value\n");
+      error_print ("Unexpected value");
       res = -errno;
       goto end;
     }
 
-  size = ftell (file);
-  rewind (file);
+  size = ftell (f);
+  rewind (f);
 
-  g_byte_array_set_size (array, size);
+  array = g_byte_array_sized_new (size);
+  array->len = size;
 
-  if (fread (array->data, 1, size, file) == size)
+  if (fread (array->data, 1, size, f) == size)
     {
-      debug_print (1, "%zu B read\n", size);
+      gchar *name = g_path_get_basename (path);
+      filename_remove_ext (name);
+      idata_init (idata, array, strdup (name), NULL);
+      g_free (name);
+      debug_print (1, "%zu B read", size);
     }
   else
     {
-      error_print ("Error while reading from file %s\n", path);
+      error_print ("Error while reading from file %s", path);
+      g_byte_array_free (array, TRUE);
       res = -errno;
     }
 
 end:
-  fclose (file);
+  fclose (f);
   return res;
 }
 
 gint
-save_file_char (const gchar * path, const guint8 * data, ssize_t len)
+file_save_data (const gchar *path, const guint8 *data, ssize_t len)
 {
   gint res;
   size_t bytes;
@@ -300,17 +272,17 @@ save_file_char (const gchar * path, const guint8 * data, ssize_t len)
       return -errno;
     }
 
-  debug_print (1, "Saving file %s...\n", path);
+  debug_print (1, "Saving file %s...", path);
 
   res = 0;
   bytes = fwrite (data, 1, len, file);
   if (bytes == len)
     {
-      debug_print (1, "%zu B written\n", bytes);
+      debug_print (1, "%zu B written", bytes);
     }
   else
     {
-      error_print ("Error while writing to file %s\n", path);
+      error_print ("Error while writing to file %s", path);
       res = -EIO;
     }
 
@@ -320,10 +292,10 @@ save_file_char (const gchar * path, const guint8 * data, ssize_t len)
 }
 
 gint
-save_file (const gchar * path, GByteArray * array,
+file_save (const gchar *path, struct idata *idata,
 	   struct job_control *control)
 {
-  return save_file_char (path, array->data, array->len);
+  return file_save_data (path, idata->content->data, idata->content->len);
 }
 
 gchar *
@@ -336,41 +308,65 @@ get_human_size (gint64 size, gboolean with_space)
     {
       *label = 0;
     }
-  else if (size < KIB)
+  else if (size < KI)
     {
       snprintf (label, LABEL_MAX, "%" PRId64 "%sB", size, space);
     }
-  else if (size < KIB * KIB)
+  else if (size < MI)
     {
-      snprintf (label, LABEL_MAX, "%.4g%sKiB", size / (double) KIB, space);
+      snprintf (label, LABEL_MAX, "%.4g%sKiB", size / (double) KI, space);
     }
-  else if (size < KIB * KIB * KIB)
+  else if (size < GI)
     {
-      snprintf (label, LABEL_MAX, "%.4g%sMiB", size / (double) (KIB * KIB),
-		space);
+      snprintf (label, LABEL_MAX, "%.4g%sMiB", size / (double) MI, space);
     }
   else
     {
-      snprintf (label, LABEL_MAX, "%.4g%sGiB",
-		size / (double) (KIB * KIB * KIB), space);
+      snprintf (label, LABEL_MAX, "%.4g%sGiB", size / (double) GI, space);
     }
 
   return label;
 }
 
 static inline void
-set_job_control_progress_value (struct job_control *control, gdouble p)
+job_control_set_progress_value (struct job_control *control, gdouble p)
 {
-  control->progress =
-    (control->part / (double) control->parts) + (p / (double) control->parts);
+  if (control->parts)
+    {
+      if (control->part == control->parts)
+	{
+	  control->progress = 1.0;
+	}
+      else
+	{
+	  control->progress =
+	    (control->part / (double) control->parts) +
+	    (p / (double) control->parts);
+	}
+    }
+  else
+    {
+      control->progress = 0.0;
+    }
 }
 
 void
-set_job_control_progress_with_cb (struct job_control *control, gdouble p,
-				  gpointer data)
+job_control_set_sample_progress_no_sync (struct job_control *control,
+					 gdouble p, gpointer data)
+{
+  job_control_set_progress_value (control, p);
+
+  if (control->callback)
+    {
+      control->callback (control);
+    }
+}
+
+void
+job_control_set_progress (struct job_control *control, gdouble p)
 {
   g_mutex_lock (&control->mutex);
-  set_job_control_progress_value (control, p);
+  job_control_set_progress_value (control, p);
   g_mutex_unlock (&control->mutex);
 
   if (control->callback)
@@ -379,65 +375,35 @@ set_job_control_progress_with_cb (struct job_control *control, gdouble p,
     }
 }
 
-void
-set_job_control_progress (struct job_control *control, gdouble p)
-{
-  set_job_control_progress_with_cb (control, p, NULL);
-}
-
-void
-set_job_control_progress_no_sync (struct job_control *control, gdouble p,
-				  gpointer data)
-{
-  set_job_control_progress_value (control, p);
-
-  if (control->callback)
-    {
-      control->callback (control);
-    }
-}
-
 gboolean
-file_matches_extensions (const gchar * name, const gchar ** extensions)
+file_matches_extensions (const gchar *name, const GSList *extensions)
 {
   const gchar *extension;
-  const gchar **e = extensions;
+  const GSList *e = extensions;
 
   if (!e)
     {
       return TRUE;
     }
 
-  extension = get_ext (name);
-  if (!extension)
+  extension = filename_get_ext (name);
+  if (!*extension)
     {
       return FALSE;
     }
 
-  while (*e)
+  while (e)
     {
-      if (!strcasecmp (extension, *e))
+      if (!strcasecmp (extension, e->data))
 	{
 	  return TRUE;
 	}
-      e++;
+      e = e->next;
     }
 
   return FALSE;
 }
 
-gboolean
-iter_is_dir_or_matches_extensions (struct item_iterator *iter,
-				   const gchar ** extensions)
-{
-  if (iter->item.type == ELEKTROID_DIR)
-    {
-      return TRUE;
-    }
-
-  return file_matches_extensions (iter->item.name, extensions);
-}
-
 static inline const gchar *
 path_get_separator (enum path_type type)
 {
@@ -458,7 +424,7 @@ path_get_separator (enum path_type type)
 }
 
 gchar *
-path_chain (enum path_type type, const gchar * parent, const gchar * child)
+path_chain (enum path_type type, const gchar *parent, const gchar *child)
 {
   const gchar *sep = path_get_separator (type);
   return g_build_path (sep, parent, child, NULL);
@@ -467,7 +433,7 @@ path_chain (enum path_type type, const gchar * parent, const gchar * child)
 //Translate from internal path to system path.
 
 gchar *
-path_translate (enum path_type type, const gchar * input)
+path_translate (enum path_type type, const gchar *input)
 {
   gchar *output, *o;
   const gchar *i;
@@ -505,7 +471,7 @@ path_translate (enum path_type type, const gchar * input)
 //under MSYS2.
 
 gchar *
-path_filename_from_uri (enum path_type type, gchar * uri)
+path_filename_from_uri (enum path_type type, gchar *uri)
 {
   if (type == PATH_SYSTEM)
     {
@@ -516,7 +482,7 @@ path_filename_from_uri (enum path_type type, gchar * uri)
 }
 
 gchar *
-path_filename_to_uri (enum path_type type, gchar * filename)
+path_filename_to_uri (enum path_type type, gchar *filename)
 {
   if (type == PATH_SYSTEM)
     {
@@ -527,3 +493,68 @@ path_filename_to_uri (enum path_type type, gchar * filename)
   g_free (uri);
   return escaped_uri;
 }
+
+void
+gslist_fill (GSList **list, ...)
+{
+  gpointer v;
+  va_list argptr;
+
+  *list = NULL;
+  va_start (argptr, list);
+  while ((v = va_arg (argptr, gpointer)))
+    {
+      *list = g_slist_append (*list, v);
+    }
+  va_end (argptr);
+}
+
+void
+idata_init (struct idata *idata, GByteArray *content, gchar *name, void *info)
+{
+  idata->content = content;
+  idata->name = name;
+  idata->info = info;
+}
+
+void
+idata_free (struct idata *idata)
+{
+  if (idata->content)
+    {
+      g_byte_array_free (idata->content, TRUE);
+      idata->content = NULL;
+    }
+  g_free (idata->name);
+  idata->name = NULL;
+  g_free (idata->info);
+  idata->info = NULL;
+}
+
+GByteArray *
+idata_steal (struct idata *idata)
+{
+  g_free (idata->name);
+  g_free (idata->info);
+  return idata->content;
+}
+
+gboolean
+job_control_get_active_lock (struct job_control *control)
+{
+  gboolean active;
+
+  g_mutex_lock (&control->mutex);
+  active = control->active;
+  g_mutex_unlock (&control->mutex);
+
+  return active;
+}
+
+void
+job_control_set_active_lock (struct job_control *control, gboolean active)
+{
+  g_mutex_lock (&control->mutex);
+  control->active = active;
+  g_mutex_unlock (&control->mutex);
+}
